@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -65,6 +65,6 @@ char const *BSckBioName(BSOCK_HANDLE hBSock);
 int BSckBufferInit(BSockLineBuffer *pBLB, int iSize = -1);
 void BSckBufferFree(BSockLineBuffer *pBLB);
 char *BSckBufferGet(BSOCK_HANDLE hBSock, BSockLineBuffer *pBLB, int iTimeout,
-		    int *piLineLength = NULL);
+		    int *piLnLength = NULL);
 
 #endif
