@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,16 +41,6 @@
 #define DNS_NS_CACHE_DIRCTORY   "ns"
 #define DNS_CACHE_LINE_MAX      2048
 
-static int CDNS_CleanupPath(char const *pszCachePath);
-static char *CDNS_GetCacheFilePath(char const *pszDomain, char const *pszSubDir,
-				   char *pszFilePath);
-static int CDNS_MxLoad(char const *pszDomain, char *&pszMXDomains);
-static int CDNS_MxSave(char const *pszDomain, char const *pszMXDomains, SYS_UINT32 TTL);
-static int CDNS_BuildStrMX(DNSAnswer *pAns, char *&pszMXDomains, SYS_UINT32 *pTTL);
-static int CDNS_QueryMX(char const *pszDomain, char *&pszMXDomains, SYS_UINT32 *pTTL);
-static int CDNS_QueryDirectMX(char const *pszDNSServer, char const *pszDomain,
-			      int iQuerySockType, char *&pszMXDomains, SYS_UINT32 *pTTL);
-
 static int iNumCacheDirs = DNS_HASH_NUM_DIRS;
 
 static int CDNS_CleanupPath(char const *pszCachePath)
@@ -247,7 +237,7 @@ static int CDNS_BuildStrMX(DNSAnswer *pAns, char *&pszMXDomains, SYS_UINT32 *pTT
 		if (StrDynSize(&DynMX) > 0)
 			StrDynAdd(&DynMX, ",");
 		if (StrDynPrint(&DynMX, "%u:%s", (unsigned int) pRec->U.MX.Pref,
-				pRec->szName) < 0) {
+				pRec->U.MX.szName) < 0) {
 			StrDynFree(&DynMX);
 			return ErrGetErrorCode();
 		}
