@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -67,25 +67,6 @@ struct FilterMacroSubstCtx {
 	FileSection FSect;
 };
 
-static char *FilGetLogExecStr(FilterLogInfo const *pFLI, int *piSize);
-static int FilLogExec(FilterMsgInfo const &FMI, char const * const *ppszExec,
-		      int iExecResult, int iExitCode, char const *pszType,
-		      char const *pszInfo);
-static int FilLoadMsgInfo(SPLF_HANDLE hFSpool, FilterMsgInfo & FMI);
-static void FilFreeMsgInfo(FilterMsgInfo & FMI);
-static int FilGetFilePath(char const *pszMode, char *pszFilePath, int iMaxPath);
-static int FilAddFilter(char **ppszFilters, int &iNumFilters, char const *pszFilterName);
-static int FilSelectFilters(char const *pszFilterFilePath, char const *pszMode,
-			    FilterMsgInfo const &FMI, char **ppszFilters, int iMaxFilters);
-static void FilFreeFilters(char **ppszFilters, int iNumFilters);
-static int FilGetFilterPath(char const *pszFileName, char *pszFilePath, int iMaxPath);
-static int FilPreExec(FilterMsgInfo const &FMI, FilterTokens *pToks, char **ppszPEError);
-static int FilApplyFilter(char const *pszFilterPath, SPLF_HANDLE hFSpool,
-			  QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage, FilterMsgInfo const &FMI,
-			  char const *pszType);
-static char *FilMacroLkupProc(void *pPrivate, char const *pszName, int iSize);
-static int FilFilterMacroSubstitutes(char **ppszCmdTokens, SPLF_HANDLE hFSpool,
-				     FilterMsgInfo const &FMI);
 
 static char *FilGetLogExecStr(FilterLogInfo const *pFLI, int *piSize)
 {
@@ -181,8 +162,8 @@ static int FilLoadMsgInfo(SPLF_HANDLE hFSpool, FilterMsgInfo & FMI)
 	char szDomain[MAX_ADDR_NAME] = "";
 
 	ZeroData(FMI);
-	if ((iFromDomains > 0) &&
-	    (USmtpSplitEmailAddr(ppszFrom[iFromDomains - 1], szUser, szDomain) == 0)) {
+	if (iFromDomains > 0 &&
+	    USmtpSplitEmailAddr(ppszFrom[iFromDomains - 1], szUser, szDomain) == 0) {
 		if ((pUI = UsrGetUserByNameOrAlias(szDomain, szUser)) != NULL) {
 			UsrGetAddress(pUI, FMI.szSender);
 
@@ -192,8 +173,8 @@ static int FilLoadMsgInfo(SPLF_HANDLE hFSpool, FilterMsgInfo & FMI)
 	} else
 		SetEmptyString(FMI.szSender);
 
-	if ((iRcptDomains > 0) &&
-	    (USmtpSplitEmailAddr(ppszRcpt[iRcptDomains - 1], szUser, szDomain) == 0)) {
+	if (iRcptDomains > 0 &&
+	    USmtpSplitEmailAddr(ppszRcpt[iRcptDomains - 1], szUser, szDomain) == 0) {
 		if ((pUI = UsrGetUserByNameOrAlias(szDomain, szUser)) != NULL) {
 			UsrGetAddress(pUI, FMI.szRecipient);
 
@@ -203,8 +184,8 @@ static int FilLoadMsgInfo(SPLF_HANDLE hFSpool, FilterMsgInfo & FMI)
 	} else
 		SetEmptyString(FMI.szRecipient);
 
-	if ((MscGetServerAddress(ppszInfo[smiServerAddr], FMI.LocalAddr) < 0) ||
-	    (MscGetServerAddress(ppszInfo[smiClientAddr], FMI.RemoteAddr) < 0))
+	if (MscGetServerAddress(ppszInfo[smiServerAddr], FMI.LocalAddr) < 0 ||
+	    MscGetServerAddress(ppszInfo[smiClientAddr], FMI.RemoteAddr) < 0)
 		return ErrGetErrorCode();
 
 	StrSNCpy(FMI.szSpoolFile, pszSpoolFile);
@@ -307,7 +288,6 @@ static int FilSelectFilters(char const *pszFilterFilePath, char const *pszMode,
 
 			}
 		}
-
 		StrFreeStrings(ppszTokens);
 	}
 	fclose(pFile);
@@ -371,6 +351,9 @@ int FilExecPreParse(FilterExecCtx *pCtx, char **ppszPEError)
 
 				return -1;
 			}
+		} else if (strcmp(pszVar, "timeo") == 0) {
+			if (pszVal != NULL)
+				pCtx->iTimeout = atoi(pszVal) * 1000;
 		}
 	}
 	StrFreeStrings(ppszEToks);
@@ -378,16 +361,78 @@ int FilExecPreParse(FilterExecCtx *pCtx, char **ppszPEError)
 	return 0;
 }
 
-static int FilPreExec(FilterMsgInfo const &FMI, FilterTokens *pToks, char **ppszPEError)
+static int FilPreExec(FilterMsgInfo const &FMI, FilterExecCtx *pFCtx,
+		      FilterTokens *pToks, char **ppszPEError)
 {
-	FilterExecCtx FCtx;
+	pFCtx->pToks = pToks;
+	pFCtx->pszAuthName = FMI.szAuthName;
+	pFCtx->ulFlags = 0;
+	pFCtx->iTimeout = iFilterTimeout;
 
-	ZeroData(FCtx);
-	FCtx.pToks = pToks;
-	FCtx.pszAuthName = FMI.szAuthName;
-	FCtx.ulFlags = 0;
+	return FilExecPreParse(pFCtx, ppszPEError);
+}
 
-	return FilExecPreParse(&FCtx, ppszPEError);
+static char *FilMacroLkupProc(void *pPrivate, char const *pszName, int iSize)
+{
+	FilterMacroSubstCtx *pFMS = (FilterMacroSubstCtx *) pPrivate;
+
+	if (MemMatch(pszName, iSize, "FROM", 4)) {
+		char const *const *ppszFrom = USmlGetMailFrom(pFMS->hFSpool);
+		int iFromDomains = StrStringsCount(ppszFrom);
+
+		return SysStrDup((iFromDomains > 0) ? ppszFrom[iFromDomains - 1] : "");
+	} else if (MemMatch(pszName, iSize, "RCPT", 4)) {
+		char const *const *ppszRcpt = USmlGetRcptTo(pFMS->hFSpool);
+		int iRcptDomains = StrStringsCount(ppszRcpt);
+
+		return SysStrDup((iRcptDomains > 0) ? ppszRcpt[iRcptDomains - 1] : "");
+	} else if (MemMatch(pszName, iSize, "RFROM", 5)) {
+
+		return SysStrDup(pFMS->pFMI->szSender);
+	} else if (MemMatch(pszName, iSize, "RRCPT", 5)) {
+
+		return SysStrDup(pFMS->pFMI->szRecipient);
+	} else if (MemMatch(pszName, iSize, "FILE", 4)) {
+
+		return SysStrDup(pFMS->FSect.szFilePath);
+	} else if (MemMatch(pszName, iSize, "MSGID", 5)) {
+
+		return SysStrDup(USmlGetSpoolFile(pFMS->hFSpool));
+	} else if (MemMatch(pszName, iSize, "MSGREF", 6)) {
+
+		return SysStrDup(USmlGetSmtpMessageID(pFMS->hFSpool));
+	} else if (MemMatch(pszName, iSize, "LOCALADDR", 9)) {
+		char const *const *ppszInfo = USmlGetInfo(pFMS->hFSpool);
+
+		return SysStrDup(ppszInfo[smiServerAddr]);
+	} else if (MemMatch(pszName, iSize, "REMOTEADDR", 10)) {
+		char const *const *ppszInfo = USmlGetInfo(pFMS->hFSpool);
+
+		return SysStrDup(ppszInfo[smiClientAddr]);
+	} else if (MemMatch(pszName, iSize, "USERAUTH", 8)) {
+
+		return SysStrDup(IsEmptyString(pFMS->pFMI->szAuthName) ? "-":
+				 pFMS->pFMI->szAuthName);
+	}
+
+	return SysStrDup("");
+}
+
+static int FilFilterMacroSubstitutes(char **ppszCmdTokens, SPLF_HANDLE hFSpool,
+				     FilterMsgInfo const &FMI)
+{
+	FilterMacroSubstCtx FMS;
+
+	FMS.hFSpool = hFSpool;
+	FMS.pFMI = &FMI;
+	/*
+	 * This function retrieve the spool file message section and sync the content.
+	 * This is necessary before passing the file name to external programs.
+	 */
+	if (USmlGetMsgFileSection(hFSpool, FMS.FSect) < 0)
+		return ErrGetErrorCode();
+
+	return MscReplaceTokens(ppszCmdTokens, FilMacroLkupProc, &FMS);
 }
 
 static int FilApplyFilter(char const *pszFilterPath, SPLF_HANDLE hFSpool,
@@ -427,12 +472,14 @@ static int FilApplyFilter(char const *pszFilterPath, SPLF_HANDLE hFSpool,
 
 		/* Perform pre-exec filtering (like exec exclude if authenticated, ...) */
 		char *pszPEError = NULL;
+		FilterExecCtx FCtx;
 		FilterTokens Toks;
 
+		ZeroData(FCtx);
 		Toks.ppszCmdTokens = ppszCmdTokens;
 		Toks.iTokenCount = iFieldsCount;
 
-		if (FilPreExec(FMI, &Toks, &pszPEError) < 0) {
+		if (FilPreExec(FMI, &FCtx, &Toks, &pszPEError) < 0) {
 			if (bFilterLogEnabled)
 				FilLogExec(FMI, Toks.ppszCmdTokens, -1,
 					   -1, pszType, pszPEError);
@@ -448,7 +495,7 @@ static int FilApplyFilter(char const *pszFilterPath, SPLF_HANDLE hFSpool,
 		int iExitCode = -1;
 		int iExitFlags = 0;
 		int iExecResult = SysExec(Toks.ppszCmdTokens[0], &Toks.ppszCmdTokens[0],
-					  iFilterTimeout, FILTER_PRIORITY, &iExitCode);
+					  FCtx.iTimeout, FILTER_PRIORITY, &iExitCode);
 
 		/* Log the operation, if requested. */
 		if (bFilterLogEnabled)
@@ -589,66 +636,3 @@ int FilFilterMessage(SPLF_HANDLE hFSpool, QUEUE_HANDLE hQueue,
 	return 0;
 }
 
-static char *FilMacroLkupProc(void *pPrivate, char const *pszName, int iSize)
-{
-	FilterMacroSubstCtx *pFMS = (FilterMacroSubstCtx *) pPrivate;
-
-	if (MemMatch(pszName, iSize, "FROM", 4)) {
-		char const *const *ppszFrom = USmlGetMailFrom(pFMS->hFSpool);
-		int iFromDomains = StrStringsCount(ppszFrom);
-
-		return SysStrDup((iFromDomains > 0) ? ppszFrom[iFromDomains - 1] : "");
-	} else if (MemMatch(pszName, iSize, "RCPT", 4)) {
-		char const *const *ppszRcpt = USmlGetRcptTo(pFMS->hFSpool);
-		int iRcptDomains = StrStringsCount(ppszRcpt);
-
-		return SysStrDup((iRcptDomains > 0) ? ppszRcpt[iRcptDomains - 1] : "");
-	} else if (MemMatch(pszName, iSize, "RFROM", 5)) {
-
-		return SysStrDup(pFMS->pFMI->szSender);
-	} else if (MemMatch(pszName, iSize, "RRCPT", 5)) {
-
-		return SysStrDup(pFMS->pFMI->szRecipient);
-	} else if (MemMatch(pszName, iSize, "FILE", 4)) {
-
-		return SysStrDup(pFMS->FSect.szFilePath);
-	} else if (MemMatch(pszName, iSize, "MSGID", 5)) {
-
-		return SysStrDup(USmlGetSpoolFile(pFMS->hFSpool));
-	} else if (MemMatch(pszName, iSize, "MSGREF", 6)) {
-
-		return SysStrDup(USmlGetSmtpMessageID(pFMS->hFSpool));
-	} else if (MemMatch(pszName, iSize, "LOCALADDR", 9)) {
-		char const *const *ppszInfo = USmlGetInfo(pFMS->hFSpool);
-
-		return SysStrDup(ppszInfo[smiServerAddr]);
-	} else if (MemMatch(pszName, iSize, "REMOTEADDR", 10)) {
-		char const *const *ppszInfo = USmlGetInfo(pFMS->hFSpool);
-
-		return SysStrDup(ppszInfo[smiClientAddr]);
-	} else if (MemMatch(pszName, iSize, "USERAUTH", 8)) {
-
-		return SysStrDup(IsEmptyString(pFMS->pFMI->szAuthName) ? "-":
-				 pFMS->pFMI->szAuthName);
-	}
-
-	return SysStrDup("");
-}
-
-static int FilFilterMacroSubstitutes(char **ppszCmdTokens, SPLF_HANDLE hFSpool,
-				     FilterMsgInfo const &FMI)
-{
-	FilterMacroSubstCtx FMS;
-
-	FMS.hFSpool = hFSpool;
-	FMS.pFMI = &FMI;
-	/*
-	 * This function retrieve the spool file message section and sync the content.
-	 * This is necessary before passing the file name to external programs.
-	 */
-	if (USmlGetMsgFileSection(hFSpool, FMS.FSect) < 0)
-		return ErrGetErrorCode();
-
-	return MscReplaceTokens(ppszCmdTokens, FilMacroLkupProc, &FMS);
-}
-
