@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #define _MAILCONFIG_H
 
 char *CfgGetRootPath(char *pszPath, int iMaxPath);
-char *CfgGetBasedPath(char const *pszFullPath, char *pszBasePath, int iMaxPath);
-char *CfgGetFullPath(char const *pszRelativePath, char *pszFullPath, int iMaxPath);
+char *CfgGetBasedPath(const char *pszFullPath, char *pszBasePath, int iMaxPath);
+char *CfgGetFullPath(const char *pszRelativePath, char *pszFullPath, int iMaxPath);
 
 #endif
