@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -72,30 +72,30 @@
 #define MAX_LMAIL_THREADS           17
 #define STD_LMAILTHREAD_SLEEP_TIME  2
 #define SVR_EXIT_WAIT               480
-#define STD_SERVER_SESSION_TIMEOUT  90
+#define STD_SERVER_SESSION_TIMEOUT  90000
 #define MAX_CLIENTS_WAIT            300
-#define CTRL_SERVER_SESSION_TIMEOUT 120
+#define CTRL_SERVER_SESSION_TIMEOUT 120000
 #define SERVER_SLEEP_TIMESLICE      2
 #define SHUTDOWN_CHECK_TIME         2
 #define STD_POP3AUTH_EXPIRE_TIME    (15 * 60)
-#define FILTER_TIMEOUT              90
+#define FILTER_TIMEOUT              90000
 #define SVR_MAX_SERVICES            32
 
 
 enum SvrServices {
 	SVC_SMAIL = 0,
-		SVC_CTRL,
-		SVC_CRTLS,
-		SVC_POP3,
-		SVC_SMTP,
-		SVC_FING,
-		SVC_LMAIL,
-		SVC_PSYNC,
-		SVC_SMTPS,
-		SVC_POP3S,
-		SVC_CTRLS,
+	SVC_CTRL,
+	SVC_CRTLS,
+	SVC_POP3,
+	SVC_SMTP,
+	SVC_FING,
+	SVC_LMAIL,
+	SVC_PSYNC,
+	SVC_SMTPS,
+	SVC_POP3S,
+	SVC_CTRLS,
 
-		SVC_MAX
+	SVC_MAX
 };
 
 struct SvrShutdownCtx {
@@ -103,39 +103,6 @@ struct SvrShutdownCtx {
 	void *pPrivate;
 };
 
-static void SvrShutdownCleanup(void);
-static int SvrSetShutdown(void);
-static int SvrAddServerAddress(char const *pszServer, SYS_INET_ADDR *pSvrAddr,
-			       int *piPos, int iSize);
-static int SvrSetupCTRL(int iArgCount, char *pszArgs[]);
-static long SvrThreadCntCTRL(ThreadConfig const *pThCfg);
-static void SvrCleanupCTRL(void);
-static int SvrSetupCTRLS(int iArgCount, char *pszArgs[]);
-static void SvrCleanupCTRLS(void);
-static int SvrSetupFING(int iArgCount, char *pszArgs[]);
-static long SvrThreadCntFING(ThreadConfig const *pThCfg);
-static void SvrCleanupFING(void);
-static int SvrSetupPOP3(int iArgCount, char *pszArgs[]);
-static long SvrThreadCntPOP3(ThreadConfig const *pThCfg);
-static void SvrCleanupPOP3(void);
-static int SvrSetupPOP3S(int iArgCount, char *pszArgs[]);
-static void SvrCleanupPOP3S(void);
-static int SvrSetupSMTP(int iArgCount, char *pszArgs[]);
-static long SvrThreadCntSMTP(ThreadConfig const *pThCfg);
-static void SvrCleanupSMTP(void);
-static int SvrSetupSMTPS(int iArgCount, char *pszArgs[]);
-static void SvrCleanupSMTPS(void);
-static int SvrSetupSMAIL(int iArgCount, char *pszArgs[]);
-static void SvrCleanupSMAIL(void);
-static int SvrSetupPSYNC(int iArgCount, char *pszArgs[]);
-static void SvrCleanupPSYNC(void);
-static int SvrSetupLMAIL(int iArgCount, char *pszArgs[]);
-static void SvrCleanupLMAIL(void);
-static int SvrSetup(int iArgCount, char *pszArgs[]);
-static void SvrCleanup(void);
-static void SvrBreakHandler(void);
-static char **SvrMergeArgs(int iArgs, char *pszArgs[], int &iArgsCount);
-
 /* External visible variabiles */
 SHB_HANDLE hShbFING;
 SHB_HANDLE hShbCTRL;
@@ -206,7 +173,7 @@ static int SvrSetShutdown(void)
 		return ERR_FILE_CREATE;
 	}
 
-	char szShutdownTimeStr[256] = "";
+	char szShutdownTimeStr[256];
 
 	MscGetTimeStr(szShutdownTimeStr, sizeof(szShutdownTimeStr));
 
@@ -258,6 +225,30 @@ static int SvrAddServerAddress(char const *pszServer, SYS_INET_ADDR *pSvrAddr,
 	return 0;
 }
 
+static long SvrThreadCntCTRL(ThreadConfig const *pThCfg)
+{
+	long lThreadCnt = 0;
+	CTRLConfig *pCTRLCfg = (CTRLConfig *) ShbLock(pThCfg->hThShb);
+
+	if (pCTRLCfg != NULL) {
+		lThreadCnt = pCTRLCfg->lThreadCount;
+		ShbUnlock(pThCfg->hThShb);
+	}
+
+	return lThreadCnt;
+}
+
+static void SvrCleanupCTRL(void)
+{
+	if (hCTRLThread != SYS_INVALID_THREAD) {
+		SysWaitThread(hCTRLThread, SVR_EXIT_WAIT);
+		SysCloseThread(hCTRLThread, 1);
+	}
+	ShbCloseBlock(hShbCTRL);
+	for (; ThCfgCTRL.iNumSockFDs > 0; ThCfgCTRL.iNumSockFDs--)
+		SysCloseSocket(ThCfgCTRL.SockFDs[ThCfgCTRL.iNumSockFDs - 1]);
+}
+
 static int SvrSetupCTRL(int iArgCount, char *pszArgs[])
 {
 	int iPort = STD_CTRL_PORT, iDisable = 0, iFamily = AF_INET;
@@ -276,7 +267,7 @@ static int SvrSetupCTRL(int iArgCount, char *pszArgs[])
 	ThCfgCTRL.pfThreadProc = CTRLClientThread;
 	ThCfgCTRL.pfThreadCnt = SvrThreadCntCTRL;
 	for (int i = 0; i < iArgCount; i++) {
-		if ((pszArgs[i][0] != '-') || (pszArgs[i][1] != 'C'))
+		if (pszArgs[i][0] != '-' || pszArgs[i][1] != 'C')
 			continue;
 
 		switch (pszArgs[i][2]) {
@@ -287,7 +278,7 @@ static int SvrSetupCTRL(int iArgCount, char *pszArgs[])
 
 		case 't':
 			if (++i < iArgCount)
-				iSessionTimeout = atoi(pszArgs[i]);
+				iSessionTimeout = atoi(pszArgs[i]) * 1000;
 			break;
 
 		case 'l':
@@ -362,35 +353,6 @@ static int SvrSetupCTRL(int iArgCount, char *pszArgs[])
 	return 0;
 }
 
-static long SvrThreadCntCTRL(ThreadConfig const *pThCfg)
-{
-	long lThreadCnt = 0;
-	CTRLConfig *pCTRLCfg = (CTRLConfig *) ShbLock(pThCfg->hThShb);
-
-	if (pCTRLCfg != NULL) {
-		lThreadCnt = pCTRLCfg->lThreadCount;
-		ShbUnlock(hShbCTRL);
-	}
-
-	return lThreadCnt;
-}
-
-static void SvrCleanupCTRL(void)
-{
-	if (hCTRLThread != SYS_INVALID_THREAD) {
-		/* Wait CTRL */
-		SysWaitThread(hCTRLThread, SVR_EXIT_WAIT);
-
-		/* Close CTRL Thread */
-		SysCloseThread(hCTRLThread, 1);
-	}
-
-	ShbCloseBlock(hShbCTRL);
-
-	for (; ThCfgCTRL.iNumSockFDs > 0; ThCfgCTRL.iNumSockFDs--)
-		SysCloseSocket(ThCfgCTRL.SockFDs[ThCfgCTRL.iNumSockFDs - 1]);
-}
-
 static int SvrSetupCTRLS(int iArgCount, char *pszArgs[])
 {
 	int iPort = STD_CTRLS_PORT, iFamily = AF_INET;
@@ -408,7 +370,7 @@ static int SvrSetupCTRLS(int iArgCount, char *pszArgs[])
 	ThCfgCTRLS.ulFlags = THCF_USE_SSL;
 	ThCfgCTRLS.hThShb = hShbCTRL;
 	for (int i = 0; i < iArgCount; i++) {
-		if ((pszArgs[i][0] != '-') || (pszArgs[i][1] != 'W'))
+		if (pszArgs[i][0] != '-' || pszArgs[i][1] != 'W')
 			continue;
 
 		switch (pszArgs[i][2]) {
@@ -455,17 +417,37 @@ static int SvrSetupCTRLS(int iArgCount, char *pszArgs[])
 static void SvrCleanupCTRLS(void)
 {
 	if (hCTRLSThread != SYS_INVALID_THREAD) {
-		/* Wait CTRLS Thread */
 		SysWaitThread(hCTRLSThread, SVR_EXIT_WAIT);
-
-		/* Close CTRLS Thread */
 		SysCloseThread(hCTRLSThread, 1);
 	}
-
 	for (; ThCfgCTRLS.iNumSockFDs > 0; ThCfgCTRLS.iNumSockFDs--)
 		SysCloseSocket(ThCfgCTRLS.SockFDs[ThCfgCTRLS.iNumSockFDs - 1]);
 }
 
+static long SvrThreadCntFING(ThreadConfig const *pThCfg)
+{
+	long lThreadCnt = 0;
+	FINGConfig *pFINGCfg = (FINGConfig *) ShbLock(pThCfg->hThShb);
+
+	if (pFINGCfg != NULL) {
+		lThreadCnt = pFINGCfg->lThreadCount;
+		ShbUnlock(pThCfg->hThShb);
+	}
+
+	return lThreadCnt;
+}
+
+static void SvrCleanupFING(void)
+{
+	if (hFINGThread != SYS_INVALID_THREAD) {
+		SysWaitThread(hFINGThread, SVR_EXIT_WAIT);
+		SysCloseThread(hFINGThread, 1);
+	}
+	ShbCloseBlock(hShbFING);
+	for (; ThCfgFING.iNumSockFDs > 0; ThCfgFING.iNumSockFDs--)
+		SysCloseSocket(ThCfgFING.SockFDs[ThCfgFING.iNumSockFDs - 1]);
+}
+
 static int SvrSetupFING(int iArgCount, char *pszArgs[])
 {
 	int iPort = STD_FINGER_PORT, iDisable = 0, iFamily = AF_INET;
@@ -482,7 +464,7 @@ static int SvrSetupFING(int iArgCount, char *pszArgs[])
 	ThCfgFING.pfThreadProc = FINGClientThread;
 	ThCfgFING.pfThreadCnt = SvrThreadCntFING;
 	for (int i = 0; i < iArgCount; i++) {
-		if ((pszArgs[i][0] != '-') || (pszArgs[i][1] != 'F'))
+		if (pszArgs[i][0] != '-' || pszArgs[i][1] != 'F')
 			continue;
 
 		switch (pszArgs[i][2]) {
@@ -555,33 +537,28 @@ static int SvrSetupFING(int iArgCount, char *pszArgs[])
 	return 0;
 }
 
-static long SvrThreadCntFING(ThreadConfig const *pThCfg)
+static long SvrThreadCntPOP3(ThreadConfig const *pThCfg)
 {
 	long lThreadCnt = 0;
-	FINGConfig *pFINGCfg = (FINGConfig *) ShbLock(pThCfg->hThShb);
+	POP3Config *pPOP3Cfg = (POP3Config *) ShbLock(pThCfg->hThShb);
 
-	if (pFINGCfg != NULL) {
-		lThreadCnt = pFINGCfg->lThreadCount;
-		ShbUnlock(hShbFING);
+	if (pPOP3Cfg != NULL) {
+		lThreadCnt = pPOP3Cfg->lThreadCount;
+		ShbUnlock(pThCfg->hThShb);
 	}
 
 	return lThreadCnt;
 }
 
-static void SvrCleanupFING(void)
+static void SvrCleanupPOP3(void)
 {
-	if (hFINGThread != SYS_INVALID_THREAD) {
-		/* Wait FINGER */
-		SysWaitThread(hFINGThread, SVR_EXIT_WAIT);
-
-		/* Close FINGER Thread */
-		SysCloseThread(hFINGThread, 1);
+	if (hPOP3Thread != SYS_INVALID_THREAD) {
+		SysWaitThread(hPOP3Thread, SVR_EXIT_WAIT);
+		SysCloseThread(hPOP3Thread, 1);
 	}
-
-	ShbCloseBlock(hShbFING);
-
-	for (; ThCfgFING.iNumSockFDs > 0; ThCfgFING.iNumSockFDs--)
-		SysCloseSocket(ThCfgFING.SockFDs[ThCfgFING.iNumSockFDs - 1]);
+	ShbCloseBlock(hShbPOP3);
+	for (; ThCfgPOP3.iNumSockFDs > 0; ThCfgPOP3.iNumSockFDs--)
+		SysCloseSocket(ThCfgPOP3.SockFDs[ThCfgPOP3.iNumSockFDs - 1]);
 }
 
 static int SvrSetupPOP3(int iArgCount, char *pszArgs[])
@@ -603,7 +580,7 @@ static int SvrSetupPOP3(int iArgCount, char *pszArgs[])
 	ThCfgPOP3.pfThreadProc = POP3ClientThread;
 	ThCfgPOP3.pfThreadCnt = SvrThreadCntPOP3;
 	for (int i = 0; i < iArgCount; i++) {
-		if ((pszArgs[i][0] != '-') || (pszArgs[i][1] != 'P'))
+		if (pszArgs[i][0] != '-' || pszArgs[i][1] != 'P')
 			continue;
 
 		switch (pszArgs[i][2]) {
@@ -614,7 +591,7 @@ static int SvrSetupPOP3(int iArgCount, char *pszArgs[])
 
 		case 't':
 			if (++i < iArgCount)
-				iSessionTimeout = atoi(pszArgs[i]);
+				iSessionTimeout = atoi(pszArgs[i]) * 1000;
 			break;
 
 		case 'w':
@@ -701,35 +678,6 @@ static int SvrSetupPOP3(int iArgCount, char *pszArgs[])
 	return 0;
 }
 
-static long SvrThreadCntPOP3(ThreadConfig const *pThCfg)
-{
-	long lThreadCnt = 0;
-	POP3Config *pPOP3Cfg = (POP3Config *) ShbLock(pThCfg->hThShb);
-
-	if (pPOP3Cfg != NULL) {
-		lThreadCnt = pPOP3Cfg->lThreadCount;
-		ShbUnlock(hShbPOP3);
-	}
-
-	return lThreadCnt;
-}
-
-static void SvrCleanupPOP3(void)
-{
-	if (hPOP3Thread != SYS_INVALID_THREAD) {
-		/* Wait POP3 Thread */
-		SysWaitThread(hPOP3Thread, SVR_EXIT_WAIT);
-
-		/* Close POP3 Thread */
-		SysCloseThread(hPOP3Thread, 1);
-	}
-
-	ShbCloseBlock(hShbPOP3);
-
-	for (; ThCfgPOP3.iNumSockFDs > 0; ThCfgPOP3.iNumSockFDs--)
-		SysCloseSocket(ThCfgPOP3.SockFDs[ThCfgPOP3.iNumSockFDs - 1]);
-}
-
 static int SvrSetupPOP3S(int iArgCount, char *pszArgs[])
 {
 	int iPort = STD_POP3S_PORT, iFamily = AF_INET;
@@ -747,7 +695,7 @@ static int SvrSetupPOP3S(int iArgCount, char *pszArgs[])
 	ThCfgPOP3S.ulFlags = THCF_USE_SSL;
 	ThCfgPOP3S.hThShb = hShbPOP3;
 	for (int i = 0; i < iArgCount; i++) {
-		if ((pszArgs[i][0] != '-') || (pszArgs[i][1] != 'B'))
+		if (pszArgs[i][0] != '-' || pszArgs[i][1] != 'B')
 			continue;
 
 		switch (pszArgs[i][2]) {
@@ -794,17 +742,37 @@ static int SvrSetupPOP3S(int iArgCount, char *pszArgs[])
 static void SvrCleanupPOP3S(void)
 {
 	if (hPOP3SThread != SYS_INVALID_THREAD) {
-		/* Wait POP3S Thread */
 		SysWaitThread(hPOP3SThread, SVR_EXIT_WAIT);
-
-		/* Close POP3S Thread */
 		SysCloseThread(hPOP3SThread, 1);
 	}
-
 	for (; ThCfgPOP3S.iNumSockFDs > 0; ThCfgPOP3S.iNumSockFDs--)
 		SysCloseSocket(ThCfgPOP3S.SockFDs[ThCfgPOP3S.iNumSockFDs - 1]);
 }
 
+static long SvrThreadCntSMTP(ThreadConfig const *pThCfg)
+{
+	long lThreadCnt = 0;
+	SMTPConfig *pSMTPCfg = (SMTPConfig *) ShbLock(pThCfg->hThShb);
+
+	if (pSMTPCfg != NULL) {
+		lThreadCnt = pSMTPCfg->lThreadCount;
+		ShbUnlock(pThCfg->hThShb);
+	}
+
+	return lThreadCnt;
+}
+
+static void SvrCleanupSMTP(void)
+{
+	if (hSMTPThread != SYS_INVALID_THREAD) {
+		SysWaitThread(hSMTPThread, SVR_EXIT_WAIT);
+		SysCloseThread(hSMTPThread, 1);
+	}
+	ShbCloseBlock(hShbSMTP);
+	for (; ThCfgSMTP.iNumSockFDs > 0; ThCfgSMTP.iNumSockFDs--)
+		SysCloseSocket(ThCfgSMTP.SockFDs[ThCfgSMTP.iNumSockFDs - 1]);
+}
+
 static int SvrSetupSMTP(int iArgCount, char *pszArgs[])
 {
 	int iPort = STD_SMTP_PORT, iDisable = 0, iFamily = AF_INET;
@@ -825,7 +793,7 @@ static int SvrSetupSMTP(int iArgCount, char *pszArgs[])
 	ThCfgSMTP.pfThreadProc = SMTPClientThread;
 	ThCfgSMTP.pfThreadCnt = SvrThreadCntSMTP;
 	for (int i = 0; i < iArgCount; i++) {
-		if ((pszArgs[i][0] != '-') || (pszArgs[i][1] != 'S'))
+		if (pszArgs[i][0] != '-' || pszArgs[i][1] != 'S')
 			continue;
 
 		switch (pszArgs[i][2]) {
@@ -836,7 +804,7 @@ static int SvrSetupSMTP(int iArgCount, char *pszArgs[])
 
 		case 't':
 			if (++i < iArgCount)
-				iSessionTimeout = atoi(pszArgs[i]);
+				iSessionTimeout = atoi(pszArgs[i]) * 1000;
 			break;
 
 		case 'l':
@@ -922,35 +890,6 @@ static int SvrSetupSMTP(int iArgCount, char *pszArgs[])
 	return 0;
 }
 
-static long SvrThreadCntSMTP(ThreadConfig const *pThCfg)
-{
-	long lThreadCnt = 0;
-	SMTPConfig *pSMTPCfg = (SMTPConfig *) ShbLock(pThCfg->hThShb);
-
-	if (pSMTPCfg != NULL) {
-		lThreadCnt = pSMTPCfg->lThreadCount;
-		ShbUnlock(hShbSMTP);
-	}
-
-	return lThreadCnt;
-}
-
-static void SvrCleanupSMTP(void)
-{
-	if (hSMTPThread != SYS_INVALID_THREAD) {
-		/* Wait SMTP Thread */
-		SysWaitThread(hSMTPThread, SVR_EXIT_WAIT);
-
-		/* Close SMTP Thread */
-		SysCloseThread(hSMTPThread, 1);
-	}
-
-	ShbCloseBlock(hShbSMTP);
-
-	for (; ThCfgSMTP.iNumSockFDs > 0; ThCfgSMTP.iNumSockFDs--)
-		SysCloseSocket(ThCfgSMTP.SockFDs[ThCfgSMTP.iNumSockFDs - 1]);
-}
-
 static int SvrSetupSMTPS(int iArgCount, char *pszArgs[])
 {
 	int iPort = STD_SMTPS_PORT, iFamily = AF_INET;
@@ -968,7 +907,7 @@ static int SvrSetupSMTPS(int iArgCount, char *pszArgs[])
 	ThCfgSMTPS.ulFlags = THCF_USE_SSL;
 	ThCfgSMTPS.hThShb = hShbSMTP;
 	for (int i = 0; i < iArgCount; i++) {
-		if ((pszArgs[i][0] != '-') || (pszArgs[i][1] != 'X'))
+		if (pszArgs[i][0] != '-' || pszArgs[i][1] != 'X')
 			continue;
 
 		switch (pszArgs[i][2]) {
@@ -1015,13 +954,9 @@ static int SvrSetupSMTPS(int iArgCount, char *pszArgs[])
 static void SvrCleanupSMTPS(void)
 {
 	if (hSMTPSThread != SYS_INVALID_THREAD) {
-		/* Wait SMTP Thread */
 		SysWaitThread(hSMTPSThread, SVR_EXIT_WAIT);
-
-		/* Close SMTP Thread */
 		SysCloseThread(hSMTPSThread, 1);
 	}
-
 	for (; ThCfgSMTPS.iNumSockFDs > 0; ThCfgSMTPS.iNumSockFDs--)
 		SysCloseSocket(ThCfgSMTPS.SockFDs[ThCfgSMTPS.iNumSockFDs - 1]);
 }
@@ -1047,7 +982,7 @@ static int SvrSetupSMAIL(int iArgCount, char *pszArgs[])
 	iNumSMAILThreads = STD_SMAIL_THREADS;
 
 	for (i = 0; i < iArgCount; i++) {
-		if ((pszArgs[i][0] != '-') || (pszArgs[i][1] != 'Q'))
+		if (pszArgs[i][0] != '-' || pszArgs[i][1] != 'Q')
 			continue;
 
 		switch (pszArgs[i][2]) {
@@ -1055,7 +990,8 @@ static int SvrSetupSMAIL(int iArgCount, char *pszArgs[])
 			if (++i < iArgCount)
 				iNumSMAILThreads = atoi(pszArgs[i]);
 
-			iNumSMAILThreads = Min(MAX_SMAIL_THREADS, Max(1, iNumSMAILThreads));
+			iNumSMAILThreads = Min(MAX_SMAIL_THREADS,
+					       Max(1, iNumSMAILThreads));
 			break;
 
 		case 't':
@@ -1079,7 +1015,7 @@ static int SvrSetupSMAIL(int iArgCount, char *pszArgs[])
 
 		case 'T':
 			if (++i < iArgCount)
-				iFilterTimeout = atoi(pszArgs[i]);
+				iFilterTimeout = atoi(pszArgs[i]) * 1000;
 			break;
 
 		case 'g':
@@ -1106,10 +1042,9 @@ static int SvrSetupSMAIL(int iArgCount, char *pszArgs[])
 	ShbUnlock(hShbSMAIL);
 
 	/* Initialize queue fs */
-	char szSpoolDir[SYS_MAX_PATH] = "";
+	char szSpoolDir[SYS_MAX_PATH];
 
 	SvrGetSpoolDir(szSpoolDir, sizeof(szSpoolDir));
-
 	if ((hSpoolQueue = QueOpen(szSpoolDir, iMaxRetry, iRetryTimeout, iRetryIncrRatio,
 				   iQueueSplitLevel)) == INVALID_QUEUE_HANDLE) {
 		ErrorPush();
@@ -1131,19 +1066,13 @@ static int SvrSetupSMAIL(int iArgCount, char *pszArgs[])
 
 static void SvrCleanupSMAIL(void)
 {
-	/* Wait SMAIL Threads */
 	int i;
 
 	for (i = 0; i < iNumSMAILThreads; i++)
 		SysWaitThread(hSMAILThreads[i], SVR_EXIT_WAIT);
-
-	/* Close SMAIL Threads */
 	for (i = 0; i < iNumSMAILThreads; i++)
 		SysCloseThread(hSMAILThreads[i], 1);
-
 	ShbCloseBlock(hShbSMAIL);
-
-	/* Close the mail queue */
 	QueClose(hSpoolQueue);
 }
 
@@ -1170,7 +1099,7 @@ static int SvrSetupPSYNC(int iArgCount, char *pszArgs[])
 	hPSYNCThread = SYS_INVALID_THREAD;
 
 	for (int i = 0; i < iArgCount; i++) {
-		if ((pszArgs[i][0] != '-') || (pszArgs[i][1] != 'Y'))
+		if (pszArgs[i][0] != '-' || pszArgs[i][1] != 'Y')
 			continue;
 
 		switch (pszArgs[i][2]) {
@@ -1183,7 +1112,8 @@ static int SvrSetupPSYNC(int iArgCount, char *pszArgs[])
 			if (++i < iArgCount)
 				iNumSyncThreads = atoi(pszArgs[i]);
 
-			iNumSyncThreads = Min(MAX_PSYNC_NUM_THREADS, Max(1, iNumSyncThreads));
+			iNumSyncThreads = Min(MAX_PSYNC_NUM_THREADS,
+					      Max(1, iNumSyncThreads));
 			break;
 
 		case 'l':
@@ -1192,7 +1122,7 @@ static int SvrSetupPSYNC(int iArgCount, char *pszArgs[])
 
 		case 'T':
 			if (++i < iArgCount)
-				iPOP3ClientTimeout = atoi(pszArgs[i]);
+				iPOP3ClientTimeout = atoi(pszArgs[i]) * 1000;
 			break;
 
 		case '-':
@@ -1250,13 +1180,9 @@ static int SvrSetupPSYNC(int iArgCount, char *pszArgs[])
 static void SvrCleanupPSYNC(void)
 {
 	if (hPSYNCThread != SYS_INVALID_THREAD) {
-		/* Wait PSYNC Thread */
 		SysWaitThread(hPSYNCThread, SVR_EXIT_WAIT);
-
-		/* Close PSYNC Thread */
 		SysCloseThread(hPSYNCThread, 1);
 	}
-
 	ShbCloseBlock(hShbPSYNC);
 	SysCloseSemaphore(hSyncSem);
 }
@@ -1280,7 +1206,7 @@ static int SvrSetupLMAIL(int iArgCount, char *pszArgs[])
 	iNumLMAILThreads = STD_LMAIL_THREADS;
 
 	for (i = 0; i < iArgCount; i++) {
-		if ((pszArgs[i][0] != '-') || (pszArgs[i][1] != 'L'))
+		if (pszArgs[i][0] != '-' || pszArgs[i][1] != 'L')
 			continue;
 
 		switch (pszArgs[i][2]) {
@@ -1335,44 +1261,37 @@ static int SvrSetupLMAIL(int iArgCount, char *pszArgs[])
 
 static void SvrCleanupLMAIL(void)
 {
-	/* Wait LMAIL Threads */
 	int i;
 
 	for (i = 0; i < iNumLMAILThreads; i++)
 		SysWaitThread(hLMAILThreads[i], SVR_EXIT_WAIT);
-
-	/* Close LMAIL Threads */
 	for (i = 0; i < iNumLMAILThreads; i++)
 		SysCloseThread(hLMAILThreads[i], 1);
-
 	ShbCloseBlock(hShbLMAIL);
 }
 
 static int SvrSetup(int iArgCount, char *pszArgs[])
 {
+	char *pszValue;
+
 	StrSNCpy(szMailPath, SYS_BASE_FS_STR);
-
-	char *pszValue = SysGetEnv(ENV_MAIN_PATH);
-
-	if (pszValue != NULL) {
+	if ((pszValue = SysGetEnv(ENV_MAIN_PATH)) != NULL) {
 		if (strncmp(szMailPath, pszValue, strlen(szMailPath)) == 0)
 			StrSNCpy(szMailPath, pszValue);
 		else
 			StrSNCat(szMailPath, pszValue);
 		DelFinalSlash(szMailPath);
-
 		SysFree(pszValue);
 	}
 
 	iNumShCtxs = 0;
 	bServerDebug = false;
 
-	int iSndBufSize = -1;
-	int iRcvBufSize = -1;
+	int iSndBufSize = -1, iRcvBufSize = -1;
 	int iDnsCacheDirs = DNS_HASH_NUM_DIRS;
 
 	for (int i = 0; i < iArgCount; i++) {
-		if ((pszArgs[i][0] != '-') || (pszArgs[i][1] != 'M'))
+		if (pszArgs[i][0] != '-' || pszArgs[i][1] != 'M')
 			continue;
 
 		switch (pszArgs[i][2]) {
@@ -1490,55 +1409,37 @@ static int SvrSetup(int iArgCount, char *pszArgs[])
 
 static void SvrCleanup(void)
 {
-	/* Cleanup SSL support */
 	BSslCleanup();
-
-	/* Cleanup resource lockers */
 	RLckCleanupLockers();
-
-	/* Clear shutdown condition */
 	SvrShutdownCleanup();
 }
 
 static void SvrBreakHandler(void)
 {
-	/* Set shutdown condition */
 	SvrSetShutdown();
 }
 
 static char **SvrMergeArgs(int iArgs, char *pszArgs[], int &iArgsCount)
 {
-	int iCmdArgs = 0;
-	char **ppszCmdArgs = NULL;
+	int i, j, iCmdArgs = 0;
+	char **ppszCmdArgs = NULL, **ppszMergeArgs;
 	char *pszCmdLine = SysGetEnv(ENV_CMD_LINE);
 
 	if (pszCmdLine != NULL) {
 		ppszCmdArgs = StrGetArgs(pszCmdLine, iCmdArgs);
-
 		SysFree(pszCmdLine);
 	}
-
-	char **ppszMergeArgs = (char **) SysAlloc((iCmdArgs + iArgs + 1) * sizeof(char *));
-
-	if (ppszMergeArgs == NULL) {
-		if (ppszCmdArgs != NULL)
-			StrFreeStrings(ppszCmdArgs);
-
+	if ((ppszMergeArgs = (char **) SysAlloc((iCmdArgs + iArgs + 1) *
+						sizeof(char *))) == NULL) {
+		StrFreeStrings(ppszCmdArgs);
 		return NULL;
 	}
-
-	iArgsCount = 0;
-
-	for (int i = 0; i < iArgs; i++, iArgsCount++)
+	for (i = iArgsCount = 0; i < iArgs; i++, iArgsCount++)
 		ppszMergeArgs[iArgsCount] = SysStrDup(pszArgs[i]);
-
-	for (int j = 0; j < iCmdArgs; j++, iArgsCount++)
+	for (j = 0; j < iCmdArgs; j++, iArgsCount++)
 		ppszMergeArgs[iArgsCount] = SysStrDup(ppszCmdArgs[j]);
-
 	ppszMergeArgs[iArgsCount] = NULL;
-
-	if (ppszCmdArgs != NULL)
-		StrFreeStrings(ppszCmdArgs);
+	StrFreeStrings(ppszCmdArgs);
 
 	return ppszMergeArgs;
 }
@@ -1563,7 +1464,6 @@ int SvrMain(int iArgCount, char *pszArgs[])
 		SysCleanupLibrary();
 		return ErrorPop();
 	}
-
 	if (SvrSetup(iMergeArgsCount, ppszMergeArgs) < 0) {
 		ErrorPush();
 		SysLogMessage(LOG_LEV_ERROR, "%s\n", ErrGetErrorString());
@@ -1591,6 +1491,8 @@ int SvrMain(int iArgCount, char *pszArgs[])
 	/* Set stop handler */
 	SysSetBreakHandler(SvrBreakHandler);
 
+	SysLogMessage(LOG_LEV_MESSAGE, APP_NAME_VERSION_STR " server started\n");
+
 	/* Server main loop */
 	for (; !SvrInShutdown(true);) {
 		SysSleep(SERVER_SLEEP_TIMESLICE);
@@ -1658,7 +1560,7 @@ bool SvrInShutdown(bool bForceCheck)
 	static time_t tLastCheck = 0;
 	static bool bShutdown = false;
 
-	if (bForceCheck || (tNow > (tLastCheck + SHUTDOWN_CHECK_TIME))) {
+	if (bForceCheck || tNow > (tLastCheck + SHUTDOWN_CHECK_TIME)) {
 		tLastCheck = tNow;
 
 		if (bServerShutdown)
@@ -1669,7 +1571,8 @@ bool SvrInShutdown(bool bForceCheck)
 			SysShutdownLibrary();
 
 			bShutdown = true;
-		}
+		} else
+			bShutdown = false;
 	}
 
 	return bShutdown;
