@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -77,28 +77,314 @@ struct QueueMessage {
 	unsigned long ulFlags;
 };
 
-static int QueCreateStruct(char const *pszRootPath);
-static int QueLoad(MessageQueue *pMQ);
-static int QueLoadMessages(MessageQueue *pMQ, int iLevel1, int iLevel2);
-static QueueMessage *QueAllocMessage(int iLevel1, int iLevel2, char const *pszQueueDir,
-				     char const *pszFileName, int iNumTries, time_t tLastTry);
-static int QueFreeMessage(QueueMessage *pQM);
-static int QueFreeMessList(SysListHead *pHead);
-static int QueLoadMessageStat(MessageQueue *pMQ, QueueMessage *pQM);
-static int QueStatMessage(MessageQueue *pMQ, QueueMessage *pQM);
 static int QueGetFilePath(MessageQueue *pMQ, QueueMessage *pQM, char *pszFilePath,
 			  char const *pszQueueDir = NULL);
-static int QueDoMessageCleanup(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage);
-static int QueAddNew(MessageQueue *pMQ, QueueMessage *pQM);
-static bool QueMessageExpired(MessageQueue *pMQ, QueueMessage *pQM);
+
+static QueueMessage *QueAllocMessage(int iLevel1, int iLevel2, char const *pszQueueDir,
+				     char const *pszFileName, int iNumTries, time_t tLastTry)
+{
+	QueueMessage *pQM = (QueueMessage *) SysAlloc(sizeof(QueueMessage));
+
+	if (pQM == NULL)
+		return NULL;
+
+	SYS_INIT_LIST_LINK(&pQM->LLink);
+	pQM->iLevel1 = iLevel1;
+	pQM->iLevel2 = iLevel2;
+	pQM->pszQueueDir = pszQueueDir;
+	pQM->pszFileName = SysStrDup(pszFileName);
+	pQM->iNumTries = iNumTries;
+	pQM->tLastTry = tLastTry;
+	pQM->ulFlags = 0;
+
+	return pQM;
+}
+
+static int QueFreeMessage(QueueMessage *pQM)
+{
+	SysFree(pQM->pszFileName);
+	SysFree(pQM);
+
+	return 0;
+}
+
+static int QueFreeMessList(SysListHead *pHead)
+{
+	SysListHead *pLLink;
+
+	while ((pLLink = SYS_LIST_FIRST(pHead)) != NULL) {
+		QueueMessage *pQM = SYS_LIST_ENTRY(pLLink, QueueMessage, LLink);
+
+		SYS_LIST_DEL(pLLink);
+		QueFreeMessage(pQM);
+	}
+
+	return 0;
+}
+
+static int QueCreateStruct(char const *pszRootPath)
+{
+	/* Create message dir (new messages queue) */
+	char szDirPath[SYS_MAX_PATH];
+
+	StrSNCpy(szDirPath, pszRootPath);
+	AppendSlash(szDirPath);
+	StrSNCat(szDirPath, QUEUE_MESS_DIR);
+
+	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
+		return ErrGetErrorCode();
+
+	/* Create message resend dir (resend messages queue) */
+	StrSNCpy(szDirPath, pszRootPath);
+	AppendSlash(szDirPath);
+	StrSNCat(szDirPath, QUEUE_RSND_DIR);
+
+	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
+		return ErrGetErrorCode();
+
+	/* Create info dir */
+	StrSNCpy(szDirPath, pszRootPath);
+	AppendSlash(szDirPath);
+	StrSNCat(szDirPath, QUEUE_INFO_DIR);
+
+	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
+		return ErrGetErrorCode();
+
+	/* Create temp dir */
+	StrSNCpy(szDirPath, pszRootPath);
+	AppendSlash(szDirPath);
+	StrSNCat(szDirPath, QUEUE_TEMP_DIR);
+
+	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
+		return ErrGetErrorCode();
+
+	/* Create send log dir */
+	StrSNCpy(szDirPath, pszRootPath);
+	AppendSlash(szDirPath);
+	StrSNCat(szDirPath, QUEUE_SLOG_DIR);
+
+	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
+		return ErrGetErrorCode();
+
+	/* Create custom message processing dir */
+	StrSNCpy(szDirPath, pszRootPath);
+	AppendSlash(szDirPath);
+	StrSNCat(szDirPath, QUEUE_CUST_DIR);
+
+	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
+		return ErrGetErrorCode();
+
+	/* Create user custom message processing dir (mailproc.tab cache) */
+	StrSNCpy(szDirPath, pszRootPath);
+	AppendSlash(szDirPath);
+	StrSNCat(szDirPath, QUEUE_MPRC_DIR);
+
+	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
+		return ErrGetErrorCode();
+
+	/* Create frozen dir */
+	StrSNCpy(szDirPath, pszRootPath);
+	AppendSlash(szDirPath);
+	StrSNCat(szDirPath, QUEUE_FROZ_DIR);
+
+	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
+		return ErrGetErrorCode();
+
+	return 0;
+}
+
+static int QueLoadMessageStat(MessageQueue *pMQ, QueueMessage *pQM)
+{
+	/* Build the slog file path */
+	char szSlogFilePath[SYS_MAX_PATH];
+
+	QueGetFilePath(pMQ, pQM, szSlogFilePath, QUEUE_SLOG_DIR);
+
+	/* Try to load message statistics */
+	FILE *pLogFile = fopen(szSlogFilePath, "rt");
+
+	if (pLogFile != NULL) {
+		int iNumTries = 0;
+		unsigned long ulLastTime = 0, ulPeekTime;
+		char szLogLine[1024];
+
+		while (MscFGets(szLogLine, sizeof(szLogLine) - 1, pLogFile) != NULL)
+			if (sscanf(szLogLine, "[PeekTime] %lu", &ulPeekTime) == 1)
+				++iNumTries, ulLastTime = ulPeekTime;
+
+		fclose(pLogFile);
+
+		pQM->iNumTries = iNumTries;
+		pQM->tLastTry = (time_t) ulLastTime;
+	}
+
+	return 0;
+}
+
+static int QueLoadMessages(MessageQueue *pMQ, int iLevel1, int iLevel2)
+{
+	/* File scan the new messages dir */
+	char szDirPath[SYS_MAX_PATH];
+
+	SysSNPrintf(szDirPath, sizeof(szDirPath) - 1, "%s%d%s%d%s%s",
+		    pMQ->pszRootPath, iLevel1, SYS_SLASH_STR, iLevel2,
+		    SYS_SLASH_STR, QUEUE_MESS_DIR);
+
+	char szMsgFileName[SYS_MAX_PATH];
+	FSCAN_HANDLE hFileScan = MscFirstFile(szDirPath, 0, szMsgFileName,
+					      sizeof(szMsgFileName));
+
+	if (hFileScan != INVALID_FSCAN_HANDLE) {
+		do {
+			if (!IsDotFilename(szMsgFileName)) {
+				QueueMessage *pQM =
+					QueAllocMessage(iLevel1, iLevel2, QUEUE_MESS_DIR,
+							szMsgFileName, 0, 0);
+
+				if (pQM != NULL) {
+					/* Add the file to the message queue */
+					SYS_LIST_ADDT(&pQM->LLink, &pMQ->ReadyQueue);
+					++pMQ->iReadyCount;
+				}
+			}
+		} while (MscNextFile(hFileScan, szMsgFileName, sizeof(szMsgFileName)));
+		MscCloseFindFile(hFileScan);
+
+		/* Set the mess event if the queue is not empty */
+		if (pMQ->iReadyCount > 0)
+			SysSetEvent(pMQ->hReadyEvent);
+	}
+	/* File scan the resend messages dir */
+	SysSNPrintf(szDirPath, sizeof(szDirPath) - 1, "%s%d%s%d%s%s",
+		    pMQ->pszRootPath, iLevel1, SYS_SLASH_STR, iLevel2,
+		    SYS_SLASH_STR, QUEUE_RSND_DIR);
+
+	if ((hFileScan = MscFirstFile(szDirPath, 0, szMsgFileName,
+				      sizeof(szMsgFileName))) != INVALID_FSCAN_HANDLE) {
+		do {
+			if (!IsDotFilename(szMsgFileName)) {
+				QueueMessage *pQM = QueAllocMessage(iLevel1, iLevel2,
+								    QUEUE_RSND_DIR,
+								    szMsgFileName, 0, 0);
+
+				if (pQM != NULL) {
+					/* Load message statistics */
+					if (QueLoadMessageStat(pMQ, pQM) < 0) {
+						SysLogMessage(LOG_LEV_ERROR,
+							      "Error loading queue file: '%s%d%s%d%s%s%s%s'\n",
+							      pMQ->pszRootPath, iLevel1,
+							      SYS_SLASH_STR, iLevel2,
+							      SYS_SLASH_STR, QUEUE_RSND_DIR,
+							      SYS_SLASH_STR, szMsgFileName);
+
+						QueFreeMessage(pQM);
+					} else {
+						/* Add the file to the resend queue */
+						SYS_LIST_ADDT(&pQM->LLink, &pMQ->RsndArenaQueue);
+						++pMQ->iRsndArenaCount;
+					}
+				}
+			}
+		} while (MscNextFile(hFileScan, szMsgFileName, sizeof(szMsgFileName)));
+		MscCloseFindFile(hFileScan);
+	}
+
+	return 0;
+}
+
+static int QueLoad(MessageQueue *pMQ)
+{
+	char szCurrPath[SYS_MAX_PATH];
+
+	for (int i = 0; i < pMQ->iNumDirsLevel; i++) {
+		SysSNPrintf(szCurrPath, sizeof(szCurrPath) - 1, "%s%d", pMQ->pszRootPath, i);
+		if (!SysExistDir(szCurrPath) && (SysMakeDir(szCurrPath) < 0))
+			return ErrGetErrorCode();
+
+		for (int j = 0; j < pMQ->iNumDirsLevel; j++) {
+			SysSNPrintf(szCurrPath, sizeof(szCurrPath) - 1, "%s%d%s%d",
+				    pMQ->pszRootPath, i, SYS_SLASH_STR, j);
+			if (!SysExistDir(szCurrPath) && (SysMakeDir(szCurrPath) < 0))
+				return ErrGetErrorCode();
+
+			if (QueCreateStruct(szCurrPath) < 0 ||
+			    QueLoadMessages(pMQ, i, j) < 0)
+				return ErrGetErrorCode();
+		}
+	}
+
+	return 0;
+}
+
 static time_t QueNextRetryOp(int iNumTries, unsigned int uRetryTimeout,
-			     unsigned int uRetryIncrRatio);
-static bool QueMessageReadyToSend(MessageQueue *pMQ, QueueMessage *pQM);
-static int QueAddRsnd(MessageQueue *pMQ, QueueMessage *pQM);
-static unsigned int QueRsndThread(void *pThreadData);
-static int QueScanRsndArena(MessageQueue *pMQ);
-static bool QueMessageDestMatch(MessageQueue *pMQ, QueueMessage *pQM,
-				char const *pszAddressMatch);
+			     unsigned int uRetryIncrRatio)
+{
+	unsigned int uNextOp = uRetryTimeout;
+
+	if (uRetryIncrRatio != 0)
+		for (int i = 1; i < iNumTries; i++)
+			uNextOp += uNextOp / uRetryIncrRatio;
+
+	return (time_t) uNextOp;
+}
+
+static bool QueMessageReadyToSend(MessageQueue *pMQ, QueueMessage *pQM)
+{
+	return (time(NULL) > (pQM->tLastTry +
+			      QueNextRetryOp(pQM->iNumTries, (unsigned int) pMQ->iRetryTimeout,
+					     (unsigned int) pMQ->iRetryIncrRatio)));
+}
+
+static int QueScanRsndArena(MessageQueue *pMQ)
+{
+	if (SysLockMutex(pMQ->hMutex, SYS_INFINITE_TIMEOUT) < 0)
+		return ErrGetErrorCode();
+
+	SysListHead *pLLink;
+
+	SYS_LIST_FOR_EACH(pLLink, &pMQ->RsndArenaQueue) {
+		QueueMessage *pQM = SYS_LIST_ENTRY(pLLink, QueueMessage, LLink);
+
+		if (QueMessageReadyToSend(pMQ, pQM)) {
+			/* Set the list pointer to the next item */
+			pLLink = pLLink->pPrev;
+
+			/* Remove item from resend arena */
+			SYS_LIST_DEL(&pQM->LLink);
+			--pMQ->iRsndArenaCount;
+
+			/* Add item from resend queue */
+			SYS_LIST_ADDT(&pQM->LLink, &pMQ->ReadyQueue);
+			++pMQ->iReadyCount;
+		}
+	}
+	if (pMQ->iReadyCount > 0)
+		SysSetEvent(pMQ->hReadyEvent);
+	SysUnlockMutex(pMQ->hMutex);
+
+	return 0;
+}
+
+static unsigned int QueRsndThread(void *pThreadData)
+{
+	MessageQueue *pMQ = (MessageQueue *) pThreadData;
+	int iElapsedTime = 0;
+
+	while ((pMQ->ulFlags & QUEF_SHUTDOWN) == 0) {
+		SysSleep(QUE_ARENA_SCAN_WAIT);
+
+		iElapsedTime += QUE_ARENA_SCAN_WAIT;
+		if (iElapsedTime > QUE_ARENA_SCAN_INTERVAL) {
+			iElapsedTime = 0;
+
+			/* Scan rsnd arena to prepare messages to resend */
+			QueScanRsndArena(pMQ);
+		}
+	}
+	pMQ->ulFlags &= ~QUEF_SHUTDOWN;
+
+	return 0;
+}
 
 QUEUE_HANDLE QueOpen(char const *pszRootPath, int iMaxRetry, int iRetryTimeout,
 		     int iRetryIncrRatio, int iNumDirsLevel)
@@ -128,7 +414,7 @@ QUEUE_HANDLE QueOpen(char const *pszRootPath, int iMaxRetry, int iRetryTimeout,
 		return INVALID_QUEUE_HANDLE;
 	}
 	/* Set the queue root path */
-	char szRootPath[SYS_MAX_PATH] = "";
+	char szRootPath[SYS_MAX_PATH];
 
 	StrSNCpy(szRootPath, pszRootPath);
 	AppendSlash(szRootPath);
@@ -197,214 +483,6 @@ char const *QueGetRootPath(QUEUE_HANDLE hQueue)
 	return pMQ->pszRootPath;
 }
 
-static int QueCreateStruct(char const *pszRootPath)
-{
-	/* Create message dir ( new messages queue ) */
-	char szDirPath[SYS_MAX_PATH] = "";
-
-	StrSNCpy(szDirPath, pszRootPath);
-	AppendSlash(szDirPath);
-	StrSNCat(szDirPath, QUEUE_MESS_DIR);
-
-	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
-		return ErrGetErrorCode();
-
-	/* Create message resend dir ( resend messages queue ) */
-	StrSNCpy(szDirPath, pszRootPath);
-	AppendSlash(szDirPath);
-	StrSNCat(szDirPath, QUEUE_RSND_DIR);
-
-	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
-		return ErrGetErrorCode();
-
-	/* Create info dir */
-	StrSNCpy(szDirPath, pszRootPath);
-	AppendSlash(szDirPath);
-	StrSNCat(szDirPath, QUEUE_INFO_DIR);
-
-	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
-		return ErrGetErrorCode();
-
-	/* Create temp dir */
-	StrSNCpy(szDirPath, pszRootPath);
-	AppendSlash(szDirPath);
-	StrSNCat(szDirPath, QUEUE_TEMP_DIR);
-
-	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
-		return ErrGetErrorCode();
-
-	/* Create send log dir */
-	StrSNCpy(szDirPath, pszRootPath);
-	AppendSlash(szDirPath);
-	StrSNCat(szDirPath, QUEUE_SLOG_DIR);
-
-	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
-		return ErrGetErrorCode();
-
-	/* Create custom message processing dir */
-	StrSNCpy(szDirPath, pszRootPath);
-	AppendSlash(szDirPath);
-	StrSNCat(szDirPath, QUEUE_CUST_DIR);
-
-	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
-		return ErrGetErrorCode();
-
-	/* Create user custom message processing dir ( mailproc.tab cache ) */
-	StrSNCpy(szDirPath, pszRootPath);
-	AppendSlash(szDirPath);
-	StrSNCat(szDirPath, QUEUE_MPRC_DIR);
-
-	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
-		return ErrGetErrorCode();
-
-	/* Create frozen dir */
-	StrSNCpy(szDirPath, pszRootPath);
-	AppendSlash(szDirPath);
-	StrSNCat(szDirPath, QUEUE_FROZ_DIR);
-
-	if (!SysExistDir(szDirPath) && SysMakeDir(szDirPath) < 0)
-		return ErrGetErrorCode();
-
-	return 0;
-}
-
-static int QueLoad(MessageQueue *pMQ)
-{
-	char szCurrPath[SYS_MAX_PATH] = "";
-
-	for (int i = 0; i < pMQ->iNumDirsLevel; i++) {
-		SysSNPrintf(szCurrPath, sizeof(szCurrPath) - 1, "%s%d", pMQ->pszRootPath, i);
-		if (!SysExistDir(szCurrPath) && (SysMakeDir(szCurrPath) < 0))
-			return ErrGetErrorCode();
-
-		for (int j = 0; j < pMQ->iNumDirsLevel; j++) {
-			SysSNPrintf(szCurrPath, sizeof(szCurrPath) - 1, "%s%d%s%d",
-				    pMQ->pszRootPath, i, SYS_SLASH_STR, j);
-			if (!SysExistDir(szCurrPath) && (SysMakeDir(szCurrPath) < 0))
-				return ErrGetErrorCode();
-
-			if (QueCreateStruct(szCurrPath) < 0 ||
-			    QueLoadMessages(pMQ, i, j) < 0)
-				return ErrGetErrorCode();
-		}
-	}
-
-	return 0;
-}
-
-static int QueLoadMessages(MessageQueue *pMQ, int iLevel1, int iLevel2)
-{
-	/* File scan the new messages dir */
-	char szDirPath[SYS_MAX_PATH] = "";
-
-	SysSNPrintf(szDirPath, sizeof(szDirPath) - 1, "%s%d%s%d%s%s",
-		    pMQ->pszRootPath, iLevel1, SYS_SLASH_STR, iLevel2,
-		    SYS_SLASH_STR, QUEUE_MESS_DIR);
-
-	char szMsgFileName[SYS_MAX_PATH] = "";
-	FSCAN_HANDLE hFileScan = MscFirstFile(szDirPath, 0, szMsgFileName,
-					      sizeof(szMsgFileName));
-
-	if (hFileScan != INVALID_FSCAN_HANDLE) {
-		do {
-			if (!IsDotFilename(szMsgFileName)) {
-				QueueMessage *pQM =
-					QueAllocMessage(iLevel1, iLevel2, QUEUE_MESS_DIR,
-							szMsgFileName, 0, 0);
-
-				if (pQM != NULL) {
-					/* Add the file to the message queue */
-					SYS_LIST_ADDT(&pQM->LLink, &pMQ->ReadyQueue);
-					++pMQ->iReadyCount;
-				}
-			}
-		} while (MscNextFile(hFileScan, szMsgFileName, sizeof(szMsgFileName)));
-		MscCloseFindFile(hFileScan);
-
-		/* Set the mess event if the queue is not empty */
-		if (pMQ->iReadyCount > 0)
-			SysSetEvent(pMQ->hReadyEvent);
-	}
-	/* File scan the resend messages dir */
-	SysSNPrintf(szDirPath, sizeof(szDirPath) - 1, "%s%d%s%d%s%s",
-		    pMQ->pszRootPath, iLevel1, SYS_SLASH_STR, iLevel2,
-		    SYS_SLASH_STR, QUEUE_RSND_DIR);
-
-	if ((hFileScan = MscFirstFile(szDirPath, 0, szMsgFileName,
-				      sizeof(szMsgFileName))) != INVALID_FSCAN_HANDLE) {
-		do {
-			if (!IsDotFilename(szMsgFileName)) {
-				QueueMessage *pQM = QueAllocMessage(iLevel1, iLevel2,
-								    QUEUE_RSND_DIR,
-								    szMsgFileName, 0, 0);
-
-				if (pQM != NULL) {
-					/* Load message statistics */
-					if (QueLoadMessageStat(pMQ, pQM) < 0) {
-						SysLogMessage(LOG_LEV_ERROR,
-							      "Error loading queue file: '%s%d%s%d%s%s%s%s'\n",
-							      pMQ->pszRootPath, iLevel1,
-							      SYS_SLASH_STR, iLevel2,
-							      SYS_SLASH_STR, QUEUE_RSND_DIR,
-							      SYS_SLASH_STR, szMsgFileName);
-
-						QueFreeMessage(pQM);
-					} else {
-						/* Add the file to the resend queue */
-						SYS_LIST_ADDT(&pQM->LLink, &pMQ->RsndArenaQueue);
-						++pMQ->iRsndArenaCount;
-					}
-				}
-			}
-		} while (MscNextFile(hFileScan, szMsgFileName, sizeof(szMsgFileName)));
-		MscCloseFindFile(hFileScan);
-	}
-
-	return 0;
-}
-
-static QueueMessage *QueAllocMessage(int iLevel1, int iLevel2, char const *pszQueueDir,
-				     char const *pszFileName, int iNumTries, time_t tLastTry)
-{
-	QueueMessage *pQM = (QueueMessage *) SysAlloc(sizeof(QueueMessage));
-
-	if (pQM == NULL)
-		return NULL;
-
-	SYS_INIT_LIST_LINK(&pQM->LLink);
-	pQM->iLevel1 = iLevel1;
-	pQM->iLevel2 = iLevel2;
-	pQM->pszQueueDir = pszQueueDir;
-	pQM->pszFileName = SysStrDup(pszFileName);
-	pQM->iNumTries = iNumTries;
-	pQM->tLastTry = tLastTry;
-	pQM->ulFlags = 0;
-
-	return pQM;
-}
-
-static int QueFreeMessage(QueueMessage *pQM)
-{
-	SysFree(pQM->pszFileName);
-	SysFree(pQM);
-
-	return 0;
-}
-
-static int QueFreeMessList(SysListHead *pHead)
-{
-	SysListHead *pLLink;
-
-	while ((pLLink = SYS_LIST_FIRST(pHead)) != NULL) {
-		QueueMessage *pQM = SYS_LIST_ENTRY(pLLink, QueueMessage, LLink);
-
-		SYS_LIST_DEL(pLLink);
-		QueFreeMessage(pQM);
-	}
-
-	return 0;
-}
-
 char *QueLoadLastLogEntry(char const *pszLogFilePath)
 {
 	FILE *pLogFile = fopen(pszLogFilePath, "rb");
@@ -413,37 +491,37 @@ char *QueLoadLastLogEntry(char const *pszLogFilePath)
 		ErrSetErrorCode(ERR_FILE_OPEN, pszLogFilePath);
 		return NULL;
 	}
-	/* Walk log entries */
-	unsigned long ulCurrOffset = 0, ulBaseOffset = (unsigned long) -1;
-	unsigned long ulEndOffset, ulPeekTime;
-	char szLogLine[1024] = "";
+
+	SYS_OFF_T llCurrOffset = 0, llBaseOffset = (SYS_OFF_T) -1, llEndOffset;
+	unsigned long ulPeekTime;
+	char szLogLine[1024];
 
 	for (;;) {
-		ulCurrOffset = (unsigned long) ftell(pLogFile);
+		llCurrOffset = Sys_ftell(pLogFile);
 
 		if (MscFGets(szLogLine, sizeof(szLogLine) - 1, pLogFile) == NULL)
 			break;
 		if (sscanf(szLogLine, "[PeekTime] %lu", &ulPeekTime) == 1)
-			ulBaseOffset = ulCurrOffset;
+			llBaseOffset = llCurrOffset;
 	}
-	if (ulBaseOffset == (unsigned long) -1) {
+	if (llBaseOffset == (SYS_OFF_T) -1) {
 		fclose(pLogFile);
 		ErrSetErrorCode(ERR_EMPTY_LOG, pszLogFilePath);
 		return NULL;
 	}
-	/* Get end offset ( end of file ) */
-	fseek(pLogFile, 0, SEEK_END);
-	ulEndOffset = (unsigned long) ftell(pLogFile);
+	/* Get end offset (end of file) */
+	Sys_fseek(pLogFile, 0, SEEK_END);
+	llEndOffset = Sys_ftell(pLogFile);
 
 	/* Load last entry */
-	unsigned int uEntrySize = (unsigned int) (ulEndOffset - ulBaseOffset);
+	unsigned int uEntrySize = (unsigned int) (llEndOffset - llBaseOffset);
 	char *pszEntry = (char *) SysAlloc(uEntrySize + 1);
 
 	if (pszEntry == NULL) {
 		fclose(pLogFile);
 		return NULL;
 	}
-	fseek(pLogFile, ulBaseOffset, SEEK_SET);
+	Sys_fseek(pLogFile, llBaseOffset, SEEK_SET);
 	if (!fread(pszEntry, uEntrySize, 1, pLogFile)) {
 		SysFree(pszEntry);
 		fclose(pLogFile);
@@ -456,39 +534,10 @@ char *QueLoadLastLogEntry(char const *pszLogFilePath)
 	return pszEntry;
 }
 
-static int QueLoadMessageStat(MessageQueue *pMQ, QueueMessage *pQM)
-{
-	/* Build the slog file path */
-	char szSlogFilePath[SYS_MAX_PATH] = "";
-
-	QueGetFilePath(pMQ, pQM, szSlogFilePath, QUEUE_SLOG_DIR);
-
-	/* Try to load message statistics */
-	FILE *pLogFile = fopen(szSlogFilePath, "rt");
-
-	if (pLogFile != NULL) {
-		int iNumTries = 0;
-		unsigned long ulLastTime = 0;
-		unsigned long ulPeekTime;
-		char szLogLine[1024] = "";
-
-		while (MscFGets(szLogLine, sizeof(szLogLine) - 1, pLogFile) != NULL)
-			if (sscanf(szLogLine, "[PeekTime] %lu", &ulPeekTime) == 1)
-				++iNumTries, ulLastTime = ulPeekTime;
-
-		fclose(pLogFile);
-
-		pQM->iNumTries = iNumTries;
-		pQM->tLastTry = (time_t) ulLastTime;
-	}
-
-	return 0;
-}
-
 static int QueStatMessage(MessageQueue *pMQ, QueueMessage *pQM)
 {
 	/* Build the slog file path */
-	char szSlogFilePath[SYS_MAX_PATH] = "";
+	char szSlogFilePath[SYS_MAX_PATH];
 
 	QueGetFilePath(pMQ, pQM, szSlogFilePath, QUEUE_SLOG_DIR);
 
@@ -500,7 +549,7 @@ static int QueStatMessage(MessageQueue *pMQ, QueueMessage *pQM)
 	}
 	/* Dump peek time */
 	time_t tCurr = time(NULL);
-	char szTime[128] = "";
+	char szTime[128];
 
 	MscGetTimeStr(szTime, sizeof(szTime) - 1, tCurr);
 	fprintf(pLogFile, "[PeekTime] %lu : %s\n", (unsigned long) tCurr, szTime);
@@ -513,24 +562,21 @@ QMSG_HANDLE QueCreateMessage(QUEUE_HANDLE hQueue)
 {
 	MessageQueue *pMQ = (MessageQueue *) hQueue;
 
-	/* Initialize random number generator */
-	SRand();
-
 	/* Build message file path */
 	int iLevel1 = rand() % pMQ->iNumDirsLevel;
 	int iLevel2 = rand() % pMQ->iNumDirsLevel;
-	char szSubPath[SYS_MAX_PATH] = "";
-	char szMsgFilePath[SYS_MAX_PATH] = "";
+	char szSubPath[SYS_MAX_PATH];
+	char szMsgFilePath[SYS_MAX_PATH];
 
 	SysSNPrintf(szSubPath, sizeof(szSubPath) - 1, "%s%d%s%d%s%s",
 		    pMQ->pszRootPath, iLevel1, SYS_SLASH_STR,
 		    iLevel2, SYS_SLASH_STR, QUEUE_TEMP_DIR);
 
-	if (MscUniqueFile(szSubPath, szMsgFilePath) < 0)
+	if (MscUniqueFile(szSubPath, szMsgFilePath, sizeof(szMsgFilePath)) < 0)
 		return INVALID_QMSG_HANDLE;
 
 	/* Extract file name */
-	char szMsgFileName[SYS_MAX_PATH] = "";
+	char szMsgFileName[SYS_MAX_PATH];
 
 	MscGetFileName(szMsgFilePath, szMsgFileName);
 
@@ -566,6 +612,52 @@ int QueGetFilePath(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage, char *pszFilePath,
 	return QueGetFilePath(pMQ, pQM, pszFilePath, pszQueueDir);
 }
 
+static int QueDoMessageCleanup(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage)
+{
+	MessageQueue *pMQ = (MessageQueue *) hQueue;
+	QueueMessage *pQM = (QueueMessage *) hMessage;
+	char szQueueFilePath[SYS_MAX_PATH];
+
+	if (pQM->ulFlags & QUMF_FREEZE) {
+		/* Move message file */
+		char szTargetFile[SYS_MAX_PATH];
+
+		QueGetFilePath(pMQ, pQM, szQueueFilePath);
+		QueGetFilePath(pMQ, pQM, szTargetFile, QUEUE_FROZ_DIR);
+		if (SysMoveFile(szQueueFilePath, szTargetFile) < 0)
+			return ErrGetErrorCode();
+
+		/* Change message location */
+		pQM->pszQueueDir = QUEUE_FROZ_DIR;
+	} else {
+		/* Clean message file */
+		QueGetFilePath(pMQ, pQM, szQueueFilePath);
+		SysRemove(szQueueFilePath);
+
+		/* Clean 'info' file */
+		QueGetFilePath(pMQ, pQM, szQueueFilePath, QUEUE_INFO_DIR);
+		SysRemove(szQueueFilePath);
+
+		/* Clean 'slog' file */
+		QueGetFilePath(pMQ, pQM, szQueueFilePath, QUEUE_SLOG_DIR);
+		SysRemove(szQueueFilePath);
+	}
+
+	/* Clean 'temp' file */
+	QueGetFilePath(pMQ, pQM, szQueueFilePath, QUEUE_TEMP_DIR);
+	SysRemove(szQueueFilePath);
+
+	/* Clean 'cust' file */
+	QueGetFilePath(pMQ, pQM, szQueueFilePath, QUEUE_CUST_DIR);
+	SysRemove(szQueueFilePath);
+
+	/* Clean 'mprc' file */
+	QueGetFilePath(pMQ, pQM, szQueueFilePath, QUEUE_MPRC_DIR);
+	SysRemove(szQueueFilePath);
+
+	return 0;
+}
+
 int QueCloseMessage(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage)
 {
 	MessageQueue *pMQ = (MessageQueue *) hQueue;
@@ -648,7 +740,7 @@ int QueInitMessageStats(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage)
 {
 	MessageQueue *pMQ = (MessageQueue *) hQueue;
 	QueueMessage *pQM = (QueueMessage *) hMessage;
-	char szQueueFilePath[SYS_MAX_PATH] = "";
+	char szQueueFilePath[SYS_MAX_PATH];
 
 	QueGetFilePath(pMQ, pQM, szQueueFilePath, QUEUE_SLOG_DIR);
 	SysRemove(szQueueFilePath);
@@ -660,52 +752,6 @@ int QueInitMessageStats(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage)
 	return 0;
 }
 
-static int QueDoMessageCleanup(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage)
-{
-	MessageQueue *pMQ = (MessageQueue *) hQueue;
-	QueueMessage *pQM = (QueueMessage *) hMessage;
-	char szQueueFilePath[SYS_MAX_PATH] = "";
-
-	if (pQM->ulFlags & QUMF_FREEZE) {
-		/* Move message file */
-		char szTargetFile[SYS_MAX_PATH] = "";
-
-		QueGetFilePath(pMQ, pQM, szQueueFilePath);
-		QueGetFilePath(pMQ, pQM, szTargetFile, QUEUE_FROZ_DIR);
-		if (SysMoveFile(szQueueFilePath, szTargetFile) < 0)
-			return ErrGetErrorCode();
-
-		/* Change message location */
-		pQM->pszQueueDir = QUEUE_FROZ_DIR;
-	} else {
-		/* Clean message file */
-		QueGetFilePath(pMQ, pQM, szQueueFilePath);
-		SysRemove(szQueueFilePath);
-
-		/* Clean 'info' file */
-		QueGetFilePath(pMQ, pQM, szQueueFilePath, QUEUE_INFO_DIR);
-		SysRemove(szQueueFilePath);
-
-		/* Clean 'slog' file */
-		QueGetFilePath(pMQ, pQM, szQueueFilePath, QUEUE_SLOG_DIR);
-		SysRemove(szQueueFilePath);
-	}
-
-	/* Clean 'temp' file */
-	QueGetFilePath(pMQ, pQM, szQueueFilePath, QUEUE_TEMP_DIR);
-	SysRemove(szQueueFilePath);
-
-	/* Clean 'cust' file */
-	QueGetFilePath(pMQ, pQM, szQueueFilePath, QUEUE_CUST_DIR);
-	SysRemove(szQueueFilePath);
-
-	/* Clean 'mprc' file */
-	QueGetFilePath(pMQ, pQM, szQueueFilePath, QUEUE_MPRC_DIR);
-	SysRemove(szQueueFilePath);
-
-	return 0;
-}
-
 int QueCleanupMessage(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage, bool bFreeze)
 {
 	MessageQueue *pMQ = (MessageQueue *) hQueue;
@@ -718,15 +764,29 @@ int QueCleanupMessage(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage, bool bFreeze)
 	return 0;
 }
 
+static int QueAddNew(MessageQueue *pMQ, QueueMessage *pQM)
+{
+	/* Add the queue entry */
+	if (SysLockMutex(pMQ->hMutex, SYS_INFINITE_TIMEOUT) < 0)
+		return ErrGetErrorCode();
+
+	SYS_LIST_ADDT(&pQM->LLink, &pMQ->ReadyQueue);
+	++pMQ->iReadyCount;
+	SysSetEvent(pMQ->hReadyEvent);
+	SysUnlockMutex(pMQ->hMutex);
+
+	return 0;
+}
+
 int QueCommitMessage(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage)
 {
 	MessageQueue *pMQ = (MessageQueue *) hQueue;
 	QueueMessage *pQM = (QueueMessage *) hMessage;
 
-	/* Move message file ( if not in mess ) */
+	/* Move message file (if not in mess) */
 	if (strcmp(pQM->pszQueueDir, QUEUE_MESS_DIR) != 0) {
-		char szSourceFile[SYS_MAX_PATH] = "";
-		char szTargetFile[SYS_MAX_PATH] = "";
+		char szSourceFile[SYS_MAX_PATH];
+		char szTargetFile[SYS_MAX_PATH];
 
 		QueGetFilePath(pMQ, pQM, szSourceFile);
 		QueGetFilePath(pMQ, pQM, szTargetFile, QUEUE_MESS_DIR);
@@ -747,43 +807,22 @@ int QueCommitMessage(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage)
 	return 0;
 }
 
-static int QueAddNew(MessageQueue *pMQ, QueueMessage *pQM)
-{
-	/* Add the queue entry */
-	if (SysLockMutex(pMQ->hMutex, SYS_INFINITE_TIMEOUT) < 0)
-		return ErrGetErrorCode();
-
-	SYS_LIST_ADDT(&pQM->LLink, &pMQ->ReadyQueue);
-	++pMQ->iReadyCount;
-	SysSetEvent(pMQ->hReadyEvent);
-	SysUnlockMutex(pMQ->hMutex);
-
-	return 0;
-}
-
 static bool QueMessageExpired(MessageQueue *pMQ, QueueMessage *pQM)
 {
 	return pQM->iNumTries >= pMQ->iMaxRetry;
 }
 
-static time_t QueNextRetryOp(int iNumTries, unsigned int uRetryTimeout,
-			     unsigned int uRetryIncrRatio)
+static int QueAddRsnd(MessageQueue *pMQ, QueueMessage *pQM)
 {
-	unsigned int uNextOp = uRetryTimeout;
+	/* Add the queue entry */
+	if (SysLockMutex(pMQ->hMutex, SYS_INFINITE_TIMEOUT) < 0)
+		return ErrGetErrorCode();
 
-	if (uRetryIncrRatio != 0)
-		for (int i = 1; i < iNumTries; i++)
-			uNextOp += uNextOp / uRetryIncrRatio;
-
-	return (time_t) uNextOp;
-}
-
-static bool QueMessageReadyToSend(MessageQueue *pMQ, QueueMessage *pQM)
-{
-	return (time(NULL) > (pQM->tLastTry +
-			      QueNextRetryOp(pQM->iNumTries, (unsigned int) pMQ->iRetryTimeout,
-					     (unsigned int) pMQ->iRetryIncrRatio)));
+	SYS_LIST_ADDT(&pQM->LLink, &pMQ->RsndArenaQueue);
+	++pMQ->iRsndArenaCount;
+	SysUnlockMutex(pMQ->hMutex);
 
+	return 0;
 }
 
 int QueResendMessage(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage)
@@ -796,10 +835,10 @@ int QueResendMessage(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage)
 		ErrSetErrorCode(ERR_SPOOL_FILE_EXPIRED);
 		return ERR_SPOOL_FILE_EXPIRED;
 	}
-	/* Move message file ( if not in rsnd dir ) */
+	/* Move message file (if not in rsnd dir) */
 	if (strcmp(pQM->pszQueueDir, QUEUE_RSND_DIR) != 0) {
-		char szSourceFile[SYS_MAX_PATH] = "";
-		char szTargetFile[SYS_MAX_PATH] = "";
+		char szSourceFile[SYS_MAX_PATH];
+		char szTargetFile[SYS_MAX_PATH];
 
 		QueGetFilePath(pMQ, pQM, szSourceFile);
 		QueGetFilePath(pMQ, pQM, szTargetFile, QUEUE_RSND_DIR);
@@ -820,19 +859,6 @@ int QueResendMessage(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage)
 	return 0;
 }
 
-static int QueAddRsnd(MessageQueue *pMQ, QueueMessage *pQM)
-{
-	/* Add the queue entry */
-	if (SysLockMutex(pMQ->hMutex, SYS_INFINITE_TIMEOUT) < 0)
-		return ErrGetErrorCode();
-
-	SYS_LIST_ADDT(&pQM->LLink, &pMQ->RsndArenaQueue);
-	++pMQ->iRsndArenaCount;
-	SysUnlockMutex(pMQ->hMutex);
-
-	return 0;
-}
-
 QMSG_HANDLE QueExtractMessage(QUEUE_HANDLE hQueue, int iTimeout)
 {
 	MessageQueue *pMQ = (MessageQueue *) hQueue;
@@ -873,65 +899,11 @@ QMSG_HANDLE QueExtractMessage(QUEUE_HANDLE hQueue, int iTimeout)
 	return (QMSG_HANDLE) pQM;
 }
 
-static unsigned int QueRsndThread(void *pThreadData)
-{
-	MessageQueue *pMQ = (MessageQueue *) pThreadData;
-	int iElapsedTime = 0;
-
-	while ((pMQ->ulFlags & QUEF_SHUTDOWN) == 0) {
-		SysSleep(QUE_ARENA_SCAN_WAIT);
-
-		iElapsedTime += QUE_ARENA_SCAN_WAIT;
-		if (iElapsedTime > QUE_ARENA_SCAN_INTERVAL) {
-			iElapsedTime = 0;
-
-			/* Scan rsnd arena to prepare messages to resend */
-			QueScanRsndArena(pMQ);
-		}
-	}
-	pMQ->ulFlags &= ~QUEF_SHUTDOWN;
-
-	return 0;
-}
-
-static int QueScanRsndArena(MessageQueue *pMQ)
-{
-	if (SysLockMutex(pMQ->hMutex, SYS_INFINITE_TIMEOUT) < 0)
-		return ErrGetErrorCode();
-
-	SysListHead *pLLink;
-
-	SYS_LIST_FOR_EACH(pLLink, &pMQ->RsndArenaQueue) {
-		QueueMessage *pQM = SYS_LIST_ENTRY(pLLink, QueueMessage, LLink);
-
-		if (QueMessageReadyToSend(pMQ, pQM)) {
-			/* Set the list pointer to the next item */
-			pLLink = pLLink->pPrev;
-
-			/* Remove item from resend arena */
-			SYS_LIST_DEL(&pQM->LLink);
-			--pMQ->iRsndArenaCount;
-
-			/* Add item from resend queue */
-			SYS_LIST_ADDT(&pQM->LLink, &pMQ->ReadyQueue);
-			++pMQ->iReadyCount;
-		}
-	}
-
-	/* If the count of rsnd queue is not zero, set the event */
-	if (pMQ->iReadyCount > 0)
-		SysSetEvent(pMQ->hReadyEvent);
-
-	SysUnlockMutex(pMQ->hMutex);
-
-	return 0;
-}
-
 int QueCheckMessage(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage)
 {
 	MessageQueue *pMQ = (MessageQueue *) hQueue;
 	QueueMessage *pQM = (QueueMessage *) hMessage;
-	char szQueueFilePath[SYS_MAX_PATH] = "";
+	char szQueueFilePath[SYS_MAX_PATH];
 
 	if (pQM->ulFlags & QUMF_DELETED) {
 		ErrSetErrorCode(ERR_MESSAGE_DELETED);
@@ -956,7 +928,7 @@ static bool QueMessageDestMatch(MessageQueue *pMQ, QueueMessage *pQM,
 				char const *pszAddressMatch)
 {
 	SpoolFileHeader SFH;
-	char szQueueFilePath[SYS_MAX_PATH] = "";
+	char szQueueFilePath[SYS_MAX_PATH];
 
 	QueGetFilePath(pMQ, pQM, szQueueFilePath);
 	if (USmlLoadSpoolFileHeader(szQueueFilePath, SFH) < 0)
@@ -965,9 +937,9 @@ static bool QueMessageDestMatch(MessageQueue *pMQ, QueueMessage *pQM,
 	bool bAddressMatch = false;
 
 	if (strchr(pszAddressMatch, '@') == NULL) {
-		/* RFC style ETRN ( domain based ) */
-		char szDestUser[MAX_ADDR_NAME] = "";
-		char szDestDomain[MAX_ADDR_NAME] = "";
+		/* RFC style ETRN (domain based) */
+		char szDestUser[MAX_ADDR_NAME];
+		char szDestDomain[MAX_ADDR_NAME];
 
 		if (StrStringsCount(SFH.ppszRcpt) < 1 ||
 		    USmtpSplitEmailAddr(SFH.ppszRcpt[0], szDestUser, szDestDomain) < 0) {
@@ -977,7 +949,7 @@ static bool QueMessageDestMatch(MessageQueue *pMQ, QueueMessage *pQM,
 
 		bAddressMatch = (StrIWildMatch(szDestDomain, pszAddressMatch) != 0);
 	} else {
-		/* XMail style ETRN ( email based ) */
+		/* XMail style ETRN (email based) */
 		bAddressMatch = (StrIWildMatch(SFH.ppszRcpt[0], pszAddressMatch) != 0);
 	}
 	USmlCleanupSpoolFileHeader(SFH);
