@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -53,8 +53,8 @@ QMSG_HANDLE QueCreateMessage(QUEUE_HANDLE hQueue);
 int QueGetFilePath(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage, char *pszFilePath,
 		   char const *pszQueueDir = NULL);
 int QueCloseMessage(QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage);
-QMSG_HANDLE QueGetHandle(QUEUE_HANDLE hQueue, int iLevel1, int iLevel2, char const *pszQueueDir,
-			 char const *pszFileName);
+QMSG_HANDLE QueGetHandle(QUEUE_HANDLE hQueue, int iLevel1, int iLevel2,
+			 char const *pszQueueDir, char const *pszFileName);
 char const *QueGetFileName(QMSG_HANDLE hMessage);
 char const *QueGetQueueDir(QMSG_HANDLE hMessage);
 int QueGetLevel1(QMSG_HANDLE hMessage);
