@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "ShBlocks.h"
 #include "StrUtils.h"
 #include "SList.h"
+#include "Hash.h"
 #include "MD5.h"
 #include "Base64Enc.h"
 #include "BuffSock.h"
@@ -34,21 +35,23 @@
 #include "UsrUtils.h"
 #include "SvrUtils.h"
 #include "MessQueue.h"
+#include "SSLMisc.h"
 #include "MailSvr.h"
 #include "MiscUtils.h"
 
 #define IPPROP_LINE_MAX             1024
-#define SERVICE_ACCEPT_TIMEOUT      4
+#define SERVICE_ACCEPT_TIMEOUT      4000
 #define SERVICE_WAIT_SLEEP          2
 #define MAX_CLIENTS_WAIT            300
+#define MAX_RND_TIME                600
 
 enum IPMapFileds {
 	ipmFromIP = 0,
-		ipmFromMask,
-		ipmAllow,
-		ipmPrecedence,
+	ipmFromMask,
+	ipmAllow,
+	ipmPrecedence,
 
-		ipmMax
+	ipmMax
 };
 
 struct FileScan {
@@ -57,12 +60,6 @@ struct FileScan {
 };
 
 
-static int MscCopyFileLL(const char *pszCopyTo, const char *pszCopyFrom,
-			 char const *pszMode);
-static char *MscMacroReplace(char const *pszIn, int *piSize,
-			     char *(*pLkupProc)(void *, char const *, int), void *pPriv);
-
-
 int MscDatumAlloc(Datum *pDm, void const *pData, long lSize)
 {
 	if ((pDm->pData = (char *) SysAllocNZ(lSize + 1)) == NULL)
@@ -113,35 +110,66 @@ void MscFreeDatumList(SysListHead *pHead)
 	}
 }
 
-int MscUniqueFile(char const *pszDir, char *pszFilePath)
+int MscUniqueFile(char const *pszDir, char *pszFilePath, int iMaxPath)
 {
-	/* Get thread ID and host name. We do not use atomic inc on ulUniqSeq, since */
-	/* collision is prevented by the thread ID */
-	static unsigned long ulUniqSeq = 0;
-	unsigned long ulThreadID = SysGetCurrentThreadId();
-	SYS_INT64 iMsTime = SysMsTime();
+	/*
+	 * Get thread ID and host name. We do not use atomic inc on ulUniqSeq, since
+	 * collision is prevented by the thread and process IDs.
+	 */
+	static unsigned long ulUniqSeq;
 	char szHostName[MAX_HOST_NAME] = "";
 
 	gethostname(szHostName, sizeof(szHostName) - 1);
-
-	sprintf(pszFilePath, "%s" SYS_SLASH_STR SYS_LLU_FMT ".%lu.%lx.%s",
-		pszDir, iMsTime, ulThreadID, ulUniqSeq++, szHostName);
+	SysSNPrintf(pszFilePath, iMaxPath,
+		    "%s" SYS_SLASH_STR SYS_LLU_FMT ".%lx.%lx.%lx.%s",
+		    pszDir, SysMsTime(), SysGetCurrentThreadId(),
+		    SysGetCurrentProcessId(), ulUniqSeq++, szHostName);
 
 	return 0;
 }
 
-int MscRecvTextFile(const char *pszFileName, BSOCK_HANDLE hBSock, int iTimeout,
+void MscSafeGetTmpFile(char *pszPath, int iMaxPath)
+{
+	time_t tmNow;
+	unsigned long ulID;
+	SYS_INT64 MsTime;
+	md5_ctx_t MCtx;
+	char szTempDir[SYS_MAX_PATH], szMD5[128];
+	static time_t tmRnd;
+	static unsigned char RndBytes[64];
+
+	if ((tmNow = time(NULL)) > tmRnd + MAX_RND_TIME) {
+		SSLGetRandBytes(RndBytes, sizeof(RndBytes));
+		tmRnd = tmNow;
+	}
+
+	md5_init(&MCtx);
+	MsTime = SysMsTime();
+	md5_update(&MCtx, (unsigned char *) &MsTime, sizeof(MsTime));
+	ulID = SysGetCurrentProcessId();
+	md5_update(&MCtx, (unsigned char *) &ulID, sizeof(ulID));
+	ulID = SysGetCurrentThreadId();
+	md5_update(&MCtx, (unsigned char *) &ulID, sizeof(ulID));
+	ulID = rand();
+	md5_update(&MCtx, (unsigned char *) &ulID, sizeof(ulID));
+	md5_update(&MCtx, RndBytes, sizeof(RndBytes));
+	md5_final(&MCtx);
+	md5_hex(MCtx.digest, szMD5);
+
+	SysGetTempDir(szTempDir, sizeof(szTempDir));
+	SysSNPrintf(pszPath, iMaxPath, "%s%s.xtmp", szTempDir, szMD5);
+}
+
+int MscRecvTextFile(char const *pszFileName, BSOCK_HANDLE hBSock, int iTimeout,
 		    int (*pStopProc) (void *), void *pParam)
 {
 	FILE *pFile = fopen(pszFileName, "wt");
+	char szBuffer[2048];
 
 	if (pFile == NULL) {
 		ErrSetErrorCode(ERR_FILE_CREATE, pszFileName);
 		return ERR_FILE_CREATE;
 	}
-
-	char szBuffer[2048] = "";
-
 	while (BSckGetString(hBSock, szBuffer, sizeof(szBuffer) - 1, iTimeout) != NULL) {
 		if (strcmp(szBuffer, ".") == 0)
 			break;
@@ -160,18 +188,16 @@ int MscRecvTextFile(const char *pszFileName, BSOCK_HANDLE hBSock, int iTimeout,
 	return 0;
 }
 
-int MscSendTextFile(const char *pszFileName, BSOCK_HANDLE hBSock, int iTimeout,
+int MscSendTextFile(char const *pszFileName, BSOCK_HANDLE hBSock, int iTimeout,
 		    int (*pStopProc) (void *), void *pParam)
 {
 	FILE *pFile = fopen(pszFileName, "rt");
+	char szBuffer[2048];
 
 	if (pFile == NULL) {
 		ErrSetErrorCode(ERR_FILE_OPEN, pszFileName);
 		return ERR_FILE_OPEN;
 	}
-
-	char szBuffer[2048] = "";
-
 	while (MscFGets(szBuffer, sizeof(szBuffer) - 1, pFile) != NULL) {
 		if (szBuffer[0] == '.')
 			for (int i = strlen(szBuffer); i >= 0; i--)
@@ -186,11 +212,50 @@ int MscSendTextFile(const char *pszFileName, BSOCK_HANDLE hBSock, int iTimeout,
 			return ErrGetErrorCode();
 		}
 	}
+	if (ferror(pFile)) {
+		fclose(pFile);
+		ErrSetErrorCode(ERR_FILE_READ, pszFileName);
+		return ERR_FILE_READ;
+	}
 	fclose(pFile);
 
 	return BSckSendString(hBSock, ".", iTimeout);
 }
 
+int MscSendFileCRLF(char const *pszFilePath, BSOCK_HANDLE hBSock, int iTimeout)
+{
+	int iLength;
+	FILE *pFile;
+	char szBuffer[2048];
+
+	if ((pFile = fopen(pszFilePath, "rb")) == NULL) {
+		ErrSetErrorCode(ERR_FILE_OPEN, pszFilePath);
+		return ERR_FILE_OPEN;
+	}
+	while (fgets(szBuffer, sizeof(szBuffer) - 1, pFile) != NULL) {
+		iLength = strlen(szBuffer);
+		if (szBuffer[iLength - 1] == '\r')
+			szBuffer[iLength++] = '\n';
+		else if (szBuffer[iLength - 1] == '\n' &&
+			 (iLength < 2 || szBuffer[iLength - 2] != '\r')) {
+			szBuffer[iLength - 1] = '\r';
+			szBuffer[iLength++] = '\n';
+		}
+		if (BSckSendData(hBSock, szBuffer, iLength, iTimeout) < 0) {
+			fclose(pFile);
+			return ErrGetErrorCode();
+		}
+	}
+	if (ferror(pFile)) {
+		fclose(pFile);
+		ErrSetErrorCode(ERR_FILE_READ, pszFilePath);
+		return ERR_FILE_READ;
+	}
+	fclose(pFile);
+
+	return 0;
+}
+
 char *MscTranslatePath(char *pszPath)
 {
 	for (int i = 0; pszPath[i] != '\0'; i++) {
@@ -207,46 +272,56 @@ char *MscTranslatePath(char *pszPath)
 
 void *MscLoadFile(char const *pszFilePath, unsigned long *pulFileSize)
 {
-	FILE *pFile = fopen(pszFilePath, "rb");
+	size_t FileSize, RdBytes;
+	SYS_OFF_T llFileSize;
+	FILE *pFile;
+	void *pFileData;
 
-	if (pFile == NULL) {
+	if ((pFile = fopen(pszFilePath, "rb")) == NULL) {
 		ErrSetErrorCode(ERR_FILE_OPEN, pszFilePath);
 		return NULL;
 	}
-	fseek(pFile, 0, SEEK_END);
-
-	unsigned long ulFileSize = (unsigned long) ftell(pFile);
+	Sys_fseek(pFile, 0, SEEK_END);
+	llFileSize = Sys_ftell(pFile);
+	if (llFileSize >= (1LL << (CHAR_BIT * sizeof(size_t) - 1))) {
+		fclose(pFile);
+		ErrSetErrorCode(ERR_TOO_BIG, pszFilePath);
+		return NULL;
+	}
+	FileSize = (size_t) llFileSize;
 
 	/*
 	 * Alloc one extra byte to enable placing a '\0' to terminate an eventual
 	 * string representation and to avoid SysAlloc() to fail if ulFileSize == 0
 	 */
-	void *pFileData = SysAlloc(ulFileSize + 1);
-
-	if (pFileData == NULL) {
+	if ((pFileData = SysAlloc(FileSize + 1)) == NULL) {
 		fclose(pFile);
 		return NULL;
 	}
 	rewind(pFile);
-	if (ulFileSize != 0)
-		fread(pFileData, ulFileSize, 1, pFile);
+	RdBytes = fread(pFileData, FileSize, 1, pFile);
 	fclose(pFile);
-	((char *) pFileData)[ulFileSize] = 0;
+	if (RdBytes != FileSize) {
+		SysFree(pFileData);
+		ErrSetErrorCode(ERR_FILE_READ, pszFilePath);
+		return NULL;
+	}
+	((char *) pFileData)[FileSize] = 0;
 	if (pulFileSize != NULL)
-		*pulFileSize = ulFileSize;
+		*pulFileSize = FileSize;
 
 	return pFileData;
 }
 
-int MscLockFile(const char *pszFileName, int iMaxWait, int iWaitStep)
+int MscLockFile(char const *pszFileName, int iMaxWait, int iWaitStep)
 {
 	while (iMaxWait > 0 && SysLockFile(pszFileName) < 0) {
-		SysSleep(iWaitStep);
-
+		if (SysSleep(iWaitStep) < 0)
+			return ErrGetErrorCode();
 		iMaxWait -= iWaitStep;
 	}
 
-	return (iMaxWait > 0) ? 0: SysLockFile(pszFileName);
+	return iMaxWait > 0 ? 0: SysLockFile(pszFileName);
 }
 
 int MscGetTimeNbrString(char *pszTimeStr, int iStringSize, time_t tTime)
@@ -267,21 +342,20 @@ int MscGetTimeNbrString(char *pszTimeStr, int iStringSize, time_t tTime)
 
 int MscGetTime(struct tm &tmLocal, int &iDiffHours, int &iDiffMins, time_t tCurr)
 {
+	time_t tLocal, tGM;
+	struct tm tmTimeLOC, tmTimeGM;
+
 	if (tCurr == 0)
 		time(&tCurr);
-
 	SysLocalTime(&tCurr, &tmLocal);
-
-	struct tm tmTimeLOC = tmLocal;
-	struct tm tmTimeGM;
-
+	tmTimeLOC = tmLocal;
 	SysGMTime(&tCurr, &tmTimeGM);
 
 	tmTimeLOC.tm_isdst = 0;
 	tmTimeGM.tm_isdst = 0;
 
-	time_t tLocal = mktime(&tmTimeLOC);
-	time_t tGM = mktime(&tmTimeGM);
+	tLocal = mktime(&tmTimeLOC);
+	tGM = mktime(&tmTimeGM);
 
 	int iSecsDiff = (int) difftime(tLocal, tGM);
 	int iSignDiff = Sign(iSecsDiff);
@@ -295,9 +369,12 @@ int MscGetTime(struct tm &tmLocal, int &iDiffHours, int &iDiffMins, time_t tCurr
 
 char *MscStrftime(struct tm const *ptmTime, char *pszDateStr, int iSize)
 {
-	const char *pszWDays[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
-	const char *pszMonths[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
-			"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
+	static char const * const pszWDays[] = {
+		"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
+	};
+	static char const * const pszMonths[] = {
+		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
+		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
 	};
 
 	SysSNPrintf(pszDateStr, iSize, "%s, %d %s %d %02d:%02d:%02d",
@@ -313,16 +390,13 @@ int MscGetTimeStr(char *pszTimeStr, int iStringSize, time_t tCurr)
 	int iDiffHours = 0;
 	int iDiffMins = 0;
 	struct tm tmTime;
+	char szDiffTime[128];
 
 	MscGetTime(tmTime, iDiffHours, iDiffMins, tCurr);
-
-	char szDiffTime[128] = "";
-
 	if (iDiffHours > 0)
 		sprintf(szDiffTime, " +%02d%02d", iDiffHours, iDiffMins);
 	else
 		sprintf(szDiffTime, " -%02d%02d", -iDiffHours, iDiffMins);
-
 	MscStrftime(&tmTime, pszTimeStr, iStringSize - strlen(szDiffTime) - 1);
 	strcat(pszTimeStr, szDiffTime);
 
@@ -332,7 +406,7 @@ int MscGetTimeStr(char *pszTimeStr, int iStringSize, time_t tCurr)
 int MscGetDirectorySize(char const *pszPath, bool bRecurse, SYS_OFF_T &llDirSize,
 			unsigned long &ulNumFiles, int (*pFNValidate) (char const *))
 {
-	char szFileName[SYS_MAX_PATH] = "";
+	char szFileName[SYS_MAX_PATH];
 	SYS_HANDLE hFind = SysFirstFile(pszPath, szFileName, sizeof(szFileName));
 
 	if (hFind == SYS_INVALID_HANDLE)
@@ -343,7 +417,7 @@ int MscGetDirectorySize(char const *pszPath, bool bRecurse, SYS_OFF_T &llDirSize
 			if (bRecurse && SYS_IS_VALID_FILENAME(szFileName)) {
 				SYS_OFF_T llSubDirSize = 0;
 				unsigned long ulSubNumFiles = 0;
-				char szSubPath[SYS_MAX_PATH] = "";
+				char szSubPath[SYS_MAX_PATH];
 
 				StrSNCpy(szSubPath, pszPath);
 				AppendSlash(szSubPath);
@@ -370,9 +444,10 @@ int MscGetDirectorySize(char const *pszPath, bool bRecurse, SYS_OFF_T &llDirSize
 
 FSCAN_HANDLE MscFirstFile(char const *pszPath, int iListDirs, char *pszFileName, int iSize)
 {
-	FileScan *pFS = (FileScan *) SysAlloc(sizeof(FileScan));
+	FileScan *pFS;
+	LstDatum *pLDm;
 
-	if (pFS == NULL)
+	if ((pFS = (FileScan *) SysAlloc(sizeof(FileScan))) == NULL)
 		return INVALID_FSCAN_HANDLE;
 
 	SYS_INIT_LIST_HEAD(&pFS->FList);
@@ -385,8 +460,7 @@ FSCAN_HANDLE MscFirstFile(char const *pszPath, int iListDirs, char *pszFileName,
 		SysFree(pFS);
 		return INVALID_FSCAN_HANDLE;
 	}
-
-	LstDatum *pLDm = SYS_LIST_ENTRY(pFS->pPos, LstDatum, LLnk);
+	pLDm = SYS_LIST_ENTRY(pFS->pPos, LstDatum, LLnk);
 
 	pFS->pPos = SYS_LIST_NEXT(pFS->pPos, &pFS->FList);
 	StrNCpy(pszFileName, pLDm->Data.pData, iSize);
@@ -397,11 +471,11 @@ FSCAN_HANDLE MscFirstFile(char const *pszPath, int iListDirs, char *pszFileName,
 int MscNextFile(FSCAN_HANDLE hFileScan, char *pszFileName, int iSize)
 {
 	FileScan *pFS = (FileScan *) hFileScan;
+	LstDatum *pLDm;
 
 	if (pFS->pPos == NULL)
 		return 0;
-
-	LstDatum *pLDm = SYS_LIST_ENTRY(pFS->pPos, LstDatum, LLnk);
+	pLDm = SYS_LIST_ENTRY(pFS->pPos, LstDatum, LLnk);
 
 	pFS->pPos = SYS_LIST_NEXT(pFS->pPos, &pFS->FList);
 	StrNCpy(pszFileName, pLDm->Data.pData, iSize);
@@ -419,7 +493,7 @@ void MscCloseFindFile(FSCAN_HANDLE hFileScan)
 
 int MscGetFileList(char const *pszPath, int iListDirs, SysListHead *pHead)
 {
-	char szFileName[SYS_MAX_PATH] = "";
+	char szFileName[SYS_MAX_PATH];
 	SYS_HANDLE hFind = SysFirstFile(pszPath, szFileName, sizeof(szFileName));
 
 	SYS_INIT_LIST_HEAD(pHead);
@@ -437,12 +511,12 @@ int MscGetFileList(char const *pszPath, int iListDirs, SysListHead *pHead)
 	return 0;
 }
 
-int MscCreateEmptyFile(const char *pszFileName)
+int MscCreateEmptyFile(char const *pszFileName)
 {
 	FILE *pFile = fopen(pszFileName, "wb");
 
 	if (pFile == NULL) {
-		ErrSetErrorCode(ERR_FILE_CREATE);
+		ErrSetErrorCode(ERR_FILE_CREATE, pszFileName);
 		return ERR_FILE_CREATE;
 	}
 	fclose(pFile);
@@ -450,19 +524,21 @@ int MscCreateEmptyFile(const char *pszFileName)
 	return 0;
 }
 
-int MscClearDirectory(const char *pszPath, int iRecurseSubs)
+int MscClearDirectory(char const *pszPath, int iRecurseSubs)
 {
-	char szFileName[SYS_MAX_PATH] = "";
+	SYS_HANDLE hFind;
+	SysListHead *pPos;
 	SysListHead FList;
-	SYS_HANDLE hFind = SysFirstFile(pszPath, szFileName, sizeof(szFileName));
+	char szFileName[SYS_MAX_PATH];
 
-	if (hFind == SYS_INVALID_HANDLE)
+	if ((hFind = SysFirstFile(pszPath, szFileName,
+				  sizeof(szFileName))) == SYS_INVALID_HANDLE)
 		return 0;
 	SYS_INIT_LIST_HEAD(&FList);
 	do {
 		if (SysIsDirectory(hFind)) {
 			if (iRecurseSubs && SYS_IS_VALID_FILENAME(szFileName)) {
-				char szSubPath[SYS_MAX_PATH] = "";
+				char szSubPath[SYS_MAX_PATH];
 
 				StrSNCpy(szSubPath, pszPath);
 				AppendSlash(szSubPath);
@@ -483,8 +559,6 @@ int MscClearDirectory(const char *pszPath, int iRecurseSubs)
 	} while (SysNextFile(hFind, szFileName, sizeof(szFileName)));
 	SysFindClose(hFind);
 
-	SysListHead *pPos;
-
 	for (pPos = SYS_LIST_FIRST(&FList); pPos != NULL;
 	     pPos = SYS_LIST_NEXT(pPos, &FList)) {
 		LstDatum *pLDm = SYS_LIST_ENTRY(pPos, LstDatum, LLnk);
@@ -502,53 +576,46 @@ int MscClearDirectory(const char *pszPath, int iRecurseSubs)
 	return 0;
 }
 
-static int MscCopyFileLL(const char *pszCopyTo, const char *pszCopyFrom,
+static int MscCopyFileLL(char const *pszCopyTo, char const *pszCopyFrom,
 			 char const *pszMode)
 {
-	FILE *pFileIn = fopen(pszCopyFrom, "rb");
-
-	if (pFileIn == NULL) {
-		ErrSetErrorCode(ERR_FILE_OPEN);
-		return ERR_FILE_OPEN;
-	}
-
-	FILE *pFileOut = fopen(pszCopyTo, pszMode);
-
-	if (pFileOut == NULL) {
-		fclose(pFileIn);
-		ErrSetErrorCode(ERR_FILE_CREATE);
-		return ERR_FILE_CREATE;
-	}
-
-	unsigned int uReaded;
+	size_t RdBytes;
+	FILE *pFileIn, *pFileOut;
 	char szBuffer[2048];
 
+	if ((pFileIn = fopen(pszCopyFrom, "rb")) == NULL) {
+		ErrSetErrorCode(ERR_FILE_OPEN, pszCopyFrom);
+		return ERR_FILE_OPEN;
+	}
+	if ((pFileOut = fopen(pszCopyTo, pszMode)) == NULL) {
+		fclose(pFileIn);
+		ErrSetErrorCode(ERR_FILE_CREATE, pszCopyTo);
+		return ERR_FILE_CREATE;
+	}
 	do {
-		uReaded = fread(szBuffer, 1, sizeof(szBuffer), pFileIn);
-
-		if (uReaded > 0) {
-			if (fwrite(szBuffer, 1, uReaded, pFileOut) != uReaded) {
+		if ((RdBytes = fread(szBuffer, 1, sizeof(szBuffer), pFileIn)) > 0) {
+			if (fwrite(szBuffer, 1, RdBytes, pFileOut) != RdBytes) {
 				fclose(pFileOut);
 				fclose(pFileIn);
 				SysRemove(pszCopyTo);
 
-				ErrSetErrorCode(ERR_FILE_WRITE);
+				ErrSetErrorCode(ERR_FILE_WRITE, pszCopyTo);
 				return ERR_FILE_WRITE;
 			}
 		}
-	} while (uReaded == sizeof(szBuffer));
+	} while (RdBytes == sizeof(szBuffer));
 	fclose(pFileOut);
 	fclose(pFileIn);
 
 	return 0;
 }
 
-int MscCopyFile(const char *pszCopyTo, const char *pszCopyFrom)
+int MscCopyFile(char const *pszCopyTo, char const *pszCopyFrom)
 {
 	return MscCopyFileLL(pszCopyTo, pszCopyFrom, "wb");
 }
 
-int MscAppendFile(const char *pszCopyTo, const char *pszCopyFrom)
+int MscAppendFile(char const *pszCopyTo, char const *pszCopyFrom)
 {
 	return MscCopyFileLL(pszCopyTo, pszCopyFrom, "a+b");
 }
@@ -556,36 +623,29 @@ int MscAppendFile(const char *pszCopyTo, const char *pszCopyFrom)
 int MscCopyFile(FILE *pFileOut, FILE *pFileIn, SYS_OFF_T llBaseOffset,
 		SYS_OFF_T llCopySize)
 {
-	/* Setup copy size and seek start byte */
+	size_t RdBytes, ToRead;
+	SYS_OFF_T llFileSize;
+	char szBuffer[2048];
+
 	if (llBaseOffset == (SYS_OFF_T) -1)
-		llBaseOffset = (SYS_OFF_T) ftell(pFileIn);
-
-	fseek(pFileIn, 0, SEEK_END);
-
-	SYS_OFF_T llFileSize = (SYS_OFF_T) ftell(pFileIn);
-
+		llBaseOffset = Sys_ftell(pFileIn);
+	Sys_fseek(pFileIn, 0, SEEK_END);
+	llFileSize = Sys_ftell(pFileIn);
 	if (llCopySize == (SYS_OFF_T) -1)
 		llCopySize = llFileSize - llBaseOffset;
 	else
 		llCopySize = Min(llCopySize, llFileSize - llBaseOffset);
-
-	fseek(pFileIn, (long) llBaseOffset, SEEK_SET);
-
-	/* Copy bytes */
-	char szBuffer[2048];
-
+	Sys_fseek(pFileIn, llBaseOffset, SEEK_SET);
 	while (llCopySize > 0) {
-		unsigned int uToRead = (unsigned int) Min(llCopySize, sizeof(szBuffer));
-		unsigned int uReaded = fread(szBuffer, 1, uToRead, pFileIn);
-
-		if (uReaded > 0) {
-			if (fwrite(szBuffer, 1, uReaded, pFileOut) != uReaded) {
+		ToRead = (size_t) Min(llCopySize, sizeof(szBuffer));
+		if ((RdBytes = fread(szBuffer, 1, ToRead, pFileIn)) > 0) {
+			if (fwrite(szBuffer, 1, RdBytes, pFileOut) != RdBytes) {
 				ErrSetErrorCode(ERR_FILE_WRITE);
 				return ERR_FILE_WRITE;
 			}
-			llCopySize -= uReaded;
+			llCopySize -= RdBytes;
 		}
-		if (uReaded != uToRead) {
+		if (RdBytes != ToRead) {
 			ErrSetErrorCode(ERR_FILE_READ);
 			return ERR_FILE_READ;
 		}
@@ -594,6 +654,28 @@ int MscCopyFile(FILE *pFileOut, FILE *pFileIn, SYS_OFF_T llBaseOffset,
 	return 0;
 }
 
+int MscDos2UnixFile(FILE *pFileOut, FILE *pFileIn)
+{
+	int iLength;
+	char szBuffer[2048];
+
+	while (fgets(szBuffer, sizeof(szBuffer), pFileIn) != NULL) {
+		iLength = strlen(szBuffer);
+		if (szBuffer[iLength - 1] == '\r')
+			szBuffer[iLength - 1] = '\n';
+		if (fwrite(szBuffer, 1, iLength, pFileOut) != iLength) {
+			ErrSetErrorCode(ERR_FILE_WRITE);
+			return ERR_FILE_WRITE;
+		}
+	}
+	if (ferror(pFileIn)) {
+		ErrSetErrorCode(ERR_FILE_READ);
+		return ERR_FILE_READ;
+	}
+
+	return 0;
+}
+
 int MscMoveFile(char const *pszOldName, char const *pszNewName)
 {
 	if (MscCopyFile(pszNewName, pszOldName) < 0)
@@ -602,15 +684,26 @@ int MscMoveFile(char const *pszOldName, char const *pszNewName)
 	return SysRemove(pszOldName);
 }
 
-char *MscGetString(FILE *pFile, char *pszBuffer, int iMaxChars)
+char *MscGetString(FILE *pFile, char *pszBuffer, int iMaxChars, int *piGotNL)
 {
-	return (fgets(pszBuffer, iMaxChars, pFile) != NULL) ?
-		StrRTrim(pszBuffer, "\r\n"): NULL;
+	int iLength;
+
+	if (fgets(pszBuffer, iMaxChars, pFile) == NULL)
+		return NULL;
+	iLength = strlen(pszBuffer);
+	if (piGotNL != NULL)
+		*piGotNL = (iLength > 0 &&
+			    strchr("\r\n", pszBuffer[iLength - 1]) != NULL);
+	for (; iLength > 0 && strchr("\r\n", pszBuffer[iLength - 1]) != NULL;
+	     iLength--);
+	pszBuffer[iLength] = '\0';
+
+	return pszBuffer;
 }
 
 char *MscFGets(char *pszLine, int iLineSize, FILE *pFile)
 {
-	return MscGetString(pFile, pszLine, iLineSize);
+	return MscGetString(pFile, pszLine, iLineSize, NULL);
 }
 
 char *MscGetConfigLine(char *pszLine, int iLineSize, FILE *pFile, bool bSkipComments)
@@ -647,7 +740,7 @@ int MscGetSockHost(SYS_SOCKET SockFD, char *pszFQDN, int iSize)
 
 int MscGetServerAddress(char const *pszServer, SYS_INET_ADDR &SvrAddr, int iPortNo)
 {
-	char szServer[MAX_HOST_NAME] = "";
+	char szServer[MAX_HOST_NAME];
 
 	if (MscSplitAddressPort(pszServer, szServer, iPortNo, iPortNo) < 0 ||
 	    SysGetHostByName(szServer, iAddrFamily, SvrAddr) < 0 ||
@@ -657,10 +750,10 @@ int MscGetServerAddress(char const *pszServer, SYS_INET_ADDR &SvrAddr, int iPort
 	return 0;
 }
 
-int MscSplitFQDN(const char *pszFQDN, char *pszHost, int iHSize,
+int MscSplitFQDN(char const *pszFQDN, char *pszHost, int iHSize,
 		 char *pszDomain, int iDSize)
 {
-	const char *pszDot = strchr(pszFQDN, '.');
+	char const *pszDot = strchr(pszFQDN, '.');
 
 	if (pszDot == NULL) {
 		if (pszHost != NULL)
@@ -696,7 +789,7 @@ char *MscLogFilePath(char const *pszLogFile, char *pszLogFilePath)
 	time_t tLogFileTime = (time_t) (NbrFloor((SYS_INT64) tCurrent - lTimeZone + lDayLight,
 						 lRotStep) + lTimeZone - lDayLight);
 	struct tm tmLocTime;
-	char szLogsDir[SYS_MAX_PATH] = "";
+	char szLogsDir[SYS_MAX_PATH];
 
 	SysLocalTime(&tLogFileTime, &tmLocTime);
 
@@ -713,25 +806,18 @@ char *MscLogFilePath(char const *pszLogFile, char *pszLogFilePath)
 
 int MscFileLog(char const *pszLogFile, char const *pszFormat, ...)
 {
-	char szLogFilePath[SYS_MAX_PATH] = "";
+	FILE *pLogFile;
+	va_list Args;
+	char szLogFilePath[SYS_MAX_PATH];
 
 	MscLogFilePath(pszLogFile, szLogFilePath);
-
-	FILE *pLogFile = fopen(szLogFilePath, "a+t");
-
-	if (pLogFile == NULL) {
+	if ((pLogFile = fopen(szLogFilePath, "a+t")) == NULL) {
 		ErrSetErrorCode(ERR_FILE_OPEN);
 		return ERR_FILE_OPEN;
 	}
-
-	va_list Args;
-
 	va_start(Args, pszFormat);
-
 	vfprintf(pLogFile, pszFormat, Args);
-
 	va_end(Args);
-
 	fclose(pLogFile);
 
 	return 0;
@@ -741,7 +827,7 @@ int MscSplitPath(char const *pszFilePath, char *pszDir, int iDSize,
 		 char *pszFName, int iFSize, char *pszExt, int iESize)
 {
 	char const *pszSlash = strrchr(pszFilePath, SYS_SLASH_CHAR);
-	char const *pszFile = NULL;
+	char const *pszFile = NULL, *pszDot;
 
 	if (pszSlash != NULL) {
 		pszFile = pszSlash + 1;
@@ -757,10 +843,7 @@ int MscSplitPath(char const *pszFilePath, char *pszDir, int iDSize,
 		if (pszDir != NULL)
 			SetEmptyString(pszDir);
 	}
-
-	char const *pszDot = strrchr(pszFile, '.');
-
-	if (pszDot != NULL) {
+	if ((pszDot = strrchr(pszFile, '.')) != NULL) {
 		if (pszFName != NULL) {
 			int iNameLength = (int) (pszDot - pszFile);
 
@@ -883,9 +966,9 @@ int MscCreateServerSockets(int iNumAddr, SYS_INET_ADDR const *pSvrAddr, int iFam
 
 int MscGetMaxSockFD(SYS_SOCKET const *pSockFDs, int iNumSockFDs)
 {
-	int iMaxFD = 0;
+	int i, iMaxFD = 0;
 
-	for (int i = 0; i < iNumSockFDs; i++)
+	for (i = 0; i < iNumSockFDs; i++)
 		if (iMaxFD < (int) pSockFDs[i])
 			iMaxFD = (int) pSockFDs[i];
 
@@ -911,7 +994,6 @@ int MscAcceptServerConnection(SYS_SOCKET const *pSockFDs, int iNumSockFDs,
 		return ErrGetErrorCode();
 
 	iNumConnSockFD = 0;
-
 	for (i = 0; i < iNumSockFDs; i++) {
 		if (SYS_FD_ISSET(pSockFDs[i], &fdReadSet)) {
 			SYS_INET_ADDR ConnAddr;
@@ -942,17 +1024,18 @@ int MscLoadAddressFilter(char const *const *ppszFilter, int iNumTokens, AddressF
 		int iMaskBits = atoi(pszMask + 1);
 		char szFilter[128];
 
-		iAddrLength = Min(iAddrLength, sizeof(szFilter) - 1);
+		iAddrLength = Min(iAddrLength, (int) sizeof(szFilter) - 1);
 		Cpy2Sz(szFilter, ppszFilter[0], iAddrLength);
 		if (SysGetHostByName(szFilter, -1, AF.Addr) < 0)
 			return ErrGetErrorCode();
 
 		ZeroData(AF.Mask);
-		iMaskBits = Min(iMaskBits, 8 * sizeof(AF.Mask));
-		for (i = 0; (i + 8) <= iMaskBits; i += 8)
-			AF.Mask[i / 8] = 0xff;
+		iMaskBits = Min(iMaskBits, CHAR_BIT * (int) sizeof(AF.Mask));
+		for (i = 0; (i + CHAR_BIT) <= iMaskBits; i += CHAR_BIT)
+			AF.Mask[i / CHAR_BIT] = 0xff;
 		if (i < iMaskBits)
-			AF.Mask[i / 8] = (SYS_UINT8) (((1 << (iMaskBits - i)) - 1) << (8 - iMaskBits + i));
+			AF.Mask[i / CHAR_BIT] = (SYS_UINT8)
+				(((1 << (iMaskBits - i)) - 1) << (CHAR_BIT - iMaskBits + i));
 	} else if (iNumTokens > 1) {
 		/*
 		 * This is for the old IPV4 representation. They both must
@@ -963,7 +1046,7 @@ int MscLoadAddressFilter(char const *const *ppszFilter, int iNumTokens, AddressF
 		    (pAData = SysInetAddrData(Mask, &iASize)) == NULL)
 			return ErrGetErrorCode();
 		ZeroData(AF.Mask);
-		memcpy(AF.Mask, pAData, Min(iASize, sizeof(AF.Mask)));
+		memcpy(AF.Mask, pAData, Min(iASize, (int) sizeof(AF.Mask)));
 	} else {
 		ErrSetErrorCode(ERR_INVALID_PARAMETER);
 		return ERR_INVALID_PARAMETER;
@@ -990,7 +1073,7 @@ int MscCheckAllowedIP(char const *pszMapFile, const SYS_INET_ADDR &PeerInfo, boo
 	bool bAllow = bDefault;
 	int iPrecedence = -1, iFieldsCount;
 	AddressFilter AF;
-	char szMapLine[512] = "";
+	char szMapLine[512];
 
 	while (MscGetConfigLine(szMapLine, sizeof(szMapLine) - 1, pMapFile) != NULL) {
 		char **ppszStrings = StrGetTabLineStrings(szMapLine);
@@ -1033,7 +1116,7 @@ char **MscGetIPProperties(char const *pszFileName, const SYS_INET_ADDR *pPeerInf
 
 	int iFieldsCount;
 	AddressFilter AFPeer;
-	char szLine[IPPROP_LINE_MAX] = "";
+	char szLine[IPPROP_LINE_MAX];
 
 	while (MscGetConfigLine(szLine, sizeof(szLine) - 1, pFile) != NULL) {
 		char **ppszTokens = StrGetTabLineStrings(szLine);
@@ -1075,16 +1158,14 @@ int MscHostSubMatch(char const *pszHostName, char const *pszHostMatch)
 
 char **MscGetHNProperties(char const *pszFileName, char const *pszHostName)
 {
-	FILE *pFile = fopen(pszFileName, "rt");
+	int iFieldsCount;
+	FILE *pFile;
+	char szLine[IPPROP_LINE_MAX];
 
-	if (pFile == NULL) {
+	if ((pFile = fopen(pszFileName, "rt")) == NULL) {
 		ErrSetErrorCode(ERR_FILE_OPEN);
 		return NULL;
 	}
-
-	int iFieldsCount;
-	char szLine[IPPROP_LINE_MAX] = "";
-
 	while (MscGetConfigLine(szLine, sizeof(szLine) - 1, pFile) != NULL) {
 		char **ppszTokens = StrGetTabLineStrings(szLine);
 
@@ -1105,12 +1186,12 @@ char **MscGetHNProperties(char const *pszFileName, char const *pszHostName)
 	return NULL;
 }
 
-int MscMD5Authenticate(const char *pszPassword, const char *pszTimeStamp, const char *pszDigest)
+int MscMD5Authenticate(char const *pszPassword, char const *pszTimeStamp, char const *pszDigest)
 {
-	char *pszHash = StrSprint("%s%s", pszTimeStamp, pszPassword);
-	char szMD5[128] = "";
+	char *pszHash;
+	char szMD5[128];
 
-	if (pszHash == NULL)
+	if ((pszHash = StrSprint("%s%s", pszTimeStamp, pszPassword)) == NULL)
 		return ErrGetErrorCode();
 	do_md5_string(pszHash, strlen(pszHash), szMD5);
 	SysFree(pszHash);
@@ -1124,14 +1205,13 @@ int MscMD5Authenticate(const char *pszPassword, const char *pszTimeStamp, const
 
 char *MscExtractServerTimeStamp(char const *pszResponse, char *pszTimeStamp, int iMaxTimeStamp)
 {
+	int iLengthTS;
 	char const *pszStartTS, *pszEndTS;
 
 	if ((pszStartTS = strchr(pszResponse, '<')) == NULL ||
 	    (pszEndTS = strchr(pszStartTS + 1, '>')) == NULL)
 		return NULL;
-
-	int iLengthTS = (int) (pszEndTS - pszStartTS) + 1;
-
+	iLengthTS = (int) (pszEndTS - pszStartTS) + 1;
 	iLengthTS = Min(iLengthTS, iMaxTimeStamp - 1);
 	Cpy2Sz(pszTimeStamp, pszStartTS, iLengthTS);
 
@@ -1142,22 +1222,22 @@ int MscRootedName(char const *pszHostName)
 {
 	char const *pszDot = strrchr(pszHostName, '.');
 
-	return (pszDot == NULL) ? 0: ((strlen(pszDot) == 0) ? 1: 0);
+	return pszDot == NULL ? 0: (strlen(pszDot) == 0 ? 1: 0);
 }
 
 int MscCramMD5(char const *pszSecret, char const *pszChallenge, char *pszDigest)
 {
 	int iLenght = (int) strlen(pszSecret);
-	struct md5_ctx ctx;
-	unsigned char isecret[64];
-	unsigned char osecret[64];
-	unsigned char md5secret[16];
+	md5_ctx_t ctx;
+	unsigned char isecret[64], osecret[64];
+	unsigned char md5secret[MD5_DIGEST_LEN];
 
 	if (iLenght > 64) {
-		md5_init_ctx(&ctx);
-		md5_process_bytes(pszSecret, iLenght, &ctx);
-		md5_finish_ctx(&ctx, md5secret);
+		md5_init(&ctx);
+		md5_update(&ctx, (unsigned char const *) pszSecret, iLenght);
+		md5_final(&ctx);
 
+		memcpy(md5secret, ctx.digest, MD5_DIGEST_LEN);
 		pszSecret = (char const *) md5secret;
 		iLenght = 16;
 	}
@@ -1173,30 +1253,40 @@ int MscCramMD5(char const *pszSecret, char const *pszChallenge, char *pszDigest)
 		osecret[i] ^= 0x5c;
 	}
 
-	md5_init_ctx(&ctx);
-	md5_process_bytes(isecret, 64, &ctx);
-	md5_process_bytes((unsigned char *) pszChallenge, (int) strlen(pszChallenge), &ctx);
-	md5_finish_ctx(&ctx, md5secret);
+	md5_init(&ctx);
+	md5_update(&ctx, isecret, 64);
+	md5_update(&ctx, (unsigned char *) pszChallenge, (int) strlen(pszChallenge));
+	md5_final(&ctx);
+	memcpy(md5secret, ctx.digest, MD5_DIGEST_LEN);
 
-	md5_init_ctx(&ctx);
-	md5_process_bytes(osecret, 64, &ctx);
-	md5_process_bytes(md5secret, 16, &ctx);
-	md5_finish_ctx(&ctx, md5secret);
+	md5_init(&ctx);
+	md5_update(&ctx, osecret, 64);
+	md5_update(&ctx, md5secret, MD5_DIGEST_LEN);
+	md5_final(&ctx);
 
-	md5_hex(md5secret, pszDigest);
+	md5_hex(ctx.digest, pszDigest);
 
 	return 0;
 }
 
+/*
+ * NOTE: This is used by TabIndex.cpp to compute the hash of its indexes.
+ *       If this function gets changed in any way, the TAB_INDEX_CURR_VERSION
+ *       value in TabIndex.cpp must be bumped to reflect a different file format.
+ */
 unsigned long MscHashString(char const *pszBuffer, int iLength, unsigned long ulHashInit)
 {
 	unsigned long ulHashVal = ulHashInit;
 
 	while (iLength > 0) {
 		--iLength;
-		ulHashVal += (ulHashVal << 5);
-		ulHashVal ^= (SYS_UINT32) *pszBuffer++;
+		ulHashVal += *(unsigned char const *) pszBuffer++;
+		ulHashVal += (ulHashVal << 10);
+		ulHashVal ^= (ulHashVal >> 6);
 	}
+	ulHashVal += (ulHashVal << 3);
+	ulHashVal ^= (ulHashVal >> 11);
+	ulHashVal += (ulHashVal << 15);
 
 	return ulHashVal;
 }
@@ -1239,6 +1329,9 @@ SYS_UINT16 MscReadUint16(void const *pData)
 {
 	SYS_UINT16 uValue;
 
+	/*
+	 * Reading possibly unaligned data ...
+	 */
 	memcpy(&uValue, pData, sizeof(uValue));
 
 	return uValue;
@@ -1248,6 +1341,9 @@ SYS_UINT32 MscReadUint32(void const *pData)
 {
 	SYS_UINT32 uValue;
 
+	/*
+	 * Reading possibly unaligned data ...
+	 */
 	memcpy(&uValue, pData, sizeof(uValue));
 
 	return uValue;
@@ -1257,6 +1353,9 @@ SYS_UINT64 MscReadUint64(void const *pData)
 {
 	SYS_UINT64 uValue;
 
+	/*
+	 * Reading possibly unaligned data ...
+	 */
 	memcpy(&uValue, pData, sizeof(uValue));
 
 	return uValue;
@@ -1280,7 +1379,7 @@ void *MscWriteUint64(void *pData, SYS_UINT64 uValue)
 int MscCmdStringCheck(char const *pszString)
 {
 	for (; *pszString != '\0'; pszString++)
-		if (((*pszString < ' ') || (*pszString > '~')) && (*pszString != '\t')) {
+		if (*((unsigned char const *) pszString) > 127) {
 			ErrSetErrorCode(ERR_BAD_CMDSTR_CHARS);
 			return ERR_BAD_CMDSTR_CHARS;
 		}
@@ -1295,7 +1394,6 @@ int MscGetSectionSize(FileSection const *pFS, SYS_OFF_T *pllSize)
 
 		if (SysGetFileInfo(pFS->szFilePath, FI) < 0)
 			return ErrGetErrorCode();
-
 		*pllSize = FI.llSize - pFS->llStartOffset;
 	} else
 		*pllSize = pFS->llEndOffset - pFS->llStartOffset;
@@ -1426,7 +1524,6 @@ int MscSslEnvCB(void *pPrivate, int iID, void const *pData)
 {
 	SslBindEnv *pSslE = (SslBindEnv *) pPrivate;
 
-
 	return 0;
 }
 
@@ -1458,3 +1555,29 @@ void MscSysFreeCB(void *pPrivate, void *pData)
 	SysFree(pData);
 }
 
+void MscRandomizeStringsOrder(char **ppszStrings)
+{
+	int i, iCount = StrStringsCount(ppszStrings);
+
+	for (i = 0; i < iCount - 1; i++) {
+		int iChoice = rand() % (iCount - i);
+		char *pszTmp = ppszStrings[i];
+
+		ppszStrings[i] = ppszStrings[i + iChoice];
+		ppszStrings[i + iChoice] = pszTmp;
+	}
+}
+
+unsigned long MscStringHashCB(void *pPrivate, HashDatum const *pDatum)
+{
+	char const *pszStr = (char const *) pDatum->pData;
+
+	return MscHashString(pszStr, strlen(pszStr));
+}
+
+int MscStringCompareCB(void *pPrivate, HashDatum const *pDatum1,
+		       HashDatum const *pDatum2)
+{
+	return strcmp((char const *) pDatum1->pData, (char const *) pDatum2->pData);
+}
+
