@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,8 +33,8 @@ typedef struct POP3_HANDLE_struct {
 struct MailSyncReport {
 	int iMsgSync;
 	int iMsgErr;
-	unsigned long ulSizeSync;
-	unsigned long ulSizeErr;
+	SYS_OFF_T llSizeSync;
+	SYS_OFF_T llSizeErr;
 };
 
 struct PopLastLoginInfo {
@@ -44,30 +44,32 @@ struct PopLastLoginInfo {
 
 int UPopGetMailboxSize(UserInfo *pUI, SYS_OFF_T &llMBSize, unsigned long &ulNumMessages);
 int UPopCheckMailboxSize(UserInfo *pUI, SYS_OFF_T *pllAvailSpace = NULL);
-int UPopAuthenticateAPOP(const char *pszDomain, const char *pszUsrName,
-			 const char *pszTimeStamp, const char *pszDigest);
-POP3_HANDLE UPopBuildSession(const char *pszDomain, const char *pszUsrName,
-			     const char *pszUsrPass, SYS_INET_ADDR const *pPeerInfo);
+int UPopAuthenticateAPOP(char const *pszDomain, char const *pszUsrName,
+			 char const *pszTimeStamp, char const *pszDigest);
+POP3_HANDLE UPopBuildSession(char const *pszDomain, char const *pszUsrName,
+			     char const *pszUsrPass, SYS_INET_ADDR const *pPeerInfo);
 void UPopReleaseSession(POP3_HANDLE hPOPSession, int iUpdate = 1);
-char *UPopGetUserInfoVar(POP3_HANDLE hPOPSession, const char *pszName,
-			 const char *pszDefault = NULL);
+char *UPopGetUserInfoVar(POP3_HANDLE hPOPSession, char const *pszName,
+			 char const *pszDefault = NULL);
 int UPopGetSessionMsgCurrent(POP3_HANDLE hPOPSession);
 int UPopGetSessionMsgTotal(POP3_HANDLE hPOPSession);
-unsigned long UPopGetSessionMBSize(POP3_HANDLE hPOPSession);
+SYS_OFF_T UPopGetSessionMBSize(POP3_HANDLE hPOPSession);
 int UPopGetSessionLastAccessed(POP3_HANDLE hPOPSession);
-int UPopGetMessageSize(POP3_HANDLE hPOPSession, int iMsgIndex, unsigned long &ulMessageSize);
-int UPopGetMessageUIDL(POP3_HANDLE hPOPSession, int iMsgIndex, char *pszMessageUIDL);
+int UPopGetMessageSize(POP3_HANDLE hPOPSession, int iMsgIndex, SYS_OFF_T &llMessageSize);
+int UPopGetMessageUIDL(POP3_HANDLE hPOPSession, int iMsgIndex, char *pszMessageUIDL,
+		       int iSize);
 int UPopDeleteMessage(POP3_HANDLE hPOPSession, int iMsgIndex);
 int UPopResetSession(POP3_HANDLE hPOPSession);
 int UPopSendErrorResponse(BSOCK_HANDLE hBSock, int iErrorCode, int iTimeout);
 int UPopSessionSendMsg(POP3_HANDLE hPOPSession, int iMsgIndex, BSOCK_HANDLE hBSock);
-int UPopSessionTopMsg(POP3_HANDLE hPOPSession, int iMsgIndex, int iNumLines, BSOCK_HANDLE hBSock);
+int UPopSessionTopMsg(POP3_HANDLE hPOPSession, int iMsgIndex, int iNumLines,
+		      BSOCK_HANDLE hBSock);
 int UPopSaveUserIP(POP3_HANDLE hPOPSession);
-int UPopSyncRemoteLink(const char *pszSyncAddr, const char *pszRmtServer,
-		       const char *pszRmtName, const char *pszRmtPassword,
-		       MailSyncReport *pSRep, const char *pszSyncCfg,
-		       const char *pszFetchHdrTags = "+X-Deliver-To,To,Cc",
-		       const char *pszErrorAccount = NULL);
+int UPopSyncRemoteLink(char const *pszSyncAddr, char const *pszRmtServer,
+		       char const *pszRmtName, char const *pszRmtPassword,
+		       MailSyncReport *pSRep, char const *pszSyncCfg,
+		       char const *pszFetchHdrTags = "+X-Deliver-To,To,Cc",
+		       char const *pszErrorAccount = NULL);
 int UPopUserIpCheck(UserInfo *pUI, SYS_INET_ADDR const *pPeerInfo, unsigned int uExpireTime);
 int UPopGetLastLoginInfo(UserInfo *pUI, PopLastLoginInfo *pInfo);
 
