@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -90,71 +90,9 @@ struct MacroSubstCtx {
 	FileSection FSect;
 };
 
-static MessageTagData *USmlAllocTag(char const *pszTagName, char const *pszTagData);
-static void USmlFreeTag(MessageTagData *pMTD);
-static MessageTagData *USmlFindTag(HSLIST &hTagList, char const *pszTagName,
-				   TAG_POSITION &TagPosition);
 static int USmlAddTag(HSLIST &hTagList, char const *pszTagName,
 		      char const *pszTagData, int iUpdate = 0);
-static void USmlFreeTagsList(HSLIST &hTagList);
-static int USmlLoadTags(FILE *pSpoolFile, HSLIST &hTagList);
-static int USmlDumpHeaders(FILE *pMsgFile, HSLIST &hTagList);
-static void USmlFreeData(SpoolFileData *pSFD);
-static void USmlInitHandle(SpoolFileData *pSFD);
-static SpoolFileData *USmlAllocEmptyHandle(void);
-static int USmlLoadHandle(SpoolFileData *pSFD, const char *pszMessFilePath);
-static int USmlFlushMessageFile(SpoolFileData *pSFD);
-static int USmlGetMailProcessFile(UserInfo *pUI, QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage,
-				  char *pszMPFilePath);
-static int USmlLocalDelivery(SVRCFG_HANDLE hSvrConfig, UserInfo *pUI, SPLF_HANDLE hFSpool,
-			     QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage, LocalMailProcConfig &LMPC);
-static int USmlProcessCustomMailingFile(SVRCFG_HANDLE hSvrConfig, UserInfo *pUI,
-					SPLF_HANDLE hFSpool, QUEUE_HANDLE hQueue,
-					QMSG_HANDLE hMessage, char const *pszMPFile,
-					LocalMailProcConfig &LMPC);
-static char *USmlMacroLkupProc(void *pPrivate, char const *pszName, int iSize);
-static int USmlCmdMacroSubstitutes(char **ppszCmdTokens, UserInfo *pUI, SPLF_HANDLE hFSpool);
-static int USmlCmd_external(char **ppszCmdTokens, int iNumTokens, SVRCFG_HANDLE hSvrConfig,
-			    UserInfo *pUI, SPLF_HANDLE hFSpool, QUEUE_HANDLE hQueue,
-			    QMSG_HANDLE hMessage, LocalMailProcConfig &LMPC);
-static int USmlCmd_filter(char **ppszCmdTokens, int iNumTokens, SVRCFG_HANDLE hSvrConfig,
-			  UserInfo *pUI, SPLF_HANDLE hFSpool, QUEUE_HANDLE hQueue,
-			  QMSG_HANDLE hMessage, LocalMailProcConfig &LMPC);
-static int USmlCmd_mailbox(char **ppszCmdTokens, int iNumTokens, SVRCFG_HANDLE hSvrConfig,
-			   UserInfo *pUI, SPLF_HANDLE hFSpool, QUEUE_HANDLE hQueue,
-			   QMSG_HANDLE hMessage, LocalMailProcConfig &LMPC);
-static int USmlCmd_redirect(char **ppszCmdTokens, int iNumTokens, SVRCFG_HANDLE hSvrConfig,
-			    UserInfo *pUI, SPLF_HANDLE hFSpool, QUEUE_HANDLE hQueue,
-			    QMSG_HANDLE hMessage, LocalMailProcConfig &LMPC);
-static int USmlCmd_lredirect(char **ppszCmdTokens, int iNumTokens, SVRCFG_HANDLE hSvrConfig,
-			     UserInfo *pUI, SPLF_HANDLE hFSpool, QUEUE_HANDLE hQueue,
-			     QMSG_HANDLE hMessage, LocalMailProcConfig &LMPC);
-static int USmlCmd_smtprelay(char **ppszCmdTokens, int iNumTokens, SVRCFG_HANDLE hSvrConfig,
-			     UserInfo *pUI, SPLF_HANDLE hFSpool, QUEUE_HANDLE hQueue,
-			     QMSG_HANDLE hMessage, LocalMailProcConfig &LMPC);
-static int USmlLogMessage(char const *pszSMTPDomain, char const *pszMessageID,
-			  char const *pszSmtpMessageID, char const *pszFrom, char const *pszRcpt,
-			  char const *pszMedium, char const *pszParam, char const *pszRmtMsgID);
-static int USmlRFC822_ATEXT(int c);
-static char const *USmlDotAtom(char const *pszStr, char const *pszTop);
-static char const *USmlIPDomain(char const *pszAddress, char const *pszTop);
-char const *USmlParseHost(char const *pszHost, char const *pszTop);
-static int USmlExtractFromAddress(HSLIST &hTagList, char *pszFromAddr, int iMaxAddress);
-static char const *USmlAddressFromAtPtr(char const *pszAt, char const *pszBase,
-					char *pszAddress, int iMaxAddress);
-static char const *USmlAddSingleAddress(char const *pszCurr, char const *pszBase,
-					DynString *pAddrDS, char const *const *ppszMatchDomains,
-					int *piAdded);
-static int USmlAddAddresses(char const *pszAddrList, DynString *pAddrDS,
-			    char const *const *ppszMatchDomains);
-static char **USmlGetAddressList(HSLIST &hTagList, char const *const *ppszMatchDomains,
-				 char const *const *ppszAddrTags);
-static int USmlExtractToAddress(HSLIST &hTagList, char *pszToAddr, int iMaxAddress);
-static char **USmlBuildTargetRcptList(char const *pszRcptTo, HSLIST &hTagList,
-				      const char *pszFetchHdrTags);
-static int USmlCreateSpoolFile(FILE *pMailFile, char const *const *ppszInfo,
-			       char const *pszMailFrom, char const *pszRcptTo,
-			       char const *pszSpoolFile);
+
 
 int USmlLoadSpoolFileHeader(char const *pszSpoolFile, SpoolFileHeader &SFH)
 {
@@ -178,8 +116,7 @@ int USmlLoadSpoolFileHeader(char const *pszSpoolFile, SpoolFileHeader &SFH)
 	if (MscGetString(pSpoolFile, szSpoolLine, sizeof(szSpoolLine) - 1) == NULL ||
 	    (SFH.ppszInfo = StrTokenize(szSpoolLine, ";")) == NULL ||
 	    StrStringsCount(SFH.ppszInfo) < smiMax) {
-		if (SFH.ppszInfo != NULL)
-			StrFreeStrings(SFH.ppszInfo);
+		StrFreeStrings(SFH.ppszInfo);
 		fclose(pSpoolFile);
 		ZeroData(SFH);
 
@@ -234,12 +171,9 @@ int USmlLoadSpoolFileHeader(char const *pszSpoolFile, SpoolFileHeader &SFH)
 
 void USmlCleanupSpoolFileHeader(SpoolFileHeader &SFH)
 {
-	if (SFH.ppszInfo != NULL)
-		StrFreeStrings(SFH.ppszInfo);
-	if (SFH.ppszRcpt != NULL)
-		StrFreeStrings(SFH.ppszRcpt);
-	if (SFH.ppszFrom != NULL)
-		StrFreeStrings(SFH.ppszFrom);
+	StrFreeStrings(SFH.ppszInfo);
+	StrFreeStrings(SFH.ppszRcpt);
+	StrFreeStrings(SFH.ppszFrom);
 	ZeroData(SFH);
 }
 
@@ -262,7 +196,6 @@ static void USmlFreeTag(MessageTagData *pMTD)
 	SysFree(pMTD->pszTagName);
 	SysFree(pMTD->pszTagData);
 	SysFree(pMTD);
-
 }
 
 static MessageTagData *USmlFindTag(HSLIST &hTagList, char const *pszTagName,
@@ -272,13 +205,12 @@ static MessageTagData *USmlFindTag(HSLIST &hTagList, char const *pszTagName,
 		(MessageTagData *) ListFirst(hTagList): (MessageTagData *) TagPosition;
 
 	for (; pMTD != INVALID_SLIST_PTR; pMTD = (MessageTagData *)
-	     ListNext(hTagList, (PLISTLINK) pMTD)) {
+		     ListNext(hTagList, (PLISTLINK) pMTD)) {
 		if (pszTagName == NULL || stricmp(pMTD->pszTagName, pszTagName) == 0) {
 			TagPosition = (TAG_POSITION) ListNext(hTagList, (PLISTLINK) pMTD);
 			return pMTD;
 		}
 	}
-
 	TagPosition = (TAG_POSITION) INVALID_SLIST_PTR;
 
 	return NULL;
@@ -319,20 +251,21 @@ static void USmlFreeTagsList(HSLIST &hTagList)
 
 	while ((pMTD = (MessageTagData *) ListRemove(hTagList)) != INVALID_SLIST_PTR)
 		USmlFreeTag(pMTD);
-
 }
 
 static int USmlLoadTags(FILE *pSpoolFile, HSLIST &hTagList)
 {
+	int iPrevGotNL, iGotNL;
+	unsigned long ulFilePos;
 	DynString TagDS;
+	char szTagName[256] = "", szSpoolLine[MAX_SPOOL_LINE];
 
+	ulFilePos = (unsigned long) ftell(pSpoolFile);
 	StrDynInit(&TagDS);
 
-	unsigned long ulFilePos = (unsigned long) ftell(pSpoolFile);
-	char szSpoolLine[MAX_SPOOL_LINE] = "";
-	char szTagName[256] = "";
-
-	while (MscGetString(pSpoolFile, szSpoolLine, sizeof(szSpoolLine) - 1) != NULL) {
+	for (iPrevGotNL = 1;
+	     MscGetString(pSpoolFile, szSpoolLine, sizeof(szSpoolLine) - 1,
+			  &iGotNL) != NULL; iPrevGotNL = iGotNL) {
 		if (IsEmptyString(szSpoolLine)) {
 			if (StrDynSize(&TagDS) > 0) {
 				if (USmlAddTag(hTagList, szTagName, StrDynGet(&TagDS)) < 0) {
@@ -347,7 +280,8 @@ static int USmlLoadTags(FILE *pSpoolFile, HSLIST &hTagList)
 			}
 			break;
 		}
-		if (szSpoolLine[0] == ' ' || szSpoolLine[0] == '\t') {
+		if (szSpoolLine[0] == ' ' || szSpoolLine[0] == '\t' ||
+		    !iPrevGotNL) {
 			if (IsEmptyString(szTagName)) {
 				StrDynFree(&TagDS);
 				fseek(pSpoolFile, ulFilePos, SEEK_SET);
@@ -355,8 +289,7 @@ static int USmlLoadTags(FILE *pSpoolFile, HSLIST &hTagList)
 				ErrSetErrorCode(ERR_INVALID_MESSAGE_FORMAT);
 				return ERR_INVALID_MESSAGE_FORMAT;
 			}
-
-			if (StrDynAdd(&TagDS, "\r\n") < 0 ||
+			if ((iPrevGotNL && StrDynAdd(&TagDS, "\r\n") < 0) ||
 			    StrDynAdd(&TagDS, szSpoolLine) < 0) {
 				ErrorPush();
 				StrDynFree(&TagDS);
@@ -387,7 +320,7 @@ static int USmlLoadTags(FILE *pSpoolFile, HSLIST &hTagList)
 			}
 
 			int iNameLength = Min((int) (pszEndTag - szSpoolLine),
-					      sizeof(szTagName) - 1);
+					      (int) sizeof(szTagName) - 1);
 			char *pszTagValue = pszEndTag + 1;
 
 			strncpy(szTagName, szSpoolLine, iNameLength);
@@ -408,15 +341,14 @@ static int USmlLoadTags(FILE *pSpoolFile, HSLIST &hTagList)
 	return 0;
 }
 
-static int USmlDumpHeaders(FILE *pMsgFile, HSLIST &hTagList)
+static int USmlDumpHeaders(FILE *pMsgFile, HSLIST &hTagList, char const *pszLF)
 {
 	MessageTagData *pMTD = (MessageTagData *) ListFirst(hTagList);
 
 	for (; pMTD != INVALID_SLIST_PTR; pMTD = (MessageTagData *)
-	     ListNext(hTagList, (PLISTLINK) pMTD)) {
-
-		fprintf(pMsgFile, "%s: %s\r\n", pMTD->pszTagName, pMTD->pszTagData);
-
+		     ListNext(hTagList, (PLISTLINK) pMTD)) {
+		fprintf(pMsgFile, "%s: %s%s", pMTD->pszTagName, pMTD->pszTagData,
+			pszLF);
 	}
 
 	return 0;
@@ -425,19 +357,11 @@ static int USmlDumpHeaders(FILE *pMsgFile, HSLIST &hTagList)
 static void USmlFreeData(SpoolFileData *pSFD)
 {
 	USmlFreeTagsList(pSFD->hTagList);
-
-	if (pSFD->ppszInfo != NULL)
-		StrFreeStrings(pSFD->ppszInfo);
-
-	if (pSFD->ppszFrom != NULL)
-		StrFreeStrings(pSFD->ppszFrom);
-
+	StrFreeStrings(pSFD->ppszInfo);
+	StrFreeStrings(pSFD->ppszFrom);
 	SysFree(pSFD->pszMailFrom);
 	SysFree(pSFD->pszSendMailFrom);
-
-	if (pSFD->ppszRcpt != NULL)
-		StrFreeStrings(pSFD->ppszRcpt);
-
+	StrFreeStrings(pSFD->ppszRcpt);
 	SysFree(pSFD->pszRcptTo);
 	SysFree(pSFD->pszSendRcptTo);
 	SysFree(pSFD->pszRelayDomain);
@@ -445,25 +369,21 @@ static void USmlFreeData(SpoolFileData *pSFD)
 
 char *USmlAddrConcat(char const *const *ppszStrings)
 {
-	int i;
+	int i, iSumLength;
 	int iStrCount = StrStringsCount(ppszStrings);
-	int iSumLength = 0;
 
-	for (i = 0; i < iStrCount; i++)
+	for (i = iSumLength = 0; i < iStrCount; i++)
 		iSumLength += strlen(ppszStrings[i]) + 1;
 
 	char *pszConcat = (char *) SysAlloc(iSumLength + 1);
 
-	if (pszConcat == NULL)
-		return NULL;
-
-	SetEmptyString(pszConcat);
-
-	for (i = 0; i < iStrCount; i++) {
-		if (i > 0)
-			strcat(pszConcat, (i == (iStrCount - 1)) ? ":": ",");
-
-		strcat(pszConcat, ppszStrings[i]);
+	if (pszConcat != NULL) {
+		SetEmptyString(pszConcat);
+		for (i = 0; i < iStrCount; i++) {
+			if (i > 0)
+				strcat(pszConcat, (i == iStrCount - 1) ? ":": ",");
+			strcat(pszConcat, ppszStrings[i]);
+		}
 	}
 
 	return pszConcat;
@@ -535,7 +455,7 @@ char *USmlBuildSendRcptTo(char const *const *ppszFrom, char const *const *ppszRc
 	return pszConcat;
 }
 
-static int USmlLoadHandle(SpoolFileData *pSFD, const char *pszMessFilePath)
+static int USmlLoadHandle(SpoolFileData *pSFD, char const *pszMessFilePath)
 {
 	char szFName[SYS_MAX_PATH] = "";
 	char szExt[SYS_MAX_PATH] = "";
@@ -622,7 +542,7 @@ static int USmlLoadHandle(SpoolFileData *pSFD, const char *pszMessFilePath)
 	}
 	/* Check if it's a relay message */
 	if (StrStringsCount(pSFD->ppszRcpt) > 1) {
-		const char *pszHost = pSFD->ppszRcpt[0];
+		char const *pszHost = pSFD->ppszRcpt[0];
 
 		if (*pszHost++ != '@') {
 			fclose(pSpoolFile);
@@ -678,7 +598,7 @@ static SpoolFileData *USmlAllocEmptyHandle(void)
 	return pSFD;
 }
 
-SPLF_HANDLE USmlCreateHandle(const char *pszMessFilePath)
+SPLF_HANDLE USmlCreateHandle(char const *pszMessFilePath)
 {
 	/* Structure allocation and initialization */
 	SpoolFileData *pSFD = USmlAllocEmptyHandle();
@@ -854,7 +774,7 @@ static int USmlFlushMessageFile(SpoolFileData *pSFD)
 		return ErrorPop();
 	}
 	/* Dump message headers */
-	if (USmlDumpHeaders(pMsgFile, pSFD->hTagList) < 0) {
+	if (USmlDumpHeaders(pMsgFile, pSFD->hTagList, "\r\n") < 0) {
 		ErrorPush();
 		fclose(pMessFile);
 		fclose(pMsgFile);
@@ -934,15 +854,16 @@ int USmlGetMsgFileSection(SPLF_HANDLE hFSpool, FileSection &FSect)
 	return 0;
 }
 
-int USmlWriteMailFile(SPLF_HANDLE hFSpool, FILE *pMsgFile)
+int USmlWriteMailFile(SPLF_HANDLE hFSpool, FILE *pMsgFile, bool bMBoxFile)
 {
 	SpoolFileData *pSFD = (SpoolFileData *) hFSpool;
+	char const *pszLF = bMBoxFile ? SYS_EOL: "\r\n";
 
 	/* Dump message tags */
-	if (USmlDumpHeaders(pMsgFile, pSFD->hTagList) < 0)
+	if (USmlDumpHeaders(pMsgFile, pSFD->hTagList, pszLF) < 0)
 		return ErrGetErrorCode();
 
-	fprintf(pMsgFile, "\r\n");
+	fputs(pszLF, pMsgFile);
 
 	/* Dump message data */
 	FILE *pMessFile = fopen(pSFD->szMessFilePath, "rb");
@@ -951,11 +872,25 @@ int USmlWriteMailFile(SPLF_HANDLE hFSpool, FILE *pMsgFile)
 		ErrSetErrorCode(ERR_FILE_OPEN, pSFD->szMessFilePath);
 		return ERR_FILE_OPEN;
 	}
-	if (MscCopyFile(pMsgFile, pMessFile, pSFD->llMailDataOffset,
-			(SYS_OFF_T) -1) < 0) {
-		ErrorPush();
-		fclose(pMessFile);
-		return ErrorPop();
+
+	bool bWantCRLF = !bMBoxFile;
+
+#ifdef SYS_CRLF_EOL
+	if (!bWantCRLF)
+		bWantCRLF = true;
+#endif
+	if (bWantCRLF) {
+		if (MscCopyFile(pMsgFile, pMessFile, pSFD->llMailDataOffset,
+				(SYS_OFF_T) -1) < 0) {
+			fclose(pMessFile);
+			return ErrGetErrorCode();
+		}
+	} else {
+		Sys_fseek(pMessFile, pSFD->llMailDataOffset, SEEK_SET);
+		if (MscDos2UnixFile(pMsgFile, pMessFile) < 0) {
+			fclose(pMessFile);
+			return ErrGetErrorCode();
+		}
 	}
 	fclose(pMessFile);
 
@@ -965,7 +900,6 @@ int USmlWriteMailFile(SPLF_HANDLE hFSpool, FILE *pMsgFile)
 char *USmlGetTag(SPLF_HANDLE hFSpool, char const *pszTagName, TAG_POSITION &TagPosition)
 {
 	SpoolFileData *pSFD = (SpoolFileData *) hFSpool;
-
 	MessageTagData *pMTD = USmlFindTag(pSFD->hTagList, pszTagName, TagPosition);
 
 	return (pMTD != NULL) ? SysStrDup(pMTD->pszTagData): NULL;
@@ -1087,7 +1021,7 @@ int USmlCreateMBFile(UserInfo *pUI, char const *pszFileName, SPLF_HANDLE hFSpool
 
 				if (iRetLength > (int) (sizeof(szReturnPath) - iExtraLength - 2))
 					pszRetPath[sizeof(szReturnPath) - iExtraLength - 2] =
-					'\0';
+						'\0';
 
 				SysSNPrintf(szReturnPath, sizeof(szReturnPath) - 1,
 					    "Return-Path: <%s>", pszRetPath);
@@ -1105,18 +1039,18 @@ int USmlCreateMBFile(UserInfo *pUI, char const *pszFileName, SPLF_HANDLE hFSpool
 			USmlSetTagAddress(hFSpool, "Reply-To", szAddress);
 		}
 
-		fprintf(pMBFile, "%s\r\n", szReturnPath);
+		fprintf(pMBFile, "%s" SYS_EOL, szReturnPath);
 
 		/* Add "Delivered-To:" tag */
 		char szUserAddress[MAX_ADDR_NAME] = "";
 
 		UsrGetAddress(pUI, szUserAddress);
-		fprintf(pMBFile, "Delivered-To: %s\r\n", szUserAddress);
+		fprintf(pMBFile, "Delivered-To: %s" SYS_EOL, szUserAddress);
 	} else
 		SysFree(pszReturnPath);
 
 	/* Write mail file */
-	if (USmlWriteMailFile(hFSpool, pMBFile) < 0) {
+	if (USmlWriteMailFile(hFSpool, pMBFile, true) < 0) {
 		ErrorPush();
 		fclose(pMBFile);
 		SysRemove(pszFileName);
@@ -1191,6 +1125,73 @@ int USmlVCreateSpoolFile(SPLF_HANDLE hFSpool, char const *pszFromUser,
 	return 0;
 }
 
+static int USmlCreateSpoolFile(FILE *pMailFile, char const *const *ppszInfo,
+			       char const *pszMailFrom, char const *pszRcptTo,
+			       char const *pszSpoolFile)
+{
+	FILE *pSpoolFile = fopen(pszSpoolFile, "wb");
+
+	if (pSpoolFile == NULL) {
+		ErrSetErrorCode(ERR_FILE_CREATE, pszSpoolFile);
+		return ERR_FILE_CREATE;
+	}
+	/* Write info line */
+	if (ppszInfo != NULL)
+		USmtpWriteInfoLine(pSpoolFile, ppszInfo[smsgiClientAddr],
+				   ppszInfo[smsgiServerAddr], ppszInfo[smsgiTime]);
+	else {
+		char szTime[256] = "";
+
+		MscGetTimeStr(szTime, sizeof(szTime) - 1);
+		USmtpWriteInfoLine(pSpoolFile, LOCAL_ADDRESS_SQB ":0",
+				   LOCAL_ADDRESS_SQB ":0", szTime);
+	}
+
+	/* Write SMTP domain */
+	char szSmtpDomain[MAX_HOST_NAME] = "";
+
+	if (USmtpSplitEmailAddr(pszRcptTo, NULL, szSmtpDomain) < 0) {
+		ErrorPush();
+		fclose(pSpoolFile);
+		SysRemove(pszSpoolFile);
+		return ErrorPop();
+	}
+
+	fprintf(pSpoolFile, "%s\r\n", szSmtpDomain);
+
+	/* Write message ID */
+	SYS_UINT64 ullMessageID = 0;
+
+	if (SvrGetMessageID(&ullMessageID) < 0) {
+		ErrorPush();
+		fclose(pSpoolFile);
+		SysRemove(pszSpoolFile);
+		return ErrorPop();
+	}
+
+	fprintf(pSpoolFile, "P" SYS_LLX_FMT "\r\n", ullMessageID);
+
+	/* Write "MAIL FROM:" */
+	fprintf(pSpoolFile, "MAIL FROM: <%s>\r\n", pszMailFrom);
+
+	/* Write "RCPT TO:" */
+	fprintf(pSpoolFile, "RCPT TO: <%s>\r\n", pszRcptTo);
+
+	/* Write SPOOL_FILE_DATA_START */
+	fprintf(pSpoolFile, "%s\r\n", SPOOL_FILE_DATA_START);
+
+	/* Write message body */
+	if (MscCopyFile(pSpoolFile, pMailFile, 0, (SYS_OFF_T) -1) < 0) {
+		ErrorPush();
+		fclose(pSpoolFile);
+		SysRemove(pszSpoolFile);
+		return ErrorPop();
+	}
+	fclose(pSpoolFile);
+
+	return 0;
+}
+
 int USmlCreateSpoolFile(SPLF_HANDLE hFSpool, char const *pszFromUser,
 			char const *pszRcptUser, char const *pszFileName, ...)
 {
@@ -1250,7 +1251,9 @@ static int USmlLocalDelivery(SVRCFG_HANDLE hSvrConfig, UserInfo *pUI, SPLF_HANDL
 	/* Create mailbox file ... */
 	char szMBFile[SYS_MAX_PATH] = "";
 
-	SysGetTmpFile(szMBFile);
+	if (UsrGetTmpFile(pUI->pszDomain, szMBFile, sizeof(szMBFile)) < 0)
+		return ErrGetErrorCode();
+
 	if (USmlCreateMBFile(pUI, szMBFile, hFSpool) < 0)
 		return ErrGetErrorCode();
 
@@ -1272,147 +1275,6 @@ static int USmlLocalDelivery(SVRCFG_HANDLE hSvrConfig, UserInfo *pUI, SPLF_HANDL
 	return 0;
 }
 
-int USmlProcessLocalUserMessage(SVRCFG_HANDLE hSvrConfig, UserInfo *pUI, SPLF_HANDLE hFSpool,
-				QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage,
-				LocalMailProcConfig &LMPC)
-{
-	/* Exist user custom message processing ? */
-	char szMPFile[SYS_MAX_PATH] = "";
-
-	if (USmlGetMailProcessFile(pUI, hQueue, hMessage, szMPFile) < 0) {
-		/*
-		 * Deliver the file locally.
-		 */
-		if (USmlLocalDelivery(hSvrConfig, pUI, hFSpool, hQueue, hMessage, LMPC) < 0)
-			return ErrGetErrorCode();
-	} else {
-		/* Process custom mailings */
-		if (USmlProcessCustomMailingFile(hSvrConfig, pUI, hFSpool, hQueue, hMessage,
-						 szMPFile, LMPC) < 0)
-			return ErrGetErrorCode();
-
-	}
-
-	return 0;
-}
-
-static int USmlProcessCustomMailingFile(SVRCFG_HANDLE hSvrConfig, UserInfo *pUI,
-					SPLF_HANDLE hFSpool, QUEUE_HANDLE hQueue,
-					QMSG_HANDLE hMessage, char const *pszMPFile,
-					LocalMailProcConfig &LMPC)
-{
-	/* Open the mail processing file */
-	FILE *pMPFile = fopen(pszMPFile, "rt");
-
-	if (pMPFile == NULL) {
-		ErrSetErrorCode(ERR_FILE_OPEN, pszMPFile);
-		return ERR_FILE_OPEN;
-	}
-	/* Create pushback command file */
-	char szTmpFile[SYS_MAX_PATH] = "";
-
-	SysGetTmpFile(szTmpFile);
-
-	FILE *pPushBFile = fopen(szTmpFile, "wt");
-
-	if (pPushBFile == NULL) {
-		fclose(pMPFile);
-
-		ErrSetErrorCode(ERR_FILE_CREATE, szTmpFile);
-		return ERR_FILE_CREATE;
-	}
-
-	int iPushBackCmds = 0;
-	char szCmdLine[CUSTOM_CMD_LINE_MAX] = "";
-
-	while (MscGetConfigLine(szCmdLine, sizeof(szCmdLine) - 1, pMPFile) != NULL) {
-		char **ppszCmdTokens = StrGetTabLineStrings(szCmdLine);
-
-		if (ppszCmdTokens == NULL)
-			continue;
-
-		int iFieldsCount = StrStringsCount(ppszCmdTokens);
-
-		if (iFieldsCount > 0) {
-			/* Do command line macro substitution */
-			USmlCmdMacroSubstitutes(ppszCmdTokens, pUI, hFSpool);
-
-			int iCmdResult = 0;
-
-			if (stricmp(ppszCmdTokens[0], "external") == 0)
-				iCmdResult =
-				USmlCmd_external(ppszCmdTokens, iFieldsCount, hSvrConfig, pUI,
-						 hFSpool, hQueue, hMessage, LMPC);
-			else if (stricmp(ppszCmdTokens[0], "filter") == 0)
-				iCmdResult =
-				USmlCmd_filter(ppszCmdTokens, iFieldsCount, hSvrConfig, pUI,
-					       hFSpool, hQueue, hMessage, LMPC);
-			else if (stricmp(ppszCmdTokens[0], "mailbox") == 0)
-				iCmdResult =
-				USmlCmd_mailbox(ppszCmdTokens, iFieldsCount, hSvrConfig, pUI,
-						hFSpool, hQueue, hMessage, LMPC);
-			else if (stricmp(ppszCmdTokens[0], "redirect") == 0)
-				iCmdResult =
-				USmlCmd_redirect(ppszCmdTokens, iFieldsCount, hSvrConfig, pUI,
-						 hFSpool, hQueue, hMessage, LMPC);
-			else if (stricmp(ppszCmdTokens[0], "lredirect") == 0)
-				iCmdResult =
-				USmlCmd_lredirect(ppszCmdTokens, iFieldsCount, hSvrConfig,
-						  pUI, hFSpool, hQueue, hMessage, LMPC);
-			else if (stricmp(ppszCmdTokens[0], "smtprelay") == 0)
-				iCmdResult =
-				USmlCmd_smtprelay(ppszCmdTokens, iFieldsCount, hSvrConfig,
-						  pUI, hFSpool, hQueue, hMessage, LMPC);
-			else {
-				SysLogMessage(LOG_LEV_ERROR,
-					      "Invalid command \"%s\" in file \"%s\"\n",
-					      ppszCmdTokens[0], pszMPFile);
-
-			}
-
-			/* Check for the stop-processing error code */
-			if (iCmdResult == SMAIL_STOP_PROCESSING) {
-				StrFreeStrings(ppszCmdTokens);
-				break;
-			}
-			/* Test if we must save a failed command */
-			/* <0 = Error ; ==0 = Success ; >0 = Transient error ( save the command ) */
-			if (iCmdResult > 0) {
-				fprintf(pPushBFile, "%s\n", szCmdLine);
-
-				++iPushBackCmds;
-			}
-			/* An error code might result if filters blocked the message. If this is the */
-			/* case QueCheckMessage() will return error and we MUST stop processing */
-			if (iCmdResult < 0 &&
-			    QueCheckMessage(hQueue, hMessage) < 0) {
-				ErrorPush();
-				StrFreeStrings(ppszCmdTokens);
-				fclose(pPushBFile);
-				fclose(pMPFile);
-				SysRemove(szTmpFile);
-				return ErrorPop();
-			}
-		}
-		StrFreeStrings(ppszCmdTokens);
-	}
-	fclose(pPushBFile);
-	fclose(pMPFile);
-	SysRemove(pszMPFile);
-
-	if (iPushBackCmds > 0) {
-		/* If commands left out of processing, push them into the custom file */
-		if (MscMoveFile(szTmpFile, pszMPFile) < 0)
-			return ErrGetErrorCode();
-
-		ErrSetErrorCode(ERR_INCOMPLETE_PROCESSING);
-		return ERR_INCOMPLETE_PROCESSING;
-	}
-	SysRemove(szTmpFile);
-
-	return 0;
-}
-
 static char *USmlMacroLkupProc(void *pPrivate, char const *pszName, int iSize)
 {
 	MacroSubstCtx *pMSC = (MacroSubstCtx *) pPrivate;
@@ -1453,7 +1315,7 @@ static char *USmlMacroLkupProc(void *pPrivate, char const *pszName, int iSize)
 	} else if (MemMatch(pszName, iSize, "TMPFILE", 7)) {
 		char szTmpFile[SYS_MAX_PATH] = "";
 
-		SysGetTmpFile(szTmpFile);
+		MscSafeGetTmpFile(szTmpFile, sizeof(szTmpFile));
 		if (MscCopyFile(szTmpFile, pMSC->FSect.szFilePath) < 0) {
 			CheckRemoveFile(szTmpFile);
 			return NULL;
@@ -1501,7 +1363,7 @@ static int USmlCmd_external(char **ppszCmdTokens, int iNumTokens, SVRCFG_HANDLE
 	}
 
 	int iPriority = atoi(ppszCmdTokens[1]);
-	int iWaitTimeout = atoi(ppszCmdTokens[2]);
+	int iWaitTimeout = atoi(ppszCmdTokens[2]) * 1000;
 	int iExitStatus = 0;
 
 	if (SysExec(ppszCmdTokens[3], &ppszCmdTokens[3], iWaitTimeout, iPriority,
@@ -1538,7 +1400,7 @@ static int USmlCmd_filter(char **ppszCmdTokens, int iNumTokens, SVRCFG_HANDLE hS
 	}
 
 	int iPriority = atoi(ppszCmdTokens[1]);
-	int iWaitTimeout = atoi(ppszCmdTokens[2]);
+	int iWaitTimeout = atoi(ppszCmdTokens[2]) * 1000;
 	int iExitStatus = 0;
 
 	if (SysExec(ppszCmdTokens[3], &ppszCmdTokens[3], iWaitTimeout, iPriority,
@@ -1761,20 +1623,8 @@ static int USmlCmd_smtprelay(char **ppszCmdTokens, int iNumTokens, SVRCFG_HANDLE
 	char **ppszRelays = NULL;
 
 	if (ppszCmdTokens[1][0] == '#') {
-		if ((ppszRelays = StrTokenize(ppszCmdTokens[1] + 1, ";")) != NULL) {
-			int iRelayCount = StrStringsCount(ppszRelays);
-
-			srand((unsigned int) time(NULL));
-			for (int i = 0; i < (iRelayCount / 2); i++) {
-				int iSwap1 = rand() % iRelayCount;
-				int iSwap2 = rand() % iRelayCount;
-				char *pszRly1 = ppszRelays[iSwap1];
-				char *pszRly2 = ppszRelays[iSwap2];
-
-				ppszRelays[iSwap1] = pszRly2;
-				ppszRelays[iSwap2] = pszRly1;
-			}
-		}
+		if ((ppszRelays = StrTokenize(ppszCmdTokens[1] + 1, ";")) != NULL)
+			MscRandomizeStringsOrder(ppszRelays);
 	} else
 		ppszRelays = StrTokenize(ppszCmdTokens[1], ";");
 
@@ -1816,11 +1666,13 @@ static int USmlCmd_smtprelay(char **ppszCmdTokens, int iNumTokens, SVRCFG_HANDLE
 
 	USmtpInitError(&SMTPE);
 
-	/* By initializing this to zero makes XMail to discharge all mail for domains */
-	/* that have an empty relay list */
+	/*
+	 * By initializing this to zero makes XMail to discharge all mail for domains
+	 * that have an empty relay list
+	 */
 	int iReturnCode = 0;
 
-	for (int i = 0; ppGws[i] != NULL; i++) {
+	for (int i = 0; ppGws[i] != NULL && iReturnCode >= 0; i++) {
 		SysLogMessage(LOG_LEV_MESSAGE,
 			      "USMAIL SMTP-Send RLYS = \"%s\" SMTP = \"%s\" From = \"%s\" To = \"%s\"\n",
 			      ppGws[i]->pszHost, pszSMTPDomain, pszMailFrom, pszRcptTo);
@@ -1872,6 +1724,147 @@ static int USmlCmd_smtprelay(char **ppszCmdTokens, int iNumTokens, SVRCFG_HANDLE
 	return iReturnCode;
 }
 
+static int USmlProcessCustomMailingFile(SVRCFG_HANDLE hSvrConfig, UserInfo *pUI,
+					SPLF_HANDLE hFSpool, QUEUE_HANDLE hQueue,
+					QMSG_HANDLE hMessage, char const *pszMPFile,
+					LocalMailProcConfig &LMPC)
+{
+	/* Open the mail processing file */
+	FILE *pMPFile = fopen(pszMPFile, "rt");
+
+	if (pMPFile == NULL) {
+		ErrSetErrorCode(ERR_FILE_OPEN, pszMPFile);
+		return ERR_FILE_OPEN;
+	}
+	/* Create pushback command file */
+	char szTmpFile[SYS_MAX_PATH] = "";
+
+	UsrGetTmpFile(NULL, szTmpFile, sizeof(szTmpFile));
+
+	FILE *pPushBFile = fopen(szTmpFile, "wt");
+
+	if (pPushBFile == NULL) {
+		fclose(pMPFile);
+
+		ErrSetErrorCode(ERR_FILE_CREATE, szTmpFile);
+		return ERR_FILE_CREATE;
+	}
+
+	int iPushBackCmds = 0;
+	char szCmdLine[CUSTOM_CMD_LINE_MAX] = "";
+
+	while (MscGetConfigLine(szCmdLine, sizeof(szCmdLine) - 1, pMPFile) != NULL) {
+		char **ppszCmdTokens = StrGetTabLineStrings(szCmdLine);
+
+		if (ppszCmdTokens == NULL)
+			continue;
+
+		int iFieldsCount = StrStringsCount(ppszCmdTokens);
+
+		if (iFieldsCount > 0) {
+			/* Do command line macro substitution */
+			USmlCmdMacroSubstitutes(ppszCmdTokens, pUI, hFSpool);
+
+			int iCmdResult = 0;
+
+			if (stricmp(ppszCmdTokens[0], "external") == 0)
+				iCmdResult =
+					USmlCmd_external(ppszCmdTokens, iFieldsCount, hSvrConfig, pUI,
+							 hFSpool, hQueue, hMessage, LMPC);
+			else if (stricmp(ppszCmdTokens[0], "filter") == 0)
+				iCmdResult =
+					USmlCmd_filter(ppszCmdTokens, iFieldsCount, hSvrConfig, pUI,
+						       hFSpool, hQueue, hMessage, LMPC);
+			else if (stricmp(ppszCmdTokens[0], "mailbox") == 0)
+				iCmdResult =
+					USmlCmd_mailbox(ppszCmdTokens, iFieldsCount, hSvrConfig, pUI,
+							hFSpool, hQueue, hMessage, LMPC);
+			else if (stricmp(ppszCmdTokens[0], "redirect") == 0)
+				iCmdResult =
+					USmlCmd_redirect(ppszCmdTokens, iFieldsCount, hSvrConfig, pUI,
+							 hFSpool, hQueue, hMessage, LMPC);
+			else if (stricmp(ppszCmdTokens[0], "lredirect") == 0)
+				iCmdResult =
+					USmlCmd_lredirect(ppszCmdTokens, iFieldsCount, hSvrConfig,
+							  pUI, hFSpool, hQueue, hMessage, LMPC);
+			else if (stricmp(ppszCmdTokens[0], "smtprelay") == 0)
+				iCmdResult =
+					USmlCmd_smtprelay(ppszCmdTokens, iFieldsCount, hSvrConfig,
+							  pUI, hFSpool, hQueue, hMessage, LMPC);
+			else {
+				SysLogMessage(LOG_LEV_ERROR,
+					      "Invalid command \"%s\" in file \"%s\"\n",
+					      ppszCmdTokens[0], pszMPFile);
+
+			}
+
+			/* Check for the stop-processing error code */
+			if (iCmdResult == SMAIL_STOP_PROCESSING) {
+				StrFreeStrings(ppszCmdTokens);
+				break;
+			}
+			/* Test if we must save a failed command */
+			/* <0 = Error ; ==0 = Success ; >0 = Transient error ( save the command ) */
+			if (iCmdResult > 0) {
+				fprintf(pPushBFile, "%s\n", szCmdLine);
+
+				++iPushBackCmds;
+			}
+			/* An error code might result if filters blocked the message. If this is the */
+			/* case QueCheckMessage() will return error and we MUST stop processing */
+			if (iCmdResult < 0 &&
+			    QueCheckMessage(hQueue, hMessage) < 0) {
+				ErrorPush();
+				StrFreeStrings(ppszCmdTokens);
+				fclose(pPushBFile);
+				fclose(pMPFile);
+				SysRemove(szTmpFile);
+				return ErrorPop();
+			}
+		}
+		StrFreeStrings(ppszCmdTokens);
+	}
+	fclose(pPushBFile);
+	fclose(pMPFile);
+	SysRemove(pszMPFile);
+
+	if (iPushBackCmds > 0) {
+		/* If commands left out of processing, push them into the custom file */
+		if (MscMoveFile(szTmpFile, pszMPFile) < 0)
+			return ErrGetErrorCode();
+
+		ErrSetErrorCode(ERR_INCOMPLETE_PROCESSING);
+		return ERR_INCOMPLETE_PROCESSING;
+	}
+	SysRemove(szTmpFile);
+
+	return 0;
+}
+
+int USmlProcessLocalUserMessage(SVRCFG_HANDLE hSvrConfig, UserInfo *pUI, SPLF_HANDLE hFSpool,
+				QUEUE_HANDLE hQueue, QMSG_HANDLE hMessage,
+				LocalMailProcConfig &LMPC)
+{
+	/* Exist user custom message processing ? */
+	char szMPFile[SYS_MAX_PATH] = "";
+
+	if (USmlGetMailProcessFile(pUI, hQueue, hMessage, szMPFile) < 0) {
+		/*
+		 * Deliver the file locally.
+		 */
+		if (USmlLocalDelivery(hSvrConfig, pUI, hFSpool, hQueue, hMessage, LMPC) < 0)
+			return ErrGetErrorCode();
+	} else {
+		/* Process custom mailings */
+		if (USmlProcessCustomMailingFile(hSvrConfig, pUI, hFSpool, hQueue, hMessage,
+						 szMPFile, LMPC) < 0)
+			return ErrGetErrorCode();
+
+	}
+
+	return 0;
+}
+
 int USmlGetDomainCustomDir(char *pszCustomDir, int iMaxPath, int iFinalSlash)
 {
 	CfgGetRootPath(pszCustomDir, iMaxPath);
@@ -2210,9 +2203,8 @@ static int USmlRFC822_ATEXT(int c)
 	return isalpha(c) || isdigit(c) || strchr("!#$%&'*+-/=?^_`{|}~", c) != NULL;
 }
 
-static char const *USmlDotAtom(char const *pszStr, char const *pszTop)
+char const *USmlDotAtom(char const *pszStr, char const *pszTop)
 {
-
 	for (;;) {
 		char const *pszAtom = pszStr;
 
@@ -2472,7 +2464,7 @@ static char **USmlGetAddressList(HSLIST &hTagList, char const *const *ppszMatchD
 							  ppszMatchDomains);
 
 			/* Exclusive tag detected, stop the scan */
-			if ((iAddrAdded > 0) && (ppszAddrTags[i][0] == '+'))
+			if (iAddrAdded > 0 && ppszAddrTags[i][0] == '+')
 				goto BuildAddrList;
 		}
 	}
@@ -2503,7 +2495,7 @@ static int USmlExtractToAddress(HSLIST &hTagList, char *pszToAddr, int iMaxAddre
 }
 
 static char **USmlBuildTargetRcptList(char const *pszRcptTo, HSLIST &hTagList,
-				      const char *pszFetchHdrTags)
+				      char const *pszFetchHdrTags)
 {
 	char **ppszRcptList = NULL;
 	char **ppszAddrTags = NULL;
@@ -2572,10 +2564,12 @@ static char **USmlBuildTargetRcptList(char const *pszRcptTo, HSLIST &hTagList,
 			StrFreeStrings(ppszAddrTags);
 			return NULL;
 		}
-		/* We need to masquerade incoming domain. In this case "pszRcptTo" is made by */
-		/* "&" + add-domain */
-		if ((ppszRcptList =
-		     USmlGetAddressList(hTagList, &ppszDomains[1], ppszAddrTags)) == NULL) {
+		/*
+		 * We need to masquerade incoming domain. In this case "pszRcptTo" is made by
+		 * "&" + add-domain + match-domains.
+		 */
+		if ((ppszRcptList = USmlGetAddressList(hTagList, &ppszDomains[1],
+						       ppszAddrTags)) == NULL) {
 			StrFreeStrings(ppszDomains);
 			StrFreeStrings(ppszAddrTags);
 			return NULL;
@@ -2596,13 +2590,12 @@ static char **USmlBuildTargetRcptList(char const *pszRcptTo, HSLIST &hTagList,
 	} else
 		ppszRcptList = StrBuildList(pszRcptTo, NULL);
 
-	if (ppszAddrTags != NULL)
-		StrFreeStrings(ppszAddrTags);
+	StrFreeStrings(ppszAddrTags);
 
 	return ppszRcptList;
 }
 
-int USmlDeliverFetchedMsg(char const *pszSyncAddr, const char *pszFetchHdrTags,
+int USmlDeliverFetchedMsg(char const *pszSyncAddr, char const *pszFetchHdrTags,
 			  char const *pszMailFile)
 {
 	FILE *pMailFile = fopen(pszMailFile, "rb");
@@ -2683,9 +2676,7 @@ int USmlDeliverFetchedMsg(char const *pszSyncAddr, const char *pszFetchHdrTags,
 
 		++iDeliverCount;
 	}
-
 	StrFreeStrings(ppszRcptList);
-
 	fclose(pMailFile);
 
 	/* Check if the message has been delivered at least one time */
@@ -2697,73 +2688,6 @@ int USmlDeliverFetchedMsg(char const *pszSyncAddr, const char *pszFetchHdrTags,
 	return 0;
 }
 
-static int USmlCreateSpoolFile(FILE *pMailFile, char const *const *ppszInfo,
-			       char const *pszMailFrom, char const *pszRcptTo,
-			       char const *pszSpoolFile)
-{
-	FILE *pSpoolFile = fopen(pszSpoolFile, "wb");
-
-	if (pSpoolFile == NULL) {
-		ErrSetErrorCode(ERR_FILE_CREATE, pszSpoolFile);
-		return ERR_FILE_CREATE;
-	}
-	/* Write info line */
-	if (ppszInfo != NULL)
-		USmtpWriteInfoLine(pSpoolFile, ppszInfo[smsgiClientAddr],
-				   ppszInfo[smsgiServerAddr], ppszInfo[smsgiTime]);
-	else {
-		char szTime[256] = "";
-
-		MscGetTimeStr(szTime, sizeof(szTime) - 1);
-		USmtpWriteInfoLine(pSpoolFile, LOCAL_ADDRESS_SQB ":0",
-				   LOCAL_ADDRESS_SQB ":0", szTime);
-	}
-
-	/* Write SMTP domain */
-	char szSmtpDomain[MAX_HOST_NAME] = "";
-
-	if (USmtpSplitEmailAddr(pszRcptTo, NULL, szSmtpDomain) < 0) {
-		ErrorPush();
-		fclose(pSpoolFile);
-		SysRemove(pszSpoolFile);
-		return ErrorPop();
-	}
-
-	fprintf(pSpoolFile, "%s\r\n", szSmtpDomain);
-
-	/* Write message ID */
-	SYS_UINT64 ullMessageID = 0;
-
-	if (SvrGetMessageID(&ullMessageID) < 0) {
-		ErrorPush();
-		fclose(pSpoolFile);
-		SysRemove(pszSpoolFile);
-		return ErrorPop();
-	}
-
-	fprintf(pSpoolFile, "P" SYS_LLX_FMT "\r\n", ullMessageID);
-
-	/* Write "MAIL FROM:" */
-	fprintf(pSpoolFile, "MAIL FROM: <%s>\r\n", pszMailFrom);
-
-	/* Write "RCPT TO:" */
-	fprintf(pSpoolFile, "RCPT TO: <%s>\r\n", pszRcptTo);
-
-	/* Write SPOOL_FILE_DATA_START */
-	fprintf(pSpoolFile, "%s\r\n", SPOOL_FILE_DATA_START);
-
-	/* Write message body */
-	if (MscCopyFile(pSpoolFile, pMailFile, 0, (unsigned long) -1) < 0) {
-		ErrorPush();
-		fclose(pSpoolFile);
-		SysRemove(pszSpoolFile);
-		return ErrorPop();
-	}
-	fclose(pSpoolFile);
-
-	return 0;
-}
-
 int USmlMailLoopCheck(SPLF_HANDLE hFSpool, SVRCFG_HANDLE hSvrConfig)
 {
 	SpoolFileData *pSFD = (SpoolFileData *) hFSpool;
@@ -2774,9 +2698,9 @@ int USmlMailLoopCheck(SPLF_HANDLE hFSpool, SVRCFG_HANDLE hSvrConfig)
 	MessageTagData *pMTD = (MessageTagData *) ListFirst(pSFD->hTagList);
 
 	for (; pMTD != INVALID_SLIST_PTR; pMTD = (MessageTagData *)
-	     ListNext(pSFD->hTagList, (PLISTLINK) pMTD))
-		if ((stricmp(pMTD->pszTagName, "Received") == 0) ||
-		    (stricmp(pMTD->pszTagName, "X-Deliver-To") == 0))
+		     ListNext(pSFD->hTagList, (PLISTLINK) pMTD))
+		if (stricmp(pMTD->pszTagName, "Received") == 0 ||
+		    stricmp(pMTD->pszTagName, "X-Deliver-To") == 0)
 			++iLoopsCount;
 
 	/* Check MTA count */
@@ -2797,7 +2721,7 @@ int USmlMessageAuth(SPLF_HANDLE hFSpool, char *pszAuthName, int iSize)
 	MessageTagData *pMTD = (MessageTagData *) ListFirst(pSFD->hTagList);
 
 	for (; pMTD != INVALID_SLIST_PTR; pMTD = (MessageTagData *)
-	     ListNext(pSFD->hTagList, (PLISTLINK) pMTD)) {
+		     ListNext(pSFD->hTagList, (PLISTLINK) pMTD)) {
 		if (stricmp(pMTD->pszTagName, "Received") == 0)
 			break;
 		if (stricmp(pMTD->pszTagName, "X-AuthUser") == 0) {
