@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ void USmlCleanupSpoolFileHeader(SpoolFileHeader &SFH);
 char *USmlAddrConcat(char const *const *ppszStrings);
 char *USmlBuildSendMailFrom(char const *const *ppszFrom, char const *const *ppszRcpt);
 char *USmlBuildSendRcptTo(char const *const *ppszFrom, char const *const *ppszRcpt);
-SPLF_HANDLE USmlCreateHandle(const char *pszMessFilePath);
+SPLF_HANDLE USmlCreateHandle(char const *pszMessFilePath);
 void USmlCloseHandle(SPLF_HANDLE hFSpool);
 int USmlReloadHandle(SPLF_HANDLE hFSpool);
 char const *USmlGetRelayDomain(SPLF_HANDLE hFSpool);
@@ -77,7 +77,7 @@ char const *USmlSendRcptTo(SPLF_HANDLE hFSpool);
 SYS_OFF_T USmlMessageSize(SPLF_HANDLE hFSpool);
 int USmlSyncChanges(SPLF_HANDLE hFSpool);
 int USmlGetMsgFileSection(SPLF_HANDLE hFSpool, FileSection &FSect);
-int USmlWriteMailFile(SPLF_HANDLE hFSpool, FILE *pMsgFile);
+int USmlWriteMailFile(SPLF_HANDLE hFSpool, FILE *pMsgFile, bool bMBoxFile = false);
 char *USmlGetTag(SPLF_HANDLE hFSpool, char const *pszTagName, TAG_POSITION &TagPosition);
 int USmlAddTag(SPLF_HANDLE hFSpool, char const *pszTagName,
 	       char const *pszTagData, int iUpdate = 0);
@@ -114,11 +114,12 @@ int USmlSetCustomDomainFile(char const *pszDestDomain, char const *pszCustFilePa
 int USmlCustomizedDomain(char const *pszDestDomain);
 int USmlLogMessage(SPLF_HANDLE hFSpool, char const *pszMedium, char const *pszRmtMsgID,
 		   char const *pszParam);
+char const *USmlDotAtom(char const *pszStr, char const *pszTop);
 int USmlValidAddress(char const *pszAddress, char const *pszTop);
 int USmlValidHost(char const *pszHost, char const *pszTop);
 int USmlParseAddress(char const *pszAddress, char *pszPreAddr,
 		     int iMaxPreAddress, char *pszEmailAddr, int iMaxAddress);
-int USmlDeliverFetchedMsg(char const *pszSyncAddr, const char *pszFetchHdrTags,
+int USmlDeliverFetchedMsg(char const *pszSyncAddr, char const *pszFetchHdrTags,
 			  char const *pszMailFile);
 int USmlMailLoopCheck(SPLF_HANDLE hFSpool, SVRCFG_HANDLE hSvrConfig);
 int USmlMessageAuth(SPLF_HANDLE hFSpool, char *pszAuthName, int iSize);
