@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -59,41 +59,37 @@
 #define SPAMMERS_LINE_MAX       512
 #define SPAM_ADDRESS_LINE_MAX   512
 #define SMTPAUTH_LINE_MAX       512
-#define SMTP_EXTAUTH_TIMEOUT    60
-#define SMTP_EXTAUTH_PRIORITY   SYS_PRIORITY_NORMAL
-#define SMTP_EXTAUTH_SUCCESS    0
-#define RFC_SPECIALS            "()<>@,/\\;:\"[]*?"
 
 #define SMTPCH_SUPPORT_SIZE     (1 << 0)
 #define SMTPCH_SUPPORT_TLS      (1 << 1)
 
 enum SmtpGwFileds {
 	gwDomain = 0,
-		gwGateway,
+	gwGateway,
 
-		gwMax
+	gwMax
 };
 
 enum SmtpFwdFileds {
 	fwdDomain = 0,
-		fwdGateway,
-		fwdOptions,
+	fwdGateway,
+	fwdOptions,
 
-		fwdMax
+	fwdMax
 };
 
 enum SmtpRelayFileds {
 	rlyFromIP = 0,
-		rlyFromMask,
+	rlyFromMask,
 
-		rlyMax
+	rlyMax
 };
 
 enum SpammerFileds {
 	spmFromIP = 0,
-		spmFromMask,
+	spmFromMask,
 
-		spmMax
+	spmMax
 };
 
 struct SmtpMXRecords {
@@ -113,41 +109,10 @@ struct SmtpChannel {
 };
 
 
-static int USmtpWriteGateway(FILE *pGwFile, const char *pszDomain, const char *pszGateway);
-static char *USmtpGetGwTableFilePath(char *pszGwFilePath, int iMaxPath);
-static char *USmtpGetFwdTableFilePath(char *pszFwdFilePath, int iMaxPath);
-static void USmtpCleanupGateway(SMTPGateway *pGw);
-static void USmtpFreeGateway(SMTPGateway *pGw);
-static SMTPGateway *USmtpCloneGateway(SMTPGateway const *pRefGw, char const *pszHost);
-static int USmtpOptionsAssign(void *pPrivate, char const *pszName, char const *pszValue);
-static int USmtpSetGwOptions(SMTPGateway *pGw, char const *pszOptions);
-static char *USmtpGetRelayFilePath(char *pszRelayFilePath, int iMaxPath);
-static int USmtpSetErrorServer(SMTPError *pSMTPE, char const *pszServer);
-static int USmtpResponseClass(int iResponseCode, int iResponseClass);
-static int USmtpGetResultCode(const char *pszResult);
-static int USmtpIsPartialResponse(char const *pszResponse);
 static int USmtpGetResponse(BSOCK_HANDLE hBSock, char *pszResponse, int iMaxResponse,
 			    int iTimeout = STD_SMTP_TIMEOUT);
-static int USmtpSendCommand(BSOCK_HANDLE hBSock, const char *pszCommand,
+static int USmtpSendCommand(BSOCK_HANDLE hBSock, char const *pszCommand,
 			    char *pszResponse, int iMaxResponse, int iTimeout = STD_SMTP_TIMEOUT);
-static int USmtpGetServerAuthFile(char const *pszServer, char *pszAuthFilePath);
-static int USmtpDoPlainAuth(SmtpChannel *pSmtpCh, char const *pszServer,
-			    char const *const *ppszAuthTokens, SMTPError *pSMTPE);
-static int USmtpDoLoginAuth(SmtpChannel *pSmtpCh, char const *pszServer,
-			    char const *const *ppszAuthTokens, SMTPError *pSMTPE);
-static int USmtpDoCramMD5Auth(SmtpChannel *pSmtpCh, char const *pszServer,
-			      char const *const *ppszAuthTokens, SMTPError *pSMTPE);
-static int USmtpServerAuthenticate(SmtpChannel *pSmtpCh, char const *pszServer,
-				   SMTPError *pSMTPE);
-static int USmtpParseEhloResponse(SmtpChannel *pSmtpCh, char const *pszResponse);
-static int USmtpSslEnvCB(void *pPrivate, int iID, void const *pData);
-static int USmtpSwitchToSSL(SmtpChannel *pSmtpCh, SMTPGateway const *pGw, SMTPError *pSMTPE);
-static void USmtpCleanEHLO(SmtpChannel *pSmtpCh);
-static void USmtpFreeChannel(SmtpChannel *pSmtpCh);
-static SMTPGateway *USmtpGetDefaultGateway(SVRCFG_HANDLE hSvrConfig, const char *pszServer);
-static int USmtpGetDomainMX(SVRCFG_HANDLE hSvrConfig, const char *pszDomain, char *&pszMXDomains);
-static char *USmtpGetSpammersFilePath(char *pszSpamFilePath, int iMaxPath);
-static char *USmtpGetSpamAddrFilePath(char *pszSpamFilePath, int iMaxPath);
 
 static char *USmtpGetGwTableFilePath(char *pszGwFilePath, int iMaxPath)
 {
@@ -265,7 +230,7 @@ void USmtpFreeGateways(SMTPGateway **ppGws)
 }
 
 SMTPGateway **USmtpGetCfgGateways(SVRCFG_HANDLE hSvrConfig,  char const * const *ppszGwHosts,
-				  const char *pszOptions)
+				  char const *pszOptions)
 {
 	int i = 0;
 	char *pszCfgOptions;
@@ -285,7 +250,7 @@ SMTPGateway **USmtpGetCfgGateways(SVRCFG_HANDLE hSvrConfig,  char const * const
 	return ppGws;
 }
 
-SMTPGateway **USmtpGetFwdGateways(SVRCFG_HANDLE hSvrConfig, const char *pszDomain)
+SMTPGateway **USmtpGetFwdGateways(SVRCFG_HANDLE hSvrConfig, char const *pszDomain)
 {
 	char szFwdFilePath[SYS_MAX_PATH] = "";
 
@@ -324,20 +289,8 @@ SMTPGateway **USmtpGetFwdGateways(SVRCFG_HANDLE hSvrConfig, const char *pszDomai
 
 			if (ppszStrings[fwdGateway][0] == '#') {
 				if ((ppszFwdGws =
-				     StrTokenize(ppszStrings[fwdGateway] + 1, ";")) != NULL) {
-					int iGwCount = StrStringsCount(ppszFwdGws);
-
-					srand((unsigned int) time(NULL));
-					for (int i = 0; i < (iGwCount / 2); i++) {
-						int iSwap1 = rand() % iGwCount;
-						int iSwap2 = rand() % iGwCount;
-						char *pszGw1 = ppszFwdGws[iSwap1];
-						char *pszGw2 = ppszFwdGws[iSwap2];
-
-						ppszFwdGws[iSwap1] = pszGw2;
-						ppszFwdGws[iSwap2] = pszGw1;
-					}
-				}
+				     StrTokenize(ppszStrings[fwdGateway] + 1, ";")) != NULL)
+					MscRandomizeStringsOrder(ppszFwdGws);
 			} else
 				ppszFwdGws = StrTokenize(ppszStrings[fwdGateway], ";");
 			if (ppszFwdGws != NULL) {
@@ -369,7 +322,7 @@ static char *USmtpGetRelayFilePath(char *pszRelayFilePath, int iMaxPath)
 	return pszRelayFilePath;
 }
 
-int USmtpGetGateway(SVRCFG_HANDLE hSvrConfig, const char *pszDomain, char *pszGateway,
+int USmtpGetGateway(SVRCFG_HANDLE hSvrConfig, char const *pszDomain, char *pszGateway,
 		    int iSize)
 {
 	char szGwFilePath[SYS_MAX_PATH] = "";
@@ -421,7 +374,7 @@ int USmtpGetGateway(SVRCFG_HANDLE hSvrConfig, const char *pszDomain, char *pszGa
 	return ERR_SMTPGW_NOT_FOUND;
 }
 
-static int USmtpWriteGateway(FILE *pGwFile, const char *pszDomain, const char *pszGateway)
+static int USmtpWriteGateway(FILE *pGwFile, char const *pszDomain, char const *pszGateway)
 {
 	/* Domain */
 	char *pszQuoted = StrQuote(pszDomain, '"');
@@ -444,7 +397,7 @@ static int USmtpWriteGateway(FILE *pGwFile, const char *pszDomain, const char *p
 	return 0;
 }
 
-int USmtpAddGateway(const char *pszDomain, const char *pszGateway)
+int USmtpAddGateway(char const *pszDomain, char const *pszGateway)
 {
 	char szGwFilePath[SYS_MAX_PATH] = "";
 
@@ -499,13 +452,13 @@ int USmtpAddGateway(const char *pszDomain, const char *pszGateway)
 	return 0;
 }
 
-int USmtpRemoveGateway(const char *pszDomain)
+int USmtpRemoveGateway(char const *pszDomain)
 {
 	char szGwFilePath[SYS_MAX_PATH] = "";
 	char szTmpFile[SYS_MAX_PATH] = "";
 
 	USmtpGetGwTableFilePath(szGwFilePath, sizeof(szGwFilePath));
-	SysGetTmpFile(szTmpFile);
+	UsrGetTmpFile(NULL, szTmpFile, sizeof(szTmpFile));
 
 	char szResLock[SYS_MAX_PATH] = "";
 	RLCK_HANDLE hResLock = RLckLockEX(CfgGetBasedPath(szGwFilePath, szResLock,
@@ -627,9 +580,9 @@ int USmtpIsAllowedRelay(const SYS_INET_ADDR &PeerInfo, SVRCFG_HANDLE hSvrConfig)
 	return ERR_RELAY_NOT_ALLOWED;
 }
 
-char **USmtpGetPathStrings(const char *pszMailCmd)
+char **USmtpGetPathStrings(char const *pszMailCmd)
 {
-	const char *pszOpen, *pszClose;
+	char const *pszOpen, *pszClose;
 
 	if ((pszOpen = strchr(pszMailCmd, '<')) == NULL ||
 	    (pszClose = strchr(pszOpen + 1, '>')) == NULL) {
@@ -643,6 +596,14 @@ char **USmtpGetPathStrings(const char *pszMailCmd)
 		ErrSetErrorCode(ERR_SMTP_PATH_PARSE_ERROR, pszMailCmd);
 		return NULL;
 	}
+	/*
+	 * USmlValidAddress() will fail with an empty address, but
+	 * USmtpGetPathStrings() should succeed anyway (returning an empty
+	 * recipient path array), so we check for length before calling
+	 * USmlValidAddress().
+	 */
+	if (iPathLength > 0 && USmlValidAddress(pszOpen + 1, pszClose) < 0)
+		return NULL;
 
 	char *pszPath = (char *) SysAlloc(iPathLength + 1);
 
@@ -657,12 +618,12 @@ char **USmtpGetPathStrings(const char *pszMailCmd)
 	return ppszDomains;
 }
 
-int USmtpSplitEmailAddr(const char *pszAddr, char *pszUser, char *pszDomain)
+int USmtpSplitEmailAddr(char const *pszAddr, char *pszUser, char *pszDomain)
 {
 	if (USmlValidAddress(pszAddr, pszAddr + strlen(pszAddr)) < 0)
 		return ErrGetErrorCode();
 
-	const char *pszAT = strchr(pszAddr, '@');
+	char const *pszAT = strchr(pszAddr, '@');
 
 	if (pszAT == NULL) {
 		ErrSetErrorCode(ERR_BAD_EMAIL_ADDR);
@@ -693,23 +654,29 @@ int USmtpSplitEmailAddr(const char *pszAddr, char *pszUser, char *pszDomain)
 
 int USmtpCheckAddressPart(char const *pszName)
 {
-	for (; *pszName; pszName++)
-		if (*pszName <= ' ' || *pszName == 127 ||
-		    strchr(RFC_SPECIALS, *pszName) != NULL) {
-			ErrSetErrorCode(ERR_BAD_RFCNAME);
-			return ERR_BAD_RFCNAME;
-		}
+	char const *pszTop = pszName + strlen(pszName);
+
+	if (USmlDotAtom(pszName, pszTop) != pszTop) {
+		ErrSetErrorCode(ERR_BAD_RFCNAME);
+		return ERR_BAD_RFCNAME;
+	}
 
 	return 0;
 }
 
+int USmtpCheckDomainPart(char const *pszName)
+{
+	return USmlValidHost(pszName, pszName + strlen(pszName));
+}
+
 int USmtpCheckAddress(char const *pszAddress)
 {
 	char szUser[MAX_ADDR_NAME] = "";
 	char szDomain[MAX_ADDR_NAME] = "";
 
 	if (USmtpSplitEmailAddr(pszAddress, szUser, szDomain) < 0 ||
-	    USmtpCheckAddressPart(szUser) < 0 || USmtpCheckAddressPart(szDomain) < 0)
+	    USmtpCheckAddressPart(szUser) < 0 ||
+	    USmtpCheckDomainPart(szDomain) < 0)
 		return ErrGetErrorCode();
 
 	return 0;
@@ -810,7 +777,7 @@ static int USmtpResponseClass(int iResponseCode, int iResponseClass)
 
 }
 
-static int USmtpGetResultCode(const char *pszResult)
+static int USmtpGetResultCode(char const *pszResult)
 {
 	int i;
 	char szResCode[64] = "";
@@ -866,7 +833,7 @@ static int USmtpGetResponse(BSOCK_HANDLE hBSock, char *pszResponse, int iMaxResp
 	return iResultCode;
 }
 
-static int USmtpSendCommand(BSOCK_HANDLE hBSock, const char *pszCommand,
+static int USmtpSendCommand(BSOCK_HANDLE hBSock, char const *pszCommand,
 			    char *pszResponse, int iMaxResponse, int iTimeout)
 {
 	if (BSckSendString(hBSock, pszCommand, iTimeout) <= 0)
@@ -884,15 +851,13 @@ static int USmtpGetServerAuthFile(char const *pszServer, char *pszAuthFilePath)
 
 	char const *pszDot = pszServer;
 
-	while ((pszDot != NULL) && (strlen(pszDot) > 0)) {
+	while (pszDot != NULL && strlen(pszDot) > 0) {
 		if (iRootedName)
 			sprintf(pszAuthFilePath, "%s%stab", szAuthPath, pszDot);
 		else
 			sprintf(pszAuthFilePath, "%s%s.tab", szAuthPath, pszDot);
-
 		if (SysExistFile(pszAuthFilePath))
 			return 0;
-
 		if ((pszDot = strchr(pszDot, '.')) != NULL)
 			++pszDot;
 	}
@@ -1222,7 +1187,21 @@ static int USmtpSwitchToSSL(SmtpChannel *pSmtpCh, SMTPGateway const *pGw, SMTPEr
 	return iError;
 }
 
-SMTPCH_HANDLE USmtpCreateChannel(SMTPGateway const *pGw, const char *pszDomain, SMTPError *pSMTPE)
+static void USmtpCleanEHLO(SmtpChannel *pSmtpCh)
+{
+	pSmtpCh->ulFlags &= ~(SMTPCH_SUPPORT_SIZE | SMTPCH_SUPPORT_TLS);
+	pSmtpCh->ulMaxMsgSize = 0;
+}
+
+static void USmtpFreeChannel(SmtpChannel *pSmtpCh)
+{
+	BSckDetach(pSmtpCh->hBSock, 1);
+	SysFree(pSmtpCh->pszServer);
+	SysFree(pSmtpCh->pszDomain);
+	SysFree(pSmtpCh);
+}
+
+SMTPCH_HANDLE USmtpCreateChannel(SMTPGateway const *pGw, char const *pszDomain, SMTPError *pSMTPE)
 {
 	/* Decode server address */
 	int iPortNo = STD_SMTP_PORT;
@@ -1371,20 +1350,6 @@ SendHELO:
 	return (SMTPCH_HANDLE) pSmtpCh;
 }
 
-static void USmtpCleanEHLO(SmtpChannel *pSmtpCh)
-{
-	pSmtpCh->ulFlags &= ~(SMTPCH_SUPPORT_SIZE | SMTPCH_SUPPORT_TLS);
-	pSmtpCh->ulMaxMsgSize = 0;
-}
-
-static void USmtpFreeChannel(SmtpChannel *pSmtpCh)
-{
-	BSckDetach(pSmtpCh->hBSock, 1);
-	SysFree(pSmtpCh->pszServer);
-	SysFree(pSmtpCh->pszDomain);
-	SysFree(pSmtpCh);
-}
-
 int USmtpCloseChannel(SMTPCH_HANDLE hSmtpCh, int iHardClose, SMTPError *pSMTPE)
 {
 	SmtpChannel *pSmtpCh = (SmtpChannel *) hSmtpCh;
@@ -1438,7 +1403,7 @@ int USmtpChannelReset(SMTPCH_HANDLE hSmtpCh, SMTPError *pSMTPE)
 	return 0;
 }
 
-int USmtpSendMail(SMTPCH_HANDLE hSmtpCh, const char *pszFrom, const char *pszRcpt,
+int USmtpSendMail(SMTPCH_HANDLE hSmtpCh, char const *pszFrom, char const *pszRcpt,
 		  FileSection const *pFS, SMTPError *pSMTPE)
 {
 	SmtpChannel *pSmtpCh = (SmtpChannel *) hSmtpCh;
@@ -1468,8 +1433,8 @@ int USmtpSendMail(SMTPCH_HANDLE hSmtpCh, const char *pszFrom, const char *pszRcp
 		if (llMessageSize == 0 && MscGetSectionSize(pFS, &llMessageSize) < 0)
 			return ErrGetErrorCode();
 
-		SysSNPrintf(szRTXBuffer, sizeof(szRTXBuffer) - 1, "MAIL FROM:<%s> SIZE=" SYS_OFFT_FMT "u",
-			    pszFrom, llMessageSize);
+		SysSNPrintf(szRTXBuffer, sizeof(szRTXBuffer) - 1,
+			    "MAIL FROM:<%s> SIZE=" SYS_OFFT_FMT, pszFrom, llMessageSize);
 	} else
 		SysSNPrintf(szRTXBuffer, sizeof(szRTXBuffer) - 1, "MAIL FROM:<%s>", pszFrom);
 
@@ -1540,8 +1505,8 @@ int USmtpSendMail(SMTPCH_HANDLE hSmtpCh, const char *pszFrom, const char *pszRcp
 	return 0;
 }
 
-int USmtpSendMail(SMTPGateway const *pGw, const char *pszDomain, const char *pszFrom,
-		  const char *pszRcpt, FileSection const *pFS, SMTPError *pSMTPE)
+int USmtpSendMail(SMTPGateway const *pGw, char const *pszDomain, char const *pszFrom,
+		  char const *pszRcpt, FileSection const *pFS, SMTPError *pSMTPE)
 {
 	/* Set server host name inside the SMTP error structure */
 	if (pSMTPE != NULL)
@@ -1560,7 +1525,7 @@ int USmtpSendMail(SMTPGateway const *pGw, const char *pszDomain, const char *psz
 	return iSendResult;
 }
 
-static SMTPGateway *USmtpGetDefaultGateway(SVRCFG_HANDLE hSvrConfig, const char *pszServer)
+static SMTPGateway *USmtpGetDefaultGateway(SVRCFG_HANDLE hSvrConfig, char const *pszServer)
 {
 	SMTPGateway *pGw;
 	char *pszCfgOptions;
@@ -1579,8 +1544,8 @@ static SMTPGateway *USmtpGetDefaultGateway(SVRCFG_HANDLE hSvrConfig, const char
 	return pGw;
 }
 
-int USmtpMailRmtDeliver(SVRCFG_HANDLE hSvrConfig, const char *pszServer, const char *pszDomain,
-			const char *pszFrom, const char *pszRcpt, FileSection const *pFS,
+int USmtpMailRmtDeliver(SVRCFG_HANDLE hSvrConfig, char const *pszServer, char const *pszDomain,
+			char const *pszFrom, char const *pszRcpt, FileSection const *pFS,
 			SMTPError *pSMTPE)
 {
 	int iError;
@@ -1621,16 +1586,18 @@ char *USmtpBuildRcptPath(char const *const *ppszRcptTo, SVRCFG_HANDLE hSvrConfig
 
 	char *pszRcptPath = (char *) SysAlloc(strlen(pszSendRcpt) + strlen(szSpecMXHost) + 2);
 
-	if (iRcptCount == 1)
-		sprintf(pszRcptPath, "%s:%s", szSpecMXHost, pszSendRcpt);
-	else
-		sprintf(pszRcptPath, "%s,%s", szSpecMXHost, pszSendRcpt);
+	if (pszRcptPath != NULL) {
+		if (iRcptCount == 1)
+			sprintf(pszRcptPath, "%s:%s", szSpecMXHost, pszSendRcpt);
+		else
+			sprintf(pszRcptPath, "%s,%s", szSpecMXHost, pszSendRcpt);
+	}
 	SysFree(pszSendRcpt);
 
 	return pszRcptPath;
 }
 
-SMTPGateway **USmtpGetMailExchangers(SVRCFG_HANDLE hSvrConfig, const char *pszDomain)
+SMTPGateway **USmtpGetMailExchangers(SVRCFG_HANDLE hSvrConfig, char const *pszDomain)
 {
 	/* Try to get default gateways */
 	char *pszDefaultGws = SvrGetConfigVar(hSvrConfig, "DefaultSMTPGateways");
@@ -1690,7 +1657,7 @@ SMTPGateway **USmtpGetMailExchangers(SVRCFG_HANDLE hSvrConfig, const char *pszDo
 	return ppGws;
 }
 
-static int USmtpGetDomainMX(SVRCFG_HANDLE hSvrConfig, const char *pszDomain, char *&pszMXDomains)
+static int USmtpGetDomainMX(SVRCFG_HANDLE hSvrConfig, char const *pszDomain, char *&pszMXDomains)
 {
 	int iResult;
 	char *pszSmartDNS = SvrGetConfigVar(hSvrConfig, "SmartDNSHost");
@@ -1718,7 +1685,7 @@ int USmtpCheckMailDomain(SVRCFG_HANDLE hSvrConfig, char const *pszDomain)
 	return 0;
 }
 
-MXS_HANDLE USmtpGetMXFirst(SVRCFG_HANDLE hSvrConfig, const char *pszDomain, char *pszMXHost)
+MXS_HANDLE USmtpGetMXFirst(SVRCFG_HANDLE hSvrConfig, char const *pszDomain, char *pszMXHost)
 {
 	/* Make a DNS query for domain MXs */
 	char *pszMXHosts = NULL;
