@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -47,11 +47,11 @@
 
 enum SysFileTypes {
 	ftNormal = 1,
-		ftDirectory,
-		ftLink,
-		ftOther,
+	ftDirectory,
+	ftLink,
+	ftOther,
 
-		ftMax
+	ftMax
 };
 
 struct SYS_FILE_INFO {
@@ -103,7 +103,6 @@ int SysGetSockInfo(SYS_SOCKET SockFD, SYS_INET_ADDR &AddrInfo);
 char *SysInetNToA(SYS_INET_ADDR const &AddrInfo, char *pszIP, int iSize);
 char *SysInetRevNToA(SYS_INET_ADDR const &AddrInfo, char *pszRevIP, int iSize);
 void const *SysInetAddrData(SYS_INET_ADDR const &AddrInfo, int *piSize);
-int SysSameAddress(SYS_INET_ADDR const &NetAddr1, SYS_INET_ADDR const &NetAddr2);
 int SysInetIPV6CompatIPV4(SYS_INET_ADDR const &Addr);
 int SysInetIPV6ToIPV4(SYS_INET_ADDR const &SAddr, SYS_INET_ADDR &DAddr);
 int SysInetAddrMatch(SYS_INET_ADDR const &Addr, SYS_UINT8 const *pMask, int iMaskSize,
@@ -129,6 +128,19 @@ int SysSetEvent(SYS_EVENT hEvent);
 int SysResetEvent(SYS_EVENT hEvent);
 int SysTryWaitEvent(SYS_EVENT hEvent);
 
+/*
+ * Why? On Unix, the best implementation for events is based on the pthread
+ * API, which cannot be waited together with file/socket descriptors (unless
+ * you do use separate wait threads, which is a lot uglier than having two
+ * event APIs).
+ */
+SYS_PEVENT SysCreatePEvent(int iManualReset);
+int SysClosePEvent(SYS_PEVENT hPEvent);
+int SysWaitPEvent(SYS_PEVENT hPEvent, int iTimeout);
+int SysSetPEvent(SYS_PEVENT hPEvent);
+int SysResetPEvent(SYS_PEVENT hPEvent);
+int SysTryWaitPEvent(SYS_PEVENT hPEvent);
+
 SYS_THREAD SysCreateThread(unsigned int (*pThreadProc) (void *), void *pThreadData);
 void SysCloseThread(SYS_THREAD ThreadID, int iForce);
 int SysSetThreadPriority(SYS_THREAD ThreadID, int iPriority);
@@ -136,7 +148,8 @@ int SysWaitThread(SYS_THREAD ThreadID, int iTimeout);
 unsigned long SysGetCurrentThreadId(void);
 int SysExec(char const *pszCommand, char const *const *pszArgs, int iWaitTimeout = 0,
 	    int iPriority = SYS_PRIORITY_NORMAL, int *piExitStatus = NULL);
-void SysSetBreakHandler(void (*BreakHandler) (void));
+void SysSetBreakHandler(void (*pBreakHandler) (void));
+unsigned long SysGetCurrentProcessId(void);
 
 int SysCreateTlsKey(SYS_TLSKEY &TlsKey, void (*pFreeProc) (void *) = NULL);
 int SysDeleteTlsKey(SYS_TLSKEY &TlsKey);
@@ -150,8 +163,8 @@ void *SysAlloc(unsigned int uSize);
 void SysFree(void *pData);
 void *SysRealloc(void *pData, unsigned int uSize);
 
-int SysLockFile(const char *pszFileName, char const *pszLockExt = ".lock");
-int SysUnlockFile(const char *pszFileName, char const *pszLockExt = ".lock");
+int SysLockFile(char const *pszFileName, char const *pszLockExt = ".lock");
+int SysUnlockFile(char const *pszFileName, char const *pszLockExt = ".lock");
 
 SYS_HANDLE SysOpenModule(char const *pszFilePath);
 int SysCloseModule(SYS_HANDLE hModule);
@@ -160,24 +173,24 @@ void *SysGetSymbol(SYS_HANDLE hModule, char const *pszSymbol);
 int SysEventLogV(int iLogLevel, char const *pszFormat, va_list Args);
 int SysEventLog(int iLogLevel, char const *pszFormat, ...);
 int SysLogMessage(int iLogLevel, char const *pszFormat, ...);
-void SysSleep(int iTimeout);
-void SysMsSleep(int iMsTimeout);
+int SysSleep(int iTimeout);
+int SysMsSleep(int iMsTimeout);
 SYS_INT64 SysMsTime(void);
-int SysExistFile(const char *pszFilePath);
-int SysExistDir(const char *pszDirPath);
-SYS_HANDLE SysFirstFile(const char *pszPath, char *pszFileName, int iSize);
+int SysExistFile(char const *pszFilePath);
+int SysExistDir(char const *pszDirPath);
+SYS_HANDLE SysFirstFile(char const *pszPath, char *pszFileName, int iSize);
 int SysIsDirectory(SYS_HANDLE hFind);
 SYS_OFF_T SysGetSize(SYS_HANDLE hFind);
 int SysNextFile(SYS_HANDLE hFind, char *pszFileName, int iSize);
 void SysFindClose(SYS_HANDLE hFind);
 int SysGetFileInfo(char const *pszFileName, SYS_FILE_INFO &FI);
 int SysSetFileModTime(char const *pszFileName, time_t tMod);
-char *SysStrDup(const char *pszString);
-char *SysGetEnv(const char *pszVarName);
-char *SysGetTmpFile(char *pszFileName);
-int SysRemove(const char *pszFileName);
-int SysMakeDir(const char *pszPath);
-int SysRemoveDir(const char *pszPath);
+char *SysStrDup(char const *pszString);
+char *SysGetEnv(char const *pszVarName);
+char *SysGetTempDir(char *pszPath, int iMaxPath);
+int SysRemove(char const *pszFileName);
+int SysMakeDir(char const *pszPath);
+int SysRemoveDir(char const *pszPath);
 int SysMoveFile(char const *pszOldName, char const *pszNewName);
 
 int SysVSNPrintf(char *pszBuffer, int iSize, char const *pszFormat, va_list Args);
