@@ -1,6 +1,6 @@
 /*
- *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,..,2004  Davide Libenzi
+ *  XMail by Davide Libenzi (Intranet and Internet mail server)
+ *  Copyright (C) 1999,..,2010  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -81,22 +81,6 @@ struct MMapData {
 	unsigned long ulFlags;
 };
 
-static void SysInitTlsKeyEntries(void);
-static void SysInitTlsKeys(void);
-static void SysCleanupTlsKeys(void);
-static int SysSetServerName(void);
-static int SysCrtReportHook(int iType, char *pszMsg, int *piRetVal);
-static void SysRunThreadExitHooks(SYS_THREAD ThreadID, int iMode);
-static int SysSetSocketsOptions(SYS_SOCKET SockFD);
-static int SysRecvLL(SYS_SOCKET SockFD, char *pszBuffer, int iBufferSize);
-static int SysSendLL(SYS_SOCKET SockFD, char const *pszBuffer, int iBufferSize);
-static unsigned int SysThreadRunner(void *pRunData);
-static int SysThreadSetup(SYS_THREAD ThreadID);
-static int SysThreadCleanup(SYS_THREAD ThreadID);
-static BOOL WINAPI SysBreakHandlerRoutine(DWORD dwCtrlType);
-static void SysTimetToFileTime(time_t tTime, LPFILETIME pFT);
-static time_t SysFileTimeToTimet(LPFILETIME pFT);
-
 static int iNumThExitHooks;
 static ThreadExitHook ThExitHooks[SYS_MAX_TH_EXIT_HOOKS];
 static char szServerName[SYS_MAX_PATH];
@@ -109,7 +93,7 @@ static SYS_INT64 PCFreq;
 static SYS_INT64 PCSysStart;
 static int iSndBufSize = -1, iRcvBufSize = -1;
 static CRITICAL_SECTION csLog;
-static void (*SysBreakHandler) (void) = NULL;
+static void (*pSysBreakHandler) (void) = NULL;
 static HANDLE hShutdownEvent = NULL;
 
 static void SysInitTlsKeyEntries(void)
@@ -130,7 +114,6 @@ static void SysInitTlsKeys(void)
 static void SysCleanupTlsKeys(void)
 {
 	EnterCriticalSection(&csTLS);
-
 	for (int i = 0; i < MAX_TLS_KEYS; i++) {
 		if (TlsKeyEntries[i].pFreeProc != UNUSED_TLS_KEY_PROC &&
 		    TlsKeyEntries[i].pFreeProc != NULL)
@@ -138,7 +121,6 @@ static void SysCleanupTlsKeys(void)
 
 		TlsKeys[i].pData = NULL;
 	}
-
 	LeaveCriticalSection(&csTLS);
 }
 
@@ -193,26 +175,50 @@ static int SysCrtReportHook(int iType, char *pszMsg, int *piRetVal)
 	default:
 		return FALSE;
 	}
-
 	SysLogMessage(iLogLevel, "%s: %s\n", pszType, pszMsg);
 
 	return iRetCode;
 }
 
+static void SysRunThreadExitHooks(SYS_THREAD ThreadID, int iMode)
+{
+	int i;
+
+	for (i = iNumThExitHooks - 1; i >= 0; i--)
+		(*ThExitHooks[i].pfHook)(ThExitHooks[i].pPrivate, ThreadID, iMode);
+}
+
+static int SysThreadSetup(SYS_THREAD ThreadID)
+{
+	SysInitTlsKeys();
+	SysRunThreadExitHooks(ThreadID, SYS_THREAD_ATTACH);
+
+	return 0;
+}
+
+static int SysThreadCleanup(SYS_THREAD ThreadID)
+{
+	SysRunThreadExitHooks(ThreadID, SYS_THREAD_DETACH);
+	SysCleanupTlsKeys();
+
+	return 0;
+}
+
 int SysInitLibrary(void)
 {
 	/* Setup timers */
 	LARGE_INTEGER PerfCntFreq;
 	LARGE_INTEGER PerfCntCurr;
 
+	_tzset();
+
 	QueryPerformanceFrequency(&PerfCntFreq);
 	QueryPerformanceCounter(&PerfCntCurr);
 
-	PCFreq = *(SYS_INT64 *) &PerfCntFreq;
+	PCFreq = (SYS_INT64) PerfCntFreq.QuadPart;
 	PCFreq /= 1000;
-	PCSysStart = *(SYS_INT64 *) &PerfCntCurr;
+	PCSysStart = (SYS_INT64) PerfCntCurr.QuadPart;
 
-	_tzset();
 	time(&tSysStart);
 
 	iNumThExitHooks = 0;
@@ -249,6 +255,7 @@ int SysInitLibrary(void)
 		DeleteCriticalSection(&csTLS);
 		return ErrorPop();
 	}
+	SRand();
 
 	return 0;
 }
@@ -274,14 +281,6 @@ int SysAddThreadExitHook(void (*pfHook)(void *, SYS_THREAD, int), void *pPrivate
 	return 0;
 }
 
-static void SysRunThreadExitHooks(SYS_THREAD ThreadID, int iMode)
-{
-	int i;
-
-	for (i = iNumThExitHooks - 1; i >= 0; i--)
-		(*ThExitHooks[i].pfHook)(ThExitHooks[i].pPrivate, ThreadID, iMode);
-}
-
 int SysShutdownLibrary(int iMode)
 {
 	SetEvent(hShutdownEvent);
@@ -299,6 +298,45 @@ int SysSetupSocketBuffers(int *piSndBufSize, int *piRcvBufSize)
 	return 0;
 }
 
+static int SysSetSocketsOptions(SYS_SOCKET SockFD)
+{
+	/* Set socket buffer sizes */
+	if (iSndBufSize > 0) {
+		int iSize = iSndBufSize;
+
+		setsockopt((int) SockFD, SOL_SOCKET, SO_SNDBUF, (char const *) &iSize,
+			   sizeof(iSize));
+	}
+	if (iRcvBufSize > 0) {
+		int iSize = iRcvBufSize;
+
+		setsockopt((int) SockFD, SOL_SOCKET, SO_RCVBUF, (char const *) &iSize,
+			   sizeof(iSize));
+	}
+
+	int iActivate = 1;
+
+	if (setsockopt(SockFD, SOL_SOCKET, SO_REUSEADDR, (char const *) &iActivate,
+		       sizeof(iActivate)) != 0) {
+		ErrSetErrorCode(ERR_SETSOCKOPT);
+		return ERR_SETSOCKOPT;
+	}
+	/* Disable linger */
+	struct linger Ling;
+
+	ZeroData(Ling);
+	Ling.l_onoff = 0;
+	Ling.l_linger = 0;
+
+	setsockopt(SockFD, SOL_SOCKET, SO_LINGER, (char const *) &Ling, sizeof(Ling));
+
+	/* Set KEEPALIVE if supported */
+	setsockopt(SockFD, SOL_SOCKET, SO_KEEPALIVE, (char const *) &iActivate,
+		   sizeof(iActivate));
+
+	return 0;
+}
+
 SYS_SOCKET SysCreateSocket(int iAddressFamily, int iType, int iProtocol)
 {
 	SOCKET SockFD = WSASocket(iAddressFamily, iType, iProtocol, NULL, 0,
@@ -334,48 +372,10 @@ int SysBlockSocket(SYS_SOCKET SockFD, int iBlocking)
 	return 0;
 }
 
-static int SysSetSocketsOptions(SYS_SOCKET SockFD)
-{
-	/* Set socket buffer sizes */
-	if (iSndBufSize > 0) {
-		int iSize = iSndBufSize;
-
-		setsockopt((int) SockFD, SOL_SOCKET, SO_SNDBUF, (const char *) &iSize,
-			   sizeof(iSize));
-	}
-	if (iRcvBufSize > 0) {
-		int iSize = iRcvBufSize;
-
-		setsockopt((int) SockFD, SOL_SOCKET, SO_RCVBUF, (const char *) &iSize,
-			   sizeof(iSize));
-	}
-
-	int iActivate = 1;
-
-	if (setsockopt(SockFD, SOL_SOCKET, SO_REUSEADDR, (const char *) &iActivate,
-		       sizeof(iActivate)) != 0) {
-		ErrSetErrorCode(ERR_SETSOCKOPT);
-		return ERR_SETSOCKOPT;
-	}
-	/* Disable linger */
-	struct linger Ling;
-
-	ZeroData(Ling);
-	Ling.l_onoff = 0;
-	Ling.l_linger = 0;
-
-	setsockopt(SockFD, SOL_SOCKET, SO_LINGER, (const char *) &Ling, sizeof(Ling));
-
-	/* Set KEEPALIVE if supported */
-	setsockopt(SockFD, SOL_SOCKET, SO_KEEPALIVE, (const char *) &iActivate,
-		   sizeof(iActivate));
-
-	return 0;
-}
-
 void SysCloseSocket(SYS_SOCKET SockFD)
 {
-	closesocket(SockFD);
+	if (SockFD != SYS_INVALID_SOCKET)
+		closesocket(SockFD);
 }
 
 int SysShutdownSocket(SYS_SOCKET SockFD, int iHow)
@@ -414,8 +414,8 @@ static int SysRecvLL(SYS_SOCKET SockFD, char *pszBuffer, int iBufferSize)
 	WSABuff.len = iBufferSize;
 	WSABuff.buf = pszBuffer;
 
-	return (WSARecv(SockFD, &WSABuff, 1, &dwRtxBytes, &dwRtxFlags,
-			NULL, NULL) == 0) ? (int) dwRtxBytes: -WSAGetLastError();
+	return WSARecv(SockFD, &WSABuff, 1, &dwRtxBytes, &dwRtxFlags,
+		       NULL, NULL) == 0 ? (int) dwRtxBytes: -WSAGetLastError();
 }
 
 static int SysSendLL(SYS_SOCKET SockFD, char const *pszBuffer, int iBufferSize)
@@ -427,8 +427,8 @@ static int SysSendLL(SYS_SOCKET SockFD, char const *pszBuffer, int iBufferSize)
 	WSABuff.len = iBufferSize;
 	WSABuff.buf = (char *) pszBuffer;
 
-	return (WSASend(SockFD, &WSABuff, 1, &dwRtxBytes, 0,
-			NULL, NULL) == 0) ? (int) dwRtxBytes: -WSAGetLastError();
+	return WSASend(SockFD, &WSABuff, 1, &dwRtxBytes, 0,
+		       NULL, NULL) == 0 ? (int) dwRtxBytes: -WSAGetLastError();
 }
 
 int SysRecvData(SYS_SOCKET SockFD, char *pszBuffer, int iBufferSize, int iTimeout)
@@ -447,7 +447,7 @@ int SysRecvData(SYS_SOCKET SockFD, char *pszBuffer, int iBufferSize, int iTimeou
 		WSAEventSelect(SockFD, (WSAEVENT) hReadEvent, FD_READ | FD_CLOSE);
 
 		DWORD dwWaitResult = WSAWaitForMultipleEvents(2, hWaitEvents, FALSE,
-							      (DWORD) (iTimeout * 1000), TRUE);
+							      iTimeout, TRUE);
 
 		WSAEventSelect(SockFD, NULL, 0);
 
@@ -517,7 +517,7 @@ int SysRecvDataFrom(SYS_SOCKET SockFD, SYS_INET_ADDR *pFrom, char *pszBuffer,
 		WSAEventSelect(SockFD, (WSAEVENT) hReadEvent, FD_READ | FD_CLOSE);
 
 		DWORD dwWaitResult = WSAWaitForMultipleEvents(2, hWaitEvents, FALSE,
-							      (DWORD) (iTimeout * 1000), TRUE);
+							      iTimeout, TRUE);
 
 		WSAEventSelect(SockFD, NULL, 0);
 
@@ -569,7 +569,7 @@ int SysSendData(SYS_SOCKET SockFD, char const *pszBuffer, int iBufferSize, int i
 		WSAEventSelect(SockFD, (WSAEVENT) hWriteEvent, FD_WRITE | FD_CLOSE);
 
 		DWORD dwWaitResult = WSAWaitForMultipleEvents(2, hWaitEvents, FALSE,
-							      (DWORD) (iTimeout * 1000), TRUE);
+							      iTimeout, TRUE);
 
 		WSAEventSelect(SockFD, NULL, 0);
 
@@ -639,7 +639,7 @@ int SysSendDataTo(SYS_SOCKET SockFD, const SYS_INET_ADDR *pTo,
 		WSAEventSelect(SockFD, (WSAEVENT) hWriteEvent, FD_WRITE | FD_CLOSE);
 
 		DWORD dwWaitResult = WSAWaitForMultipleEvents(2, hWaitEvents, FALSE,
-							      (DWORD) (iTimeout * 1000), TRUE);
+							      iTimeout, TRUE);
 
 		WSAEventSelect(SockFD, NULL, 0);
 
@@ -691,7 +691,7 @@ int SysConnect(SYS_SOCKET SockFD, const SYS_INET_ADDR *pSockName, int iTimeout)
 	if (iConnectResult != 0 && iConnectError == WSAEWOULDBLOCK) {
 		HANDLE hWaitEvents[2] = { hConnectEvent, hShutdownEvent };
 		DWORD dwWaitResult = WSAWaitForMultipleEvents(2, hWaitEvents, FALSE,
-							      (DWORD) (iTimeout * 1000), TRUE);
+							      iTimeout, TRUE);
 
 		if (dwWaitResult == WSA_WAIT_EVENT_0) {
 			WSANETWORKEVENTS NetEvents;
@@ -737,14 +737,14 @@ SYS_SOCKET SysAccept(SYS_SOCKET SockFD, SYS_INET_ADDR *pSockName, int iTimeout)
 	if (SockFDAccept == INVALID_SOCKET && iConnectError == WSAEWOULDBLOCK) {
 		HANDLE hWaitEvents[2] = { hAcceptEvent, hShutdownEvent };
 		DWORD dwWaitResult = WSAWaitForMultipleEvents(2, hWaitEvents, FALSE,
-							      (DWORD) (iTimeout * 1000), TRUE);
+							      iTimeout, TRUE);
 
 		if (dwWaitResult == WSA_WAIT_EVENT_0) {
 			iNameLen = (INT) sizeof(pSockName->Addr);
 			SockFDAccept = WSAAccept(SockFD, (struct sockaddr *) pSockName->Addr,
 						 &iNameLen, NULL, 0);
 		} else if (dwWaitResult == (WSA_WAIT_EVENT_0 + 1))
-				ErrSetErrorCode(ERR_SERVER_SHUTDOWN);
+			ErrSetErrorCode(ERR_SERVER_SHUTDOWN);
 		else
 			ErrSetErrorCode(ERR_TIMEOUT);
 	}
@@ -771,10 +771,13 @@ int SysSelect(int iMaxFD, SYS_fd_set *pReadFDs, SYS_fd_set *pWriteFDs, SYS_fd_se
 	struct timeval TV;
 
 	ZeroData(TV);
-	TV.tv_sec = iTimeout;
-	TV.tv_usec = 0;
+	if (iTimeout != SYS_INFINITE_TIMEOUT) {
+		TV.tv_sec = iTimeout / 1000;
+		TV.tv_usec = (iTimeout % 1000) * 1000;
+	}
 
-	int iSelectResult = select(iMaxFD + 1, pReadFDs, pWriteFDs, pExcptFDs, &TV);
+	int iSelectResult = select(iMaxFD + 1, pReadFDs, pWriteFDs, pExcptFDs,
+				   iTimeout != SYS_INFINITE_TIMEOUT ? &TV: NULL);
 
 	if (iSelectResult < 0) {
 		ErrSetErrorCode(ERR_SELECT);
@@ -821,18 +824,19 @@ int SysSendFile(SYS_SOCKET SockFD, char const *pszFileName, SYS_OFF_T llBaseOffs
 		ErrSetErrorCode(ERR_MAPVIEWOFFILE);
 		return ERR_MAPVIEWOFFILE;
 	}
-	/* Send the file */
-	int iSndBuffSize = MIN_TCP_SEND_SIZE;
+
+	int iSndBuffSize = (iTimeout != SYS_INFINITE_TIMEOUT) ?
+		MIN_TCP_SEND_SIZE: MAX_TCP_SEND_SIZE;
 	SYS_UINT64 ullFileSize = (((SYS_UINT64) dwFileSizeHi) << 32) | (SYS_UINT64) dwFileSizeLo;
 	SYS_UINT64 ullEndOffset = (llEndOffset != -1) ? (SYS_UINT64) llEndOffset: ullFileSize;
 	SYS_UINT64 ullCurrOffset = (SYS_UINT64) llBaseOffset;
 	char *pszBuffer = (char *) pAddress + llBaseOffset;
-	time_t tStart;
+	SYS_INT64 tStart;
 
 	while (ullCurrOffset < ullEndOffset) {
 		int iCurrSend = (int) Min(iSndBuffSize, ullEndOffset - ullCurrOffset);
 
-		tStart = time(NULL);
+		tStart = SysMsTime();
 		if ((iCurrSend = SysSendData(SockFD, pszBuffer, iCurrSend, iTimeout)) < 0) {
 			ErrorPush();
 			UnmapViewOfFile(pAddress);
@@ -840,11 +844,9 @@ int SysSendFile(SYS_SOCKET SockFD, char const *pszFileName, SYS_OFF_T llBaseOffs
 			CloseHandle(hFile);
 			return ErrorPop();
 		}
-
-		if ((((time(NULL) - tStart) * K_IO_TIME_RATIO) < iTimeout) &&
-		    (iSndBuffSize < MAX_TCP_SEND_SIZE))
+		if (iSndBuffSize < MAX_TCP_SEND_SIZE &&
+		    ((SysMsTime() - tStart) * K_IO_TIME_RATIO) < (SYS_INT64) iTimeout)
 			iSndBuffSize = Min(iSndBuffSize * 2, MAX_TCP_SEND_SIZE);
-
 		pszBuffer += iCurrSend;
 		ullCurrOffset += (SYS_UINT64) iCurrSend;
 	}
@@ -869,9 +871,11 @@ SYS_SEMAPHORE SysCreateSemaphore(int iInitCount, int iMaxCount)
 
 int SysCloseSemaphore(SYS_SEMAPHORE hSemaphore)
 {
-	if (!CloseHandle((HANDLE) hSemaphore)) {
-		ErrSetErrorCode(ERR_CLOSEHANDLE);
-		return ERR_CLOSEHANDLE;
+	if (hSemaphore != SYS_INVALID_SEMAPHORE) {
+		if (!CloseHandle((HANDLE) hSemaphore)) {
+			ErrSetErrorCode(ERR_CLOSEHANDLE);
+			return ERR_CLOSEHANDLE;
+		}
 	}
 
 	return 0;
@@ -880,7 +884,7 @@ int SysCloseSemaphore(SYS_SEMAPHORE hSemaphore)
 int SysWaitSemaphore(SYS_SEMAPHORE hSemaphore, int iTimeout)
 {
 	if (WaitForSingleObject((HANDLE) hSemaphore,
-				(DWORD) (iTimeout * 1000)) != WAIT_OBJECT_0) {
+				iTimeout) != WAIT_OBJECT_0) {
 		ErrSetErrorCode(ERR_TIMEOUT);
 		return ERR_TIMEOUT;
 	}
@@ -922,9 +926,11 @@ SYS_MUTEX SysCreateMutex(void)
 
 int SysCloseMutex(SYS_MUTEX hMutex)
 {
-	if (!CloseHandle((HANDLE) hMutex)) {
-		ErrSetErrorCode(ERR_CLOSEHANDLE);
-		return ERR_CLOSEHANDLE;
+	if (hMutex != SYS_INVALID_MUTEX) {
+		if (!CloseHandle((HANDLE) hMutex)) {
+			ErrSetErrorCode(ERR_CLOSEHANDLE);
+			return ERR_CLOSEHANDLE;
+		}
 	}
 
 	return 0;
@@ -933,7 +939,7 @@ int SysCloseMutex(SYS_MUTEX hMutex)
 int SysLockMutex(SYS_MUTEX hMutex, int iTimeout)
 {
 	if (WaitForSingleObject((HANDLE) hMutex,
-				(DWORD) (iTimeout * 1000)) != WAIT_OBJECT_0) {
+				iTimeout) != WAIT_OBJECT_0) {
 		ErrSetErrorCode(ERR_TIMEOUT);
 		return ERR_TIMEOUT;
 	}
@@ -972,9 +978,11 @@ SYS_EVENT SysCreateEvent(int iManualReset)
 
 int SysCloseEvent(SYS_EVENT hEvent)
 {
-	if (!CloseHandle((HANDLE) hEvent)) {
-		ErrSetErrorCode(ERR_CLOSEHANDLE);
-		return ERR_CLOSEHANDLE;
+	if (hEvent != SYS_INVALID_EVENT) {
+		if (!CloseHandle((HANDLE) hEvent)) {
+			ErrSetErrorCode(ERR_CLOSEHANDLE);
+			return ERR_CLOSEHANDLE;
+		}
 	}
 
 	return 0;
@@ -983,7 +991,7 @@ int SysCloseEvent(SYS_EVENT hEvent)
 int SysWaitEvent(SYS_EVENT hEvent, int iTimeout)
 {
 	if (WaitForSingleObject((HANDLE) hEvent,
-				(DWORD) (iTimeout * 1000)) != WAIT_OBJECT_0) {
+				iTimeout) != WAIT_OBJECT_0) {
 		ErrSetErrorCode(ERR_TIMEOUT);
 		return ERR_TIMEOUT;
 	}
@@ -1015,6 +1023,40 @@ int SysTryWaitEvent(SYS_EVENT hEvent)
 	return 0;
 }
 
+/*
+ * On Windows, Events and PEvents maps to the same implementation, since
+ * we use WaitForMultipleObjects() for general waiting.
+ */
+SYS_PEVENT SysCreatePEvent(int iManualReset)
+{
+	return (SYS_PEVENT) SysCreateEvent(iManualReset);
+}
+
+int SysClosePEvent(SYS_PEVENT hPEvent)
+{
+	return SysCloseEvent((SYS_EVENT) hPEvent);
+}
+
+int SysWaitPEvent(SYS_PEVENT hPEvent, int iTimeout)
+{
+	return SysWaitEvent((SYS_EVENT) hPEvent, iTimeout);
+}
+
+int SysSetPEvent(SYS_PEVENT hPEvent)
+{
+	return SysSetEvent((SYS_EVENT) hPEvent);
+}
+
+int SysResetPEvent(SYS_PEVENT hPEvent)
+{
+	return SysResetEvent((SYS_EVENT) hPEvent);
+}
+
+int SysTryWaitPEvent(SYS_PEVENT hPEvent)
+{
+	return SysTryWaitEvent((SYS_EVENT) hPEvent);
+}
+
 static unsigned int SysThreadRunner(void *pRunData)
 {
 	ThreadRunner *pTR = (ThreadRunner *) pRunData;
@@ -1058,27 +1100,13 @@ SYS_THREAD SysCreateThread(unsigned int (*pThreadProc) (void *), void *pThreadDa
 	return (SYS_THREAD) ulThread;
 }
 
-static int SysThreadSetup(SYS_THREAD ThreadID)
-{
-	SysInitTlsKeys();
-	SysRunThreadExitHooks(ThreadID, SYS_THREAD_ATTACH);
-
-	return 0;
-}
-
-static int SysThreadCleanup(SYS_THREAD ThreadID)
-{
-	SysRunThreadExitHooks(ThreadID, SYS_THREAD_DETACH);
-	SysCleanupTlsKeys();
-
-	return 0;
-}
-
 void SysCloseThread(SYS_THREAD ThreadID, int iForce)
 {
-	if (iForce)
-		TerminateThread((HANDLE) ThreadID, (DWORD) -1);
-	CloseHandle((HANDLE) ThreadID);
+	if (ThreadID != SYS_INVALID_THREAD) {
+		if (iForce)
+			TerminateThread((HANDLE) ThreadID, (DWORD) -1);
+		CloseHandle((HANDLE) ThreadID);
+	}
 }
 
 int SysSetThreadPriority(SYS_THREAD ThreadID, int iPriority)
@@ -1109,7 +1137,7 @@ int SysSetThreadPriority(SYS_THREAD ThreadID, int iPriority)
 int SysWaitThread(SYS_THREAD ThreadID, int iTimeout)
 {
 	if (WaitForSingleObject((HANDLE) ThreadID,
-				(DWORD) (iTimeout * 1000)) != WAIT_OBJECT_0) {
+				iTimeout) != WAIT_OBJECT_0) {
 		ErrSetErrorCode(ERR_TIMEOUT);
 		return ERR_TIMEOUT;
 	}
@@ -1158,8 +1186,8 @@ int SysExec(char const *pszCommand, char const *const *pszArgs, int iWaitTimeout
 
 	SysSetThreadPriority((SYS_THREAD) PI.hThread, iPriority);
 
-	if (iWaitTimeout > 0) {
-		if (WaitForSingleObject(PI.hProcess, iWaitTimeout * 1000) != WAIT_OBJECT_0) {
+	if (iWaitTimeout > 0 || iWaitTimeout == SYS_INFINITE_TIMEOUT) {
+		if (WaitForSingleObject(PI.hProcess, iWaitTimeout) != WAIT_OBJECT_0) {
 			CloseHandle(PI.hThread);
 			CloseHandle(PI.hProcess);
 
@@ -1175,9 +1203,8 @@ int SysExec(char const *pszCommand, char const *const *pszArgs, int iWaitTimeout
 			else
 				*piExitStatus = (int) dwExitCode;
 		}
-	} else if (piExitStatus != NULL) {
+	} else if (piExitStatus != NULL)
 		*piExitStatus = -1;
-	}
 	CloseHandle(PI.hThread);
 	CloseHandle(PI.hProcess);
 
@@ -1192,39 +1219,39 @@ static BOOL WINAPI SysBreakHandlerRoutine(DWORD dwCtrlType)
 	case CTRL_C_EVENT:
 	case CTRL_CLOSE_EVENT:
 	case CTRL_SHUTDOWN_EVENT:
-		if (SysBreakHandler != NULL)
-			SysBreakHandler(), bReturnValue = TRUE;
-
+		if (pSysBreakHandler != NULL)
+			(*pSysBreakHandler)(), bReturnValue = TRUE;
 		break;
 	}
 
 	return bReturnValue;
 }
 
-void SysSetBreakHandler(void (*BreakHandler) (void))
+void SysSetBreakHandler(void (*pBreakHandler) (void))
 {
-	if (SysBreakHandler == NULL)
+	if (pSysBreakHandler == NULL)
 		SetConsoleCtrlHandler(SysBreakHandlerRoutine,
-				      (BreakHandler != NULL) ? TRUE: FALSE);
-	SysBreakHandler = BreakHandler;
+				      (pBreakHandler != NULL) ? TRUE: FALSE);
+	pSysBreakHandler = pBreakHandler;
+}
+
+unsigned long SysGetCurrentProcessId(void)
+{
+	return GetCurrentProcessId();
 }
 
 int SysCreateTlsKey(SYS_TLSKEY &TlsKey, void (*pFreeProc) (void *))
 {
 	EnterCriticalSection(&csTLS);
-
 	for (int i = 0; i < MAX_TLS_KEYS; i++) {
 		if (TlsKeyEntries[i].pFreeProc == UNUSED_TLS_KEY_PROC) {
 			TlsKeyEntries[i].pFreeProc = pFreeProc;
-
 			LeaveCriticalSection(&csTLS);
-
 			TlsKey = (SYS_TLSKEY) i;
 
 			return 0;
 		}
 	}
-
 	LeaveCriticalSection(&csTLS);
 
 	ErrSetErrorCode(ERR_NOMORE_TLSKEYS);
@@ -1325,9 +1352,9 @@ void *SysRealloc(void *pData, unsigned int uSize)
 	return pNewData;
 }
 
-int SysLockFile(const char *pszFileName, char const *pszLockExt)
+int SysLockFile(char const *pszFileName, char const *pszLockExt)
 {
-	char szLockFile[SYS_MAX_PATH] = "";
+	char szLockFile[SYS_MAX_PATH];
 
 	SysSNPrintf(szLockFile, sizeof(szLockFile) - 1, "%s%s", pszFileName, pszLockExt);
 
@@ -1346,7 +1373,7 @@ int SysLockFile(const char *pszFileName, char const *pszLockExt)
 	}
 
 	DWORD dwWritten = 0;
-	char szLock[128] = "";
+	char szLock[128];
 
 	sprintf(szLock, "%lu", (unsigned long) GetCurrentThreadId());
 	if (!WriteFile(hFile, szLock, strlen(szLock) + 1, &dwWritten, NULL)) {
@@ -1359,7 +1386,7 @@ int SysLockFile(const char *pszFileName, char const *pszLockExt)
 	return 0;
 }
 
-int SysUnlockFile(const char *pszFileName, char const *pszLockExt)
+int SysUnlockFile(char const *pszFileName, char const *pszLockExt)
 {
 	char szLockFile[SYS_MAX_PATH] = "";
 
@@ -1386,7 +1413,8 @@ SYS_HANDLE SysOpenModule(char const *pszFilePath)
 
 int SysCloseModule(SYS_HANDLE hModule)
 {
-	FreeLibrary((HMODULE) hModule);
+	if (hModule != SYS_INVALID_HANDLE)
+		FreeLibrary((HMODULE) hModule);
 
 	return 0;
 }
@@ -1421,7 +1449,7 @@ int SysEventLogV(int iLogLevel, char const *pszFormat, va_list Args)
 	ReportEvent(hEventSource,
 		    iLogLevel == LOG_LEV_ERROR ? EVENTLOG_ERROR_TYPE:
 		    iLogLevel == LOG_LEV_WARNING ? EVENTLOG_WARNING_TYPE:
-		    EVENTLOG_INFORMATION_TYPE, 0, 0, NULL, 1, 0, (const char **) pszStrings, NULL);
+		    EVENTLOG_INFORMATION_TYPE, 0, 0, NULL, 1, 0, (char const **) pszStrings, NULL);
 
 	DeregisterEventSource(hEventSource);
 
@@ -1430,15 +1458,14 @@ int SysEventLogV(int iLogLevel, char const *pszFormat, va_list Args)
 
 int SysEventLog(int iLogLevel, char const *pszFormat, ...)
 {
+	int iError;
 	va_list Args;
 
 	va_start(Args, pszFormat);
-
-	int iLogResult = SysEventLogV(iLogLevel, pszFormat, Args);
-
+	iError = SysEventLogV(iLogLevel, pszFormat, Args);
 	va_end(Args);
 
-	return 0;
+	return iError;
 }
 
 int SysLogMessage(int iLogLevel, char const *pszFormat, ...)
@@ -1466,14 +1493,20 @@ int SysLogMessage(int iLogLevel, char const *pszFormat, ...)
 	return 0;
 }
 
-void SysSleep(int iTimeout)
+int SysSleep(int iTimeout)
 {
-	SysMsSleep(iTimeout * 1000);
+	return SysMsSleep(iTimeout * 1000);
 }
 
-void SysMsSleep(int iMsTimeout)
+int SysMsSleep(int iMsTimeout)
 {
-	Sleep(iMsTimeout);
+	if (WaitForSingleObject(hShutdownEvent,
+				iMsTimeout) == WAIT_OBJECT_0) {
+		ErrSetErrorCode(ERR_SERVER_SHUTDOWN);
+		return ERR_SERVER_SHUTDOWN;
+	}
+
+	return 0;
 }
 
 static void SysTimetToFileTime(time_t tTime, LPFILETIME pFT)
@@ -1494,20 +1527,19 @@ static time_t SysFileTimeToTimet(LPFILETIME pFT)
 
 SYS_INT64 SysMsTime(void)
 {
+	SYS_INT64 MsTicks;
 	LARGE_INTEGER PerfCntCurr;
 
 	QueryPerformanceCounter(&PerfCntCurr);
-
-	SYS_INT64 MsTicks = *(SYS_INT64 *) &PerfCntCurr;
-
+	MsTicks = (SYS_INT64) PerfCntCurr.QuadPart;
 	MsTicks -= PCSysStart;
 	MsTicks /= PCFreq;
-	MsTicks += (SYS_INT64) tSysStart *1000;
+	MsTicks += (SYS_INT64) tSysStart * 1000;
 
 	return MsTicks;
 }
 
-int SysExistFile(const char *pszFilePath)
+int SysExistFile(char const *pszFilePath)
 {
 	DWORD dwAttr = GetFileAttributes(pszFilePath);
 
@@ -1517,7 +1549,7 @@ int SysExistFile(const char *pszFilePath)
 	return (dwAttr & FILE_ATTRIBUTE_DIRECTORY) ? 0: 1;
 }
 
-int SysExistDir(const char *pszDirPath)
+int SysExistDir(char const *pszDirPath)
 {
 	DWORD dwAttr = GetFileAttributes(pszDirPath);
 
@@ -1527,11 +1559,11 @@ int SysExistDir(const char *pszDirPath)
 	return (dwAttr & FILE_ATTRIBUTE_DIRECTORY) ? 1: 0;
 }
 
-SYS_HANDLE SysFirstFile(const char *pszPath, char *pszFileName, int iSize)
+SYS_HANDLE SysFirstFile(char const *pszPath, char *pszFileName, int iSize)
 {
-	char szMatch[SYS_MAX_PATH] = "";
+	char szMatch[SYS_MAX_PATH];
 
-	strcpy(szMatch, pszPath);
+	StrNCpy(szMatch, pszPath, sizeof(szMatch) - 2);
 	AppendSlash(szMatch);
 	strcat(szMatch, "*");
 
@@ -1551,7 +1583,7 @@ SYS_HANDLE SysFirstFile(const char *pszPath, char *pszFileName, int iSize)
 		return SYS_INVALID_HANDLE;
 	}
 
-	strcpy(pFFD->szFindPath, pszPath);
+	StrNCpy(pFFD->szFindPath, pszPath, sizeof(pFFD->szFindPath) - 1);
 	AppendSlash(pFFD->szFindPath);
 
 	pFFD->hFind = hFind;
@@ -1591,8 +1623,10 @@ void SysFindClose(SYS_HANDLE hFind)
 {
 	FileFindData *pFFD = (FileFindData *) hFind;
 
-	FindClose(pFFD->hFind);
-	SysFree(pFFD);
+	if (pFFD != NULL) {
+		FindClose(pFFD->hFind);
+		SysFree(pFFD);
+	}
 }
 
 int SysGetFileInfo(char const *pszFileName, SYS_FILE_INFO &FI)
@@ -1639,7 +1673,7 @@ int SysSetFileModTime(char const *pszFileName, time_t tMod)
 	return 0;
 }
 
-char *SysStrDup(const char *pszString)
+char *SysStrDup(char const *pszString)
 {
 	int iStrLength = strlen(pszString);
 	char *pszBuffer = (char *) SysAllocNZ(iStrLength + 1);
@@ -1650,20 +1684,20 @@ char *SysStrDup(const char *pszString)
 	return pszBuffer;
 }
 
-char *SysGetEnv(const char *pszVarName)
+char *SysGetEnv(char const *pszVarName)
 {
 	HKEY hKey;
-	char szRKeyPath[256] = "";
+	char szRKeyPath[256];
 
 	SysSNPrintf(szRKeyPath, sizeof(szRKeyPath) - 1, "SOFTWARE\\%s\\%s",
 		    APP_PRODUCER, szServerName);
 	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, szRKeyPath, 0, KEY_QUERY_VALUE,
 			 &hKey) == ERROR_SUCCESS) {
 		char szKeyValue[2048] = "";
-		DWORD dwSize = sizeof(szKeyValue);
-		DWORD dwKeyType;
+		DWORD dwKeyType, dwSize = sizeof(szKeyValue);
 
-		if (RegQueryValueEx(hKey, pszVarName, NULL, &dwKeyType, (u_char *) szKeyValue,
+		if (RegQueryValueEx(hKey, pszVarName, NULL, &dwKeyType,
+				    (u_char *) szKeyValue,
 				    &dwSize) == ERROR_SUCCESS) {
 			RegCloseKey(hKey);
 
@@ -1672,26 +1706,20 @@ char *SysGetEnv(const char *pszVarName)
 		RegCloseKey(hKey);
 	}
 
-	const char *pszValue = getenv(pszVarName);
+	char const *pszValue = getenv(pszVarName);
 
 	return (pszValue != NULL) ? SysStrDup(pszValue): NULL;
 }
 
-char *SysGetTmpFile(char *pszFileName)
+char *SysGetTempDir(char *pszPath, int iMaxPath)
 {
-	SYS_LONGLONG llFileID;
-	char szTmpPath[SYS_MAX_PATH] = "";
-	static LONG lFileSeqNr = 0;
+	GetTempPath(iMaxPath - 2, pszPath);
+	AppendSlash(pszPath);
 
-	GetTempPath(sizeof(szTmpPath) - 1, szTmpPath);
-	llFileID = (((SYS_LONGLONG) GetCurrentThreadId()) << 32) |
-		(SYS_LONGLONG) InterlockedIncrement(&lFileSeqNr);
-	SysSNPrintf(pszFileName, SYS_MAX_PATH - 1, "%smsrv%I64x.tmp", szTmpPath, llFileID);
-
-	return pszFileName;
+	return pszPath;
 }
 
-int SysRemove(const char *pszFileName)
+int SysRemove(char const *pszFileName)
 {
 	if (!DeleteFile(pszFileName)) {
 		ErrSetErrorCode(ERR_FILE_DELETE);
@@ -1701,7 +1729,7 @@ int SysRemove(const char *pszFileName)
 	return 0;
 }
 
-int SysMakeDir(const char *pszPath)
+int SysMakeDir(char const *pszPath)
 {
 	if (!CreateDirectory(pszPath, NULL)) {
 		ErrSetErrorCode(ERR_DIR_CREATE);
@@ -1711,7 +1739,7 @@ int SysMakeDir(const char *pszPath)
 	return 0;
 }
 
-int SysRemoveDir(const char *pszPath)
+int SysRemoveDir(char const *pszPath)
 {
 	if (!RemoveDirectory(pszPath)) {
 		ErrSetErrorCode(ERR_DIR_DELETE);
@@ -1789,7 +1817,7 @@ long SysGetDayLight(void)
 	time_t tCurr = time(NULL);
 	struct tm tmCurr = *localtime(&tCurr);
 
-	return (long) ((tmCurr.tm_isdst <= 0) ? 0: 3600);
+	return (tmCurr.tm_isdst <= 0) ? 0: 3600;
 }
 
 int SysGetDiskSpace(char const *pszPath, SYS_INT64 *pTotal, SYS_INT64 *pFree)
@@ -1814,7 +1842,6 @@ int SysMemoryInfo(SYS_INT64 *pRamTotal, SYS_INT64 *pRamFree,
 		  SYS_INT64 *pVirtTotal, SYS_INT64 *pVirtFree)
 {
 #if _WIN32_WINNT >= 0x0500
-
 	MEMORYSTATUSEX MSEX;
 
 	ZeroData(MSEX);
@@ -1822,14 +1849,11 @@ int SysMemoryInfo(SYS_INT64 *pRamTotal, SYS_INT64 *pRamFree,
 		ErrSetErrorCode(ERR_GET_MEMORY_INFO);
 		return ERR_GET_MEMORY_INFO;
 	}
-
 	*pRamTotal = (SYS_INT64) MSEX.ullTotalPhys;
 	*pRamFree = (SYS_INT64) MSEX.ullAvailPhys;
 	*pVirtTotal = (SYS_INT64) MSEX.ullTotalVirtual;
 	*pVirtFree = (SYS_INT64) MSEX.ullAvailVirtual;
-
 #else
-
 	MEMORYSTATUS MS;
 
 	ZeroData(MS);
@@ -1838,7 +1862,6 @@ int SysMemoryInfo(SYS_INT64 *pRamTotal, SYS_INT64 *pRamFree,
 	*pRamFree = (SYS_INT64) MS.dwAvailPhys;
 	*pVirtTotal = (SYS_INT64) MS.dwTotalVirtual;
 	*pVirtFree = (SYS_INT64) MS.dwAvailVirtual;
-
 #endif
 
 	return 0;
@@ -1856,8 +1879,7 @@ SYS_MMAP SysCreateMMap(char const *pszFileName, unsigned long ulFlags)
 		return SYS_INVALID_MMAP;
 	}
 
-	DWORD dwFileSizeHi = 0;
-	DWORD dwFileSizeLo = GetFileSize(hFile, &dwFileSizeHi);
+	DWORD dwFileSizeHi = 0, dwFileSizeLo = GetFileSize(hFile, &dwFileSizeHi);
 	HANDLE hFileMap = CreateFileMapping(hFile, NULL, (ulFlags & SYS_MMAP_WRITE) ?
 					    PAGE_READWRITE: PAGE_READONLY,
 					    dwFileSizeHi, dwFileSizeLo, NULL);
@@ -1892,12 +1914,14 @@ void SysCloseMMap(SYS_MMAP hMap)
 {
 	MMapData *pMMD = (MMapData *) hMap;
 
-	if (pMMD->iNumMaps > 0) {
+	if (pMMD != NULL) {
+		if (pMMD->iNumMaps > 0) {
 
+		}
+		CloseHandle(pMMD->hFileMap);
+		CloseHandle(pMMD->hFile);
+		SysFree(pMMD);
 	}
-	CloseHandle(pMMD->hFileMap);
-	CloseHandle(pMMD->hFile);
-	SysFree(pMMD);
 }
 
 SYS_OFF_T SysMMapSize(SYS_MMAP hMap)
